#!/bin/bash

if [[ ! -w /etc/passwd ]] ; then
  echo "Only 'root' can execute this script."
  exit 1
fi

#------------------------------------------------------------------------------
# define variables and functions
#------------------------------------------------------------------------------

EPAGES_CONFIG_ENV=$(readlink -m $EPAGES_CONFIG 2>/dev/null)
. /etc/default/epages6
. /etc/sysconfig/epages6
export set_mysql_root_password run_sql get_mysql_connection find_ini_config mktempdir

EPAGES_CONFIG=${EPAGES_CONFIG:-/srv/epages/eproot/Shared/Config}
EPAGES_CONFIG_ENV=${EPAGES_CONFIG_ENV:-$EPAGES_CONFIG}
DATABASE_D=$EPAGES_CONFIG_ENV/Database.d

# EPAGES_INSTALL_MY* come from /etc/sysconfig/epages6 but
# INSTALL_EPAGES_MY* from install-epages.conf have higher priority

MY_CNF=${INSTALL_EPAGES_MY_CNF-$EPAGES_INSTALL_MY_CNF}
export EPAGES_INSTALL_MY_CNF=$MY_CNF
MY_CNF=${MY_CNF:-$(my_cnf)}

MYSQL_PASS=${INSTALL_EPAGES_MYSQL_PASS-$EPAGES_INSTALL_MYSQL_PASS}
export EPAGES_INSTALL_MYSQL_PASS=$MYSQL_PASS

# unbind address
if [[ -f $MY_CNF ]] ; then
  CNFFILE=$(mktempdir epages-mysql)/my.cnf.~$$~
  sed -r 's,^\s*(bind-address),#\1,' $MY_CNF > $CNFFILE
  if ! cmp -s $MY_CNF $CNFFILE ; then
    cat $CNFFILE > $MY_CNF
    service $(mysqld_name) restart
    sleep 2
  fi
fi

#------------------------------------------------------------------------------
# change database
#------------------------------------------------------------------------------

# set_mysql_root_password is executed in either:
# - postinstall of package (here):
#   /var/epages/bin/SetupMySQL.sh -> EPAGES_MYSQL_ROOT_SCRIPT=SetupMySQL.sh
# - after package installation:
#   /var/epages/postinstall/mysqld.sh -> EPAGES_MYSQL_ROOT_SCRIPT=postinstall/mysqld.sh
EPAGES_MYSQL_ROOT_SCRIPT=${EPAGES_MYSQL_ROOT_SCRIPT:-SetupMySQL.sh}

if [[ $EPAGES_MYSQL_ROOT_SCRIPT = SetupMySQL.sh ]] ; then
  OUTFILE=$(mktempdir epages-mysql)/set_mysql_root_password-setup.~$$~
  LOGFILE=$(mktempdir epages-mysql)/SetupMySQL.sh.~$$~.log

  # set_mysql_root_password
  echo "EPAGES_CONFIG=$EPAGES_CONFIG_ENV set_mysql_root_password --setup '$MYSQL_PASS'" >> $LOGFILE
  EPAGES_CONFIG=$EPAGES_CONFIG_ENV set_mysql_root_password --setup "$MYSQL_PASS" > $OUTFILE
  MYSQL_PASS=$(grep '^MYSQL_PSWD=' $OUTFILE | tail -1)
  MYSQL_PASS=${MYSQL_PASS#*=}

  # delete empty users
  STATEMENT="DELETE FROM mysql.user WHERE user = ''; FLUSH PRIVILEGES;"
  echo "run_sql -p '$MYSQL_PASS' '$STATEMENT'" >> $LOGFILE
  run_sql -p "$MYSQL_PASS" "$STATEMENT" || exit 1

  # grant all privileges on *.* to root
  STATEMENT="GRANT ALL ON *.* TO 'root' %s WITH GRANT OPTION;\n"
  [[ -z $MYSQL_PASS ]] && IDENTIFIED_BY= || IDENTIFIED_BY="IDENTIFIED BY '$MYSQL_PASS'"
  echo "run_sql -p '$MYSQL_PASS' '$(printf "$STATEMENT" "$IDENTIFIED_BY")'" >> $LOGFILE
  run_sql -p "$MYSQL_PASS" "$(printf "$STATEMENT" "$IDENTIFIED_BY")" || exit 1

  # database engine must be InnoDB
  STATEMENT="SHOW ENGINES;"
  echo "run_sql -p '$MYSQL_PASS' '$STATEMENT'" >> $LOGFILE
  if ! run_sql -p "$MYSQL_PASS" "$STATEMENT" | grep -Pq '^InnoDB\s+(YES|DEFAULT)\s+' ; then
    echo "ERROR: SHOW ENGINES does not show InnoDB."
    exit 1
  fi

fi

#------------------------------------------------------------------------------
# define variables
#------------------------------------------------------------------------------

# EPAGES_INSTALL_MY* come from /etc/sysconfig/epages6 but
# INSTALL_EPAGES_MY* from install-epages.conf have higher priority

MYSQL_CONN=${INSTALL_EPAGES_MYSQL_CONNECTION:-$EPAGES_INSTALL_MYSQL_CONNECTION}
export EPAGES_INSTALL_MYSQL_CONNECTION=$MYSQL_CONN

MYSQL_HOST=${INSTALL_EPAGES_MYSQL_HOST:-$EPAGES_INSTALL_MYSQL_HOST}
export EPAGES_INSTALL_MYSQL_HOST=$MYSQL_HOST

CONN=($(get_mysql_connection))
MYSQL_HOST=${CONN[0]}
MYSQL_CONN=${MYSQL_CONN:-$MYSQL_HOST}
INI_CONFIG=$(find_ini_config)

#------------------------------------------------------------------------------
# change configuration files
#------------------------------------------------------------------------------

# set EPAGES_MYSQL_CONNECTION in /etc/default/epages6
if [[ -n $MYSQL_CONN && -z $EPAGES_INSTALLATION_MOCK ]] ; then
  $INI_CONFIG -f /etc/default/epages6 -k EPAGES_MYSQL_CONNECTION -v "\"$MYSQL_CONN\"" -i
  $INI_CONFIG -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_MYSQL_CONNECTION -v "\"$MYSQL_CONN\"" -i
fi
if [[ -n $MYSQL_HOST && -z $EPAGES_INSTALLATION_MOCK ]] ; then
  $INI_CONFIG -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_MYSQL_HOST -v "\"$MYSQL_HOST\"" -i
fi

if [[ -n ${INSTALL_EPAGES_ASPOOLDB-undef} ]] ; then
  # copy ASPoolDB.conf.sample.initial to ASPoolDB.conf.initial
  DB_CONF=$DATABASE_D/ASPoolDB.conf
  if [[ -f $DB_CONF.sample.initial ]] ; then
    cp -p $DB_CONF.sample.initial $DB_CONF.initial
  fi
  # and create ASPoolDB.conf from ASPoolDB.conf.initial (unless exists)
  [[ -f $DB_CONF || ! -f  $DB_CONF.initial ]] || cp -p $DB_CONF.initial $DB_CONF
  [[ -f $DB_CONF ]] || touch $DB_CONF

  # write ASPoolDB.conf unless it contains datasource=dbi:mysql:host=
  if ! grep -q 'datasource=dbi:mysql:host=' $DB_CONF ; then
    echo "$INI_CONFIG -f $DB_CONF -s Connection -k datasource -v 'dbi:mysql:host=$MYSQL_CONN' -i" >> $LOGFILE
    $INI_CONFIG -f $DB_CONF -s Connection -k datasource -v "dbi:mysql:host=$MYSQL_CONN" -i
    MY_CNF=${INSTALL_EPAGES_MY_CNF-$EPAGES_INSTALL_MY_CNF}
    if [[ -n $MY_CNF && $MY_CNF != $(my_cnf) ]] ; then
      echo "$INI_CONFIG -f $DB_CONF -s Connection -k MySQLConfigFile -v '$MY_CNF' -i" >> $LOGFILE
      $INI_CONFIG -f $DB_CONF -s Connection -k MySQLConfigFile -v "$MY_CNF" -i
    fi
    chown ep_appl:epages $DB_CONF
  fi
fi

/etc/init.d/epages6 perm admin >/dev/null 2>&1

exit 0
