#!/srv/epages/eproot/Perl/bin/perl

use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Cartridge;
use DE_EPAGES::Core::API::File qw( CopyDirectory );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Installer::API::LanguagePack qw(
    DownloadLanguagePack
    UntarLanguagePack
    DictionaryXmlLocation
    LanguagePackLocation
    GetLanguagesByVersion
);

sub usage {
    my ($ErrorMessage) = @_;
    print "Error:\n  $ErrorMessage\n\n" if $ErrorMessage;
    print <<END_USAGE;
Description:
  For given languages, downloads and unpacks language packs into
  EPAGES_PATCHES/LanguagePacks, then copies files to EPAGES_CARTRIDGES.

Usage:
  perl $0 [options] [flags]

Options:
  -lang         language code list            (use 'any' for any language)
  -version      install language files for that version (default: current)
  -unpack-only  do not copy files to EPAGES_CARTRIDGES.

Flags:
  -help      show the command line options
  -nooutput  do not print any output

Examples:
  perl $0 -version 6.17.36.1 -lang 'ca es'
  perl $0 -lang any

END_USAGE
    exit 2;
}

sub Main {

    local $| = 1;

    my %Options = ();
    my @GetOpt = qw(help nooutput unpack-only lang=s version=s);

    usage() unless GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};
    usage('Missing parameter -lang') unless $Options{lang};

    $Options{version} //= DE_EPAGES::Core::API::Cartridge->new()->version();

    # up to now we have language packs only for versions with 3 parts like x.y.z
    my $VersionParts = scalar(split(/\./,$Options{version}));
    if ($VersionParts > 3) {
        $Options{version} =~ s,^(\d+\.\d+\.\d+).*,$1,
    } elsif ($VersionParts < 3) {
        usage("Void format of -version $Options{version} (must be x.y.z[.more])");
    }
    usage("Void format of -version $Options{version} (only numbers and dots allowed)")
        unless $Options{version} =~ /^(\d+\.\d+\.\d+)$/;

    my $time1 = time;

    my $Languages = GetLanguagesByVersion($Options{version}, $Options{lang});

    if (not scalar @$Languages) {
        print "No languages for version $Options{version} found.\n";
        exit 1;
    }

    # unpack language packs into $ENV{EPAGES_PATCHES}/LanguagePacks
    # then copy files to $ENV{EPAGES_CARTRIDGES}
    InstallLanguageFiles($_, \%Options) foreach @$Languages;

    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1 unless $Options{nooutput};
}

#========================================================================================
# function     InstallLanguageFiles
# state        private
#----------------------------------------------------------------------------------------
# syntax       InstallLanguageFiles($Language, $Options)
# example      InstallLanguageFiles('es', { version => '6.17.33' });
#----------------------------------------------------------------------------------------
# description  unpacks language pack into $ENV{EPAGES_PATCHES}/LanguagePacks
#               then copy files to $ENV{EPAGES_CARTRIDGES}
#----------------------------------------------------------------------------------------
# input        $Language | language code | string
# input        $Options | options
#                   <ul>
#                       <li>version     - ePages version
#                       <li>nooutput    - run script quietly
#                       <li>unpack-only - just unpack, do not copy to $ENV{EPAGES_CARTRIDGES}
#                   </ul>
#               | ref.hash
#========================================================================================
sub InstallLanguageFiles {
    my ($Lang, $Options) = @_;
    my $Version = $Options->{version};
    my $Quiet = $Options->{nooutput};

    my $LangPack = LanguagePackLocation($Lang, $Version, {IfExists => 1});
    if ($LangPack) {
        print "[$Version, $Lang] language pack found: $LangPack\n" unless $Quiet;
    } else {
        print "[$Version, $Lang] download language pack: " unless $Quiet;
        $LangPack = DownloadLanguagePack($Lang, $Version);
        print "$LangPack\n" unless $Quiet;
    }

    my $LangPackDir = $LangPack;
    $LangPackDir =~ s,\.tar\.gz$,,;

    my $DictionaryXml = DictionaryXmlLocation(
        $Lang, {IfExists => 1, Dir => "$LangPackDir/Cartridges"});
    if ($DictionaryXml) {
        print "[$Version, $Lang] unpacked language pack found: $LangPackDir\n" unless $Quiet;
    } else {
        print "[$Version, $Lang] unpack language pack: " unless $Quiet;
        $LangPackDir = UntarLanguagePack($Lang, $Version);
        print "$LangPackDir\n" unless $Quiet;
    }

    return if $Options->{'unpack-only'};

    print "[$Version, $Lang] CopyDirectory('$LangPackDir/Cartridges', $ENV{EPAGES_CARTRIDGES})\n" unless $Quiet;
    CopyDirectory("$LangPackDir/Cartridges", $ENV{EPAGES_CARTRIDGES}, undef,
                  {unlink => 1, stat => 'epages'});
}

RunScript(
    'Sub' => \&Main
);

1;
