#!/bin/bash

#=======================================================================
# update_cartridges_xml.sh
#=======================================================================

if [ ! -w /etc/passwd ] ; then
  echo "$0: only 'root' can execute this program"
  exit 1
fi

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  [[ -n $EPAGES_INSTALLATION_MOCK ]] && echo "$@" || echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

#-----------------------------------------------------------------------
# define variables and functions
#-----------------------------------------------------------------------

BN=${0##*/}

usage() {
    [[ -n $1 ]] && echo -e "Error:\n  $1\n"
  cat << USAGE_END
Usage: $BN -s STORETYPE [-b BASE_STORETYPE] [-a ADDITIONAL_CARTRIDGES] [-x CARTRIDGES_XML]

Description:
  With BASE_STORETYPE, copies tag structure <DatabaseType Name=BASE_STORETYPE>
  to <DatabaseType Name=STORETYPE> in Cartridges.xml.
  With ADDITIONAL_CARTRIDGES, adds additional cartidges to STORETYPE.
  With CARTRIDGES_XML, uses this Cartridges.xml instead of
  $EPAGES_CARTRIDGES/Cartridges.xml

Example:
  $BN -s Store -a 'XX::YY XX::ZZ'
  $BN -s StoreXY -b Store
  $BN -s StoreXY -b Store -a 'XX::YY XX::ZZ'
USAGE_END
  exit 1
}

. /etc/default/epages6

unset STORETYPE BASE_STORETYPE ADDITIONAL_CARTRIDGES
CARTRIDGES_XML=$EPAGES_CARTRIDGES/Cartridges.xml

while getopts s:b:x:a: option ; do
  case $option in
    s) STORETYPE=$OPTARG ;;
    b) BASE_STORETYPE=$OPTARG ;;
    a) ADDITIONAL_CARTRIDGES=$OPTARG ;;
    x) CARTRIDGES_XML=$OPTARG ;;
    *) usage ;;
  esac
done

[[ -n $STORETYPE ]] || usage "Option -s missing"
[[ -n "$BASE_STORETYPE$ADDITIONAL_CARTRIDGES" ]] || usage "Options -a and/or -b missing"
BASE_STORETYPE=${BASE_STORETYPE:-$STORETYPE}

ls $CARTRIDGES_XML >/dev/null
CARTRIDGES_XML=$(readlink -f $CARTRIDGES_XML)
TEMP_DIR=$(dirname $CARTRIDGES_XML)/${BN%.sh}.d

if ! grep -Pq "<DatabaseType\b.*\bName=\W$BASE_STORETYPE\W.*>" $CARTRIDGES_XML
then
  echo "DatabaseType Name='$BASE_STORETYPE': not found in: $CARTRIDGES_XML"
  exit 1
fi

set -e

#-----------------------------------------------------------------------
# split Cartridges.xml into <DatabaseType> parts (update_cartridges_xml.d/#.)
#-----------------------------------------------------------------------

rm -rf $TEMP_DIR
mkdir -m 755 -p $TEMP_DIR
CONTENT=
PART=0
ECHO $LINENO "split $CARTRIDGES_XML > ${BN%.sh}.d/*"
while read line ; do
  if [[ $line != ${line#*</DatabaseType} ]] ; then
    # new <DatabaseType> line found
    echo "$CONTENT" > $TEMP_DIR/$PART.
    PART=$(($PART + 1))
    CONTENT=$line
    continue
  fi
  CONTENT="$CONTENT"$'\n'"$line"
done < $CARTRIDGES_XML

echo "$CONTENT" > $TEMP_DIR/$PART.
CONTENT=$line

#-----------------------------------------------------------------------
# copy <DatabaseType Name=BASE_STORETYPE> to <DatabaseType Name=STORETYPE>
#-----------------------------------------------------------------------

if [[ $BASE_STORETYPE != $STORETYPE ]] ; then
  for part in $(grep -Pl "<DatabaseType\b.*\bName=\W$STORETYPE\W.*>" $TEMP_DIR/*. 2>/dev/null) ; do
    rm -f $part
  done
  for part in $(grep -Pl "<DatabaseType\b.*\bName=\W$BASE_STORETYPE\W.*>" $TEMP_DIR/*.) ; do
    ECHO $LINENO "sed -r 's/Name=\\S+/Name=\"$STORETYPE\"/' ${part##*/} > ${part##*/}1"
    sed -r "s/(\bName)=[^[:space:]>]+/\1=\"$STORETYPE\"/" $part > ${part}1
  done
fi

#-----------------------------------------------------------------------
# add cartridges to <DatabaseType> parts
#-----------------------------------------------------------------------

if [[ -n $ADDITIONAL_CARTRIDGES ]] ; then
  for part in $(grep -Pl "<DatabaseType\b.*\bName=\W$STORETYPE\W.*>" $TEMP_DIR/*) ; do
    for cartridge in $ADDITIONAL_CARTRIDGES ; do
      ECHO $LINENO "echo '<Cartridge Package=\"$cartridge\" />' >> ${part##*/}"
      echo "<Cartridge Package=\"$cartridge\" />" >> $part
    done
  done
fi

#-----------------------------------------------------------------------
# concatenate <DatabaseType> parts to Cartridges.xml
#-----------------------------------------------------------------------

ECHO $LINENO "cat \$(ls -v ${BN%.sh}.d/*) > $CARTRIDGES_XML"
cat $(ls -v $TEMP_DIR/*) | grep -Pv '^\s*$' > $CARTRIDGES_XML
grep -q '</epages>' $CARTRIDGES_XML || echo '</epages>' >> $CARTRIDGES_XML

#-----------------------------------------------------------------------
# cleanup
#-----------------------------------------------------------------------

rm -rf $TEMP_DIR

exit 0
