#!/bin/bash

#==============================================================================
# epages-fs.sh
#==============================================================================

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!" 1>&2
  exit 1
fi

BN=${0##*/}
PATH=/usr/sbin:/sbin:$PATH

LANG=C # use base language for standard output of some commands
LC_ALL=C

SHARE_OPTIONS='async,rw,no_subtree_check,no_auth_nlm,no_root_squash'
MOUNT_OPTIONS='async,exec,noatime,nodev,nolock,bg,hard,intr,rw,rsize=16384,wsize=16384'
MOUNTPOINTS='etc/sysconfig eproot/Shared'

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/functions/fun-arch.sh
export EPAGES_KERNEL

. $(dirname $0)/functions/fun-linux.sh
export chkconfig_do

. $(dirname $0)/functions/fun-repoedit.sh
export get_arch_repo update_repo_version update_repo_url
export update_repo_arch use_new_repos delete_obsolete_repos

#------------------------------------------------------------------------------
# arguments
#------------------------------------------------------------------------------

usage() {
  ( echo "Usage:"
    echo "  $BN {--share | --mount NFSHOST | --unmount | --unshare} [--all]"
    echo "Options:"
    echo "  --share: share /srv/epages with other hosts"
    echo "  --mount: mount etc/sysconfig eproot/Shared from NFSHOST:/srv/epages"
    echo "  --unmount: unmount /srv/epages"
    echo "  --unshare: unshare /srv/epages"
    echo "  --all: mount eproot/Perl eproot/Cartridges from NFSHOST:/srv/epages as well" ) 1>&2
  exit 1
}

[[ $# -eq 0 ]] && usage

# legacy requires -share for --share etc. as well
unset TASK NFSHOST
while [[ $# -gt 0 ]] ; do
  case $1 in
    -share|--share|-unshare|--unshare|-unmount|--unmount) TASK=${1//-/} ; shift ;;
    -mount|--mount) TASK=${1//-/} ; [[ -z $2 ]] && usage ; NFSHOST=$2 ; shift 2 ;;
    -all|--all) MOUNTPOINTS='etc/sysconfig eproot/Shared eproot/Perl eproot/Cartridges' ; shift ;;
    *) usage ;;
  esac
done

case $EPAGES_KERNEL in
  *r) NFSSERVICE=nfs ;; # redhat
  *d) NFSSERVICE=nfs-kernel-server ;; # debian
  *) echo "$BN: operating system not supported: $(uname -a)" ; exit 1 ;;
esac

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

# exit if neither NFS client nor NFS server is installed
EXPORTFS_BIN=$(type -p exportfs)
ECHO $LINENO "exportfs is: ${EXPORTFS_BIN:-(not found)}"
[[ -z $EXPORTFS_BIN ]] && exit 0

#------------------------------------------------------------------------------
# TASK = unmount
#------------------------------------------------------------------------------

if [[ $TASK = unmount ]] ; then
  ECHO $LINENO /etc/init.d/epages6 stop
  /etc/init.d/epages6 stop >/dev/null
  mount | while read share on mount has type options ; do
    [[ $type = nfs && ${mount#/srv/epages} != $mount ]] || continue
    ECHO $LINENO "umount -f $mount"
    umount -f $mount
  done
  ECHO $LINENO "grep -v ' /srv/epages' /etc/fstab"
  sed -i '/ \/srv\/epages/d' /etc/fstab
  ECHO $LINENO "rm -f /etc/sysconfig/epages6"
  rm -f /etc/sysconfig/epages6
  exit 0
fi

#------------------------------------------------------------------------------
# TASK = unshare
#------------------------------------------------------------------------------

if [[ $TASK = unshare ]] ; then
  ECHO $LINENO "grep -v '/srv/epages' /etc/exports"
  sed -i '/\/srv\/epages/d' /etc/exports
  ECHO $LINENO "exportfs -r"
  exportfs -r
  showmount -e
  exit 0
fi

#------------------------------------------------------------------------------
# TASK = share
#------------------------------------------------------------------------------

for i in $MOUNTPOINTS ; do
  mkdir -p -m 755 /srv/epages/$i || exit 1
done

if [[ $TASK = share ]] ; then
  # start NFS server at boot time
  if ! chkconfig_do --list $NFSSERVICE | grep -q '3:on' ; then
    chkconfig_do --del $NFSSERVICE >/dev/null
    sed -ri 's,(chkconfig:) -,\1 35,' /etc/init.d/$NFSSERVICE 2>/dev/null
    ECHO $LINENO "chkconfig_do --add $NFSSERVICE"
    chkconfig_do --add $NFSSERVICE
  fi

  # exit if /etc/sysconfig/epages6 not found
  ls /etc/sysconfig/epages6 >/dev/null || exit 1

  # link /srv/epages/etc/sysconfig/epages6 to /etc/sysconfig/epages6
  if [[ ! -f /srv/epages/etc/sysconfig/epages6 ]] ; then
    mv -f $(readlink -f /etc/sysconfig/epages6) /srv/epages/etc/sysconfig/epages6
    ECHO $LINENO "cp -s /srv/epages/etc/sysconfig/epages6 /etc/sysconfig/epages6"
    cp -s --remove-destination /srv/epages/etc/sysconfig/epages6 /etc/sysconfig/epages6
  fi

  # set EPAGES_DISTRIBUTED_INSTALLATION=1
  if ! grep -q EPAGES_DISTRIBUTED_INSTALLATION=1 /etc/sysconfig/epages6 ; then
    ECHO $LINENO "echo EPAGES_DISTRIBUTED_INSTALLATION=1 >> /etc/sysconfig/epages6"
    echo EPAGES_DISTRIBUTED_INSTALLATION=1 >> /etc/sysconfig/epages6
  fi

  # check if directories to share are mounted itself
  if mount | grep " on /srv/epages/" || grep "^[^#]* /srv/epages/" /etc/fstab ; then
    echo "$BN: Can't share mounted directories, exiting."
    exit 1
  fi

  # share /srv/epages
  if ! grep -q "^/srv/epages " /etc/exports ; then
    grep -q fsid=0 /etc/exports && FSID= || FSID=',fsid=0'
    ECHO $LINENO "echo '/srv/epages *($SHARE_OPTIONS$FSID)' >> /etc/exports"
    echo "/srv/epages *($SHARE_OPTIONS$FSID)" >> /etc/exports
  fi

  # restart NFS server
  ECHO $LINENO "exportfs -r"
  exportfs -r || exit 1
  ECHO $LINENO service $NFSSERVICE restart
  service $NFSSERVICE restart
  showmount -e
fi

#------------------------------------------------------------------------------
# TASK = mount
#------------------------------------------------------------------------------

if [[ $TASK = mount ]] ; then

  # check if directories to mount exist in local filesystem
  if exportfs 2>/dev/null | grep "^/srv/epages " \
     || grep "^[[:space:]]*/srv/epages" /etc/exports 2>/dev/null ; then
    echo "$BN: Can't mount own directories, exiting."
    exit 1
  fi

  # check if DNS entry for $NFSHOST exist
  host $NFSHOST || exit 1

  # mount /srv/epages
  for i in $MOUNTPOINTS ; do
    if ! grep -q "^[^#]* /srv/epages/$i " /etc/fstab ; then
      ECHO $LINENO "echo '$NFSHOST:/srv/epages/$i /srv/epages/$i nfs $MOUNT_OPTIONS 0 0' >> /etc/fstab"
      echo "$NFSHOST:/srv/epages/$i /srv/epages/$i nfs $MOUNT_OPTIONS 0 0" >> /etc/fstab
    fi
    if ! mount | grep -q " on /srv/epages/$i " ; then
      ECHO $LINENO "mount /srv/epages/$i"
      mount /srv/epages/$i || exit 1
    fi
  done

  # exit if /srv/epages/etc/sysconfig/epages6 not found
  ls /srv/epages/etc/sysconfig/epages6 >/dev/null || exit 1

  # set EPAGES_DISTRIBUTED_INSTALLATION=1
  if ! grep -q EPAGES_DISTRIBUTED_INSTALLATION=1 /srv/epages/etc/sysconfig/epages6 ; then
    ECHO $LINENO "echo EPAGES_DISTRIBUTED_INSTALLATION=1 >> /etc/sysconfig/epages6"
    echo EPAGES_DISTRIBUTED_INSTALLATION=1 >> /srv/epages/etc/sysconfig/epages6
  fi

  # exit if /etc/sysconfig/epages6 is a symlink to /srv/epages/etc/sysconfig/epages6
  [[ $(readlink -f /etc/sysconfig/epages6) = /srv/epages/etc/sysconfig/epages6 ]] && exit 0

  # otherwise create this symlink
  ECHO $LINENO "cp -s /srv/epages/etc/sysconfig/epages6 /etc/sysconfig/epages6"
  cp -s --remove-destination /srv/epages/etc/sysconfig/epages6 /etc/sysconfig/epages6 || exit 1
fi

exit 0
