#!/bin/bash

#==============================================================================
# perl
#==============================================================================

fun_perl_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export backup_file

[[ -n $fun_pkg_sourced ]] || . $(dirname $BASH_SOURCE)/fun-pkg.sh
export pkg_installed mktempdir

[[ -n $fun_java_sourced ]] || . $(dirname $BASH_SOURCE)/fun-java.sh
export test_version

[[ -n $config_sourced ]] || . $(dirname $BASH_SOURCE)/config.sh
export PERL_BINARIES_CHANGED

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export backup_obsolete_perl set_perl_tree what_needs_lib new_perl_binaries

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# new_perl_binaries
#..............................................................................
new_perl_binaries() {
  local USAGE='
syntax: new_perl_binaries [-f] UPDATE_VERSION ...
description: checks if perl binaries have changed in UPDATE_VERSION.
options:
  -f : check if /tmp/epages/epages-perl/new_binaries.UPDATE_VERSION exists
  <none> : check depends on installed version and upgrade path in
  PERL_BINARIES_CHANGED. if new binaries, write new_binaries.UPDATE_VERSION
  else delete new_binaries.UPDATE_VERSION
returns: true (if new perl binaries) or false (else)
example: new_perl_binaries 6.17.36
'
  local CHECK_FILE
  [[ $1 = -f ]] && { CHECK_FILE=1; shift; }
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local UPDATE_VERSION=$1
  local NEW_PERL_BINARIES=$(mktempdir epages-perl)/new_binaries.$UPDATE_VERSION

  if [[ -n $CHECK_FILE ]] ; then
    # new_binaries.$UPDATE_VERSION exists -> new perl binaries
    [[ -f $NEW_PERL_BINARIES ]] && return 0 || return 1
  fi

  rm -f $NEW_PERL_BINARIES
  local INSTALLED_VERSION=$(pkg_installed -V epages-perl | tail -1)
  if [[ -z $INSTALLED_VERSION ]] ; then
    # epages-perl not yet installed -> new perl binaries
    touch $NEW_PERL_BINARIES ; echo 0 ; return 0
  fi

  local VERSION
  for VERSION in $PERL_BINARIES_CHANGED ; do
    # go on if $VERSION is not a version string
    [[ -n ${VERSION//[0-9.-]/} ]] && continue
    # find $VERSION with: installed < version <= update
    if test_version $INSTALLED_VERSION -lt $VERSION && \
       test_version $VERSION -le $UPDATE_VERSION ; then
      touch $NEW_PERL_BINARIES ; echo $VERSION ; return 0
    fi
  done

  # no new perl binaries found
  return 1
}

#..............................................................................
# backup_obsolete_perl
#..............................................................................
backup_obsolete_perl() {
  local USAGE='
syntax: backup_obsolete_perl
description: find files that existed in the old RPM, but not in the new RPM.
  if found, those files will be packed into a .tgz file.
returns: name of file with list of found files (nothing if no file found)
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local EPAGES=${EPAGES:-"${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot}
  local EPAGES_PERL=${EPAGES_PERL:-"$EPAGES"/Perl}

  local INSTALLED_VERSION=$(pkg_installed -V epages-perl | tail -1)
  [[ -n $INSTALLED_VERSION ]] || return

  local PERL_PKG=epages-perl-$INSTALLED_VERSION
  local RPMLIST=$(mktempdir epages-perl)/rpmlist.$INSTALLED_VERSION

  case $EPAGES_KERNEL in
    *d) dpkg-query -L epages-perl ;; # debian
    *r) rpm -q $PERL_PKG --list ;; # redhat
  esac | sort > $RPMLIST

  BIN_EX='bin/((ep)?node|karma.sh|npm|optipng|jsl)'
  LINUX_EX='lib/linux/(CORE/config.h|Config.pm|Config_heavy.pl)'
  SITE_EX='lib/site_perl/ppm.xml'

  local INSTALLED=$(mktempdir epages-perl)/installed.$INSTALLED_VERSION
  find "$EPAGES_PERL"/{*,.??*}/ -type f 2>/dev/null \
  | egrep -v "/($BIN_EX|$LINUX_EX|$SITE_EX)" | sort > $INSTALLED

  [[ -s $INSTALLED ]] || return

  local NORPM=$(mktempdir epages-perl)/norpm.$INSTALLED_VERSION
  comm -13 $RPMLIST $INSTALLED > $NORPM 2>/dev/null
  [[ -s $NORPM ]] || return

  cat $NORPM | tar zcfT - - > "$EPAGES"/$PERL_PKG-norpm.tgz 2>/dev/null
  echo $NORPM
}

#..............................................................................
# what_needs_lib
#..............................................................................
what_needs_lib() {
  local USAGE='
syntax: what_needs_lib FILE LIB1 ...
description: FILE contains a file list (as e.g. from backup_obsolete_perl).
  for each file of that list show files that use at least one of the given LIBs.
returns: list of files which use one of the given LIBs
example: what_needs_lib /tmp/$PERL_PKG.norpm libssl.so.1.0.0 libcrypto.so.1.0.0
'
  [[ $# -lt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local FILELIST=$1 ; shift
  local LIBRARIES=$@

  [[ -s $FILELIST ]] || return

  while read path ; do
    [[ -f "$path" ]] || continue
    file -L "$path" | egrep -q 'ELF [0-9]+-bit' || continue
    objdump -p "$path" | grep NEEDED | egrep -q " (${LIBRARIES// /|})$" && echo "$path"
  done < $FILELIST
}

#..............................................................................
# set_perl_tree
#..............................................................................
set_perl_tree () {
  local USAGE='
syntax: set_perl_tree
description: sets path to epages-perl in perl files
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local EPAGES=${EPAGES:-"${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot}
  local EPAGES_PERL=${EPAGES_PERL:-"$EPAGES"/Perl}

  pushd "$EPAGES_PERL"/lib/linux >/dev/null || return 1

  for i in Config.pm Config_heavy.pl ../site_perl/ppm.xml CORE/config.h ; do
    backup_file $i.ept
    sed "s,=EPAGES_PERL=,$EPAGES_PERL,g" $i.ept > $i
  done

  if grep -q ' Perl/' perllocal.pod 2>/dev/null ; then
    cp -p perllocal.pod perllocal.pod.tmp
    sed "s, Perl/, $EPAGES_PERL/,g" perllocal.pod.tmp > perllocal.pod
    rm -f perllocal.pod.tmp
  fi

  popd >/dev/null
  pushd "$EPAGES_PERL"/lib >/dev/null

  for i in $(find . -name .packlist -exec grep \-l '^Perl/' {} \;) ; do
    cp -p $i $i.tmp
    sed "s,^Perl/,$EPAGES_PERL/,g" $i.tmp > $i
    rm -f $i.tmp
  done
  popd >/dev/null
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
