#!/bin/bash

#==============================================================================
# package manager
#==============================================================================

fun_pkg_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL EPAGES_ARCH

[[ -n $fun_java_sourced ]] || . $(dirname $BASH_SOURCE)/fun-java.sh
export test_version

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export show_rpm pkg_instance pkg_installed required_pkg_version
export mktempdir created_directories grep_server
export touch_pkg_result show_failed_dependency

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------


#..............................................................................
# show_rpm
#..............................................................................
show_rpm() {
  local USAGE='
syntax: show_rpm PACKAGE(S)
description: shows installed packages
example: for rpm in $(show_rpm mysql mysql-libs) ; do rpm -e --nodeps $rpm ; done
'
  [[ $# = 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  type rpm >/dev/null || return 1

  for pkg_name in $@ ; do
    rpm -q --quiet $pkg_name || continue
    for rpm in $(rpm -q $pkg_name) ; do
       echo $rpm
    done
  done
}

#..............................................................................
# pkg_instance
#..............................................................................
pkg_instance() {
  local USAGE='
syntax: pkg_instance NUMBER
NUMBER: 0 - remove last version of package
        1 - install the first time
        else - update (depending on the number of versions installed)
returns: first  - if NUMBER==1
         last   - if NUMBER==0
         update - else
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  [[ $1 = 1 ]] && { echo first; return; }
  [[ $1 = 0 ]] && { echo last; return; }
  echo update
}

#..............................................................................
# pkg_installed
#..............................................................................
pkg_installed() {
  local USAGE='
syntax: pkg_installed [-v|-V] [-and|-or] PKG1 PKG2...
arguments:
         -v: verbose output
         -V: outputs version number (works only with 1 PKG)
description: checks if epages-PKG is installed
returns: success (0) - if all PKG1 PKG2... are installed
         success (0) - if one of PKG1 PKG2... is installed and -or is set
         failure (1) - else
'
  local pkg version SHOW_VERSION
  local OR=0
  local QUIET=--quiet
  while [[ $1 =~ ^- ]] ; do
    case $1 in
     -v) QUIET= ; shift ;;
     -V) SHOW_VERSION=1 ; shift ;;
     -or) OR=1 ; shift ;;
     -and) shift ;;
     -*)  echo "$USAGE" 1>&2; return 1 ;;
     *) break ;;
    esac
  done

  [[ $# = 0 ]] && { echo "$USAGE" 1>&2; return 1; }
  local all=$#
  local found=0

  for pkg in $@ ; do
    pkg=${pkg#epages-}
    if [[ $pkg = pubkey && $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] ; then
      rpm -qa | grep -qi gpg-pubkey-b1d28320- || continue
    elif [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] ; then
      rpm $QUIET -q epages-$pkg || continue
      [[ -n $SHOW_VERSION ]] && version=$(rpm -q epages-$pkg --qf '[%{VERSION}-%{RELEASE}\n]' | sort -V | tail -1)
    elif [[ $pkg = pubkey ]] ; then
      apt-key list | grep -qi '/B1D28320 ' || continue
    elif [[ -n $QUIET ]] ; then
      dpkg --status epages-$pkg >/dev/null 2>&1 || continue
      [[ -n $SHOW_VERSION ]] && version=$(dpkg-query --show -f '${Version}\n' epages-$pkg)
    else
      dpkg --status epages-$pkg || continue
      [[ -n $SHOW_VERSION ]] && version=$(dpkg-query --show -f '${Version}\n' epages-$pkg)
    fi
    found=$(($found + 1))
  done
  [[ -n $SHOW_VERSION ]] && echo -n $version
  [[ $found = $all || ( $found -gt 0 && $OR = 1 ) ]] && return 0
  return 1
}

#..............................................................................
# required_pkg_version
#..............................................................................
required_pkg_version() {
  local USAGE='
syntax: required_pkg_version PACKAGE REQ_VERSION
description: checks if PACKAGE is installed with version >= REQ_VERSION
returns: success (0) - if PACKAGE is not installed
         success (0) - if PACKAGE is installed with version >= REQ_VERSION
         failure (1) - else
example: required_pkg_version epages-perl 6.17.36
'
  [[ $# = 2 ]] || { echo "$USAGE" 1>&2; return 1; }
  local PACKAGE=$1
  local REQ_VERSION=$2

  local INSTALLED_VERSION=$(pkg_installed -V $PACKAGE)
  [[ -n $INSTALLED_VERSION ]] || return 0

  test_version $INSTALLED_VERSION -ge $REQ_VERSION
  return $?
}

#..............................................................................
# created_directories
#..............................................................................
created_directories() {
  local USAGE='
syntax: created_directories [-m MODE] "DIRECTORY"
description: creates directory DIRECTORY (with MODE,
  default: 755) and returns the list of created directories
example: created_directories -m 1777 /tmp/epages/$PACKAGE
'
  [[ $# = 0 ]] && { echo "$USAGE" 1>&2; return 1; }
  local DIRECTORY
  local MODE=755
  case $1 in
    -m) [[ $# = 3 ]] || { echo "$USAGE" 1>&2; return 1; } ; MODE=$2 ; DIRECTORY=$3 ;;
    -*) echo "$USAGE" 1>&2; return 1 ;;
     *) [[ $# = 1 ]] || { echo "$USAGE" 1>&2; return 1; } ; DIRECTORY=$1 ;;
  esac

  LANG=C mkdir -m $MODE -pv "$DIRECTORY" | while read mkdir created directory path ; do
    [[ -z $path ]] && continue
    # unquote path
    path=${path#?} ; path=${path%?}
    echo "$path"
  done
  return ${PIPESTATUS[0]}
}

#..............................................................................
# mktempdir
#..............................................................................
mktempdir() {
  local USAGE='
syntax: mktempdir PACKAGE
description: creates directory /var/epages/PACKAGE and returns its name
example: TMPDIR=$(mktempdir epages-perl)
'
  [[ $# = 1 && $1 = ${1#-} ]] || { echo "$USAGE" 1>&2; return 1; }
  local PACKAGE=$1

  created_directories -m 1777 /tmp/epages/$PACKAGE | while read dir ; do
    chmod 1777 "$dir"
  done

  echo /tmp/epages/$PACKAGE
  return ${PIPESTATUS[0]}
}

#..............................................................................
# touch_pkg_result
#..............................................................................
touch_pkg_result() {
  local USAGE='
syntax: touch_pkg_result PACKAGE SCRIPTLET RESULT [MESSAGE]
description: writes MESSAGE into /tmp/epages/PACKAGE/SCRIPTLET.success (RESULT=0)
  or /tmp/epages/PACKAGE/SCRIPTLET.failure (if RESULT is set and not 0)
returns: echoes value of RESULT
example: touch_pkg_result epages-perl post 0 "success"
'
  [[ $# = 2 ]] && return
  local PACKAGE=$1
  local SCRIPTLET=$2
  local RESULT=$3
  shift 3
  local MESSAGE=$@
  local RETCODE=0

  local TMPDIR=$(mktempdir $PACKAGE)
  rm -f $TMPDIR/$SCRIPTLET.*

  [[ $RESULT = 0 ]] && FILE=$SCRIPTLET.success || FILE=$SCRIPTLET.failure
  echo "$MESSAGE" > $TMPDIR/$FILE || RETCODE=1

  echo $RESULT
  return $RETCODE
}

#..............................................................................
# show_failed_dependency
#..............................................................................
show_failed_dependency() {
  local USAGE='
syntax: show_failed_dependency PACKAGE REQUIRED_PKG FIX
description: shows that PACKAGE requires other REQUIRED_PKG and how to FIX
example: show_failed_dependency epages-patch-6.17.36 epages-perl-6.17.36 \
    "ENABLE_REPO=epages-setup /var/epages/install-RPM.sh epages-perl"
'
  [[ $# -lt 2 ]] && { echo "$USAGE" 1>&2; return 1; }

  local WHAT_PACKAGE=$1
  local PACKAGE=${1%-[0-9]*}
  local REQUIRED_PKG=${2%-[0-9]*}

  local REQUIRED_VERSION=${2##*-}
  # unset REQUIRED_VERSION if $2 does not end in -$REQUIRED_VERSION
  [[ $REQUIRED_PKG = $2 ]] && REQUIRED_VERSION=

  shift 2
  local FIX="$@"

  local INSTALLED_VERSION=$(pkg_installed -V $REQUIRED_PKG)

  if [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] ; then
    # if redhat
    local INSTALLED="           Installed: $REQUIRED_PKG-$INSTALLED_VERSION"'\n'
    [[ -z $INSTALLED_VERSION ]] && INSTALLED=
    [[ -n $REQUIRED_VERSION ]] && REQUIRED_VERSION=" >= $REQUIRED_VERSION"
    echo "
Error: Package: $WHAT_PACKAGE
           Requires: $REQUIRED_PKG$REQUIRED_VERSION"
    echo -e "$INSTALLED"
  else
    # if debian
    local REASON="$INSTALLED_VERSION is"
    [[ -z $INSTALLED_VERSION ]] && REASON="it is not going"
    [[ -n $REQUIRED_VERSION ]] && REQUIRED_VERSION=" (>= $REQUIRED_VERSION)"
    echo "
Some packages could not be installed. This may mean that you have
requested an impossible situation or if you are using the unstable
distribution that some required packages have not yet been created
or been moved out of Incoming.
The following information may help to resolve the situation:

The following packages have unmet dependencies:
 $PACKAGE : Depends: $REQUIRED_PKG$REQUIRED_VERSION but $REASON to be installed
"
  fi

  if [[ -n $FIX ]] ; then
    echo "Fix this problem by executing following command(s):"
    echo -e "$FIX" | sed 's,^, ,'
    echo
  fi

  return 0
}

#..............................................................................
# grep_server
#..............................................................................
grep_server() {
  local USAGE='
syntax: grep_server SERVER [["SERVER_LIST"] "GLOBAL_SETTING"]
description: checks GLOBAL_SETTING and if SERVER belongs to SERVER_LIST
returns: echoes 1 if SERVER belongs to SERVER_LIST or GLOBAL_SETTING is on, else nothing
'
  [[ $# = 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local SERVER=$1
  local SERVER_LIST=$2
  local GLOBAL_SETTING=$3

  # return if global setting found
  [[ -n $GLOBAL_SETTING && $GLOBAL_SETTING != 0 ]] && { echo 1 ; return; }

  for srv in $SERVER_LIST ; do
    case $srv in
      # return if found myself
      $SERVER) echo 1 ; return ;;
      # ignore other servers
      appsrv|ascsrv|reqsrv|mysqld|webconf|j) continue ;;
      # return if something else found
      *) echo 1 ; return ;;
    esac
  done
  return
}

#------------------------------------------------------------------------------
# end
#-----------------------------------------------------------------------------
