#!/bin/bash

#==============================================================================
# create configuration scripts (mainly in Shared/Config)
#==============================================================================

install_mysqld_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export find_ini_config get_ini_config set_ini_config

[[ -n $fun_mysql_sourced ]] || . $(dirname $BASH_SOURCE)/fun-mysql.sh
export update_database_conf_passwd

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export update_database_conf
export declare_DATABASE_CONF_HASH declare_DATABASE_CONF_DEFAULT

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

declare -A DATABASE_CONF_DEFAULT DATABASE_CONF_HASH
export DATABASE_CONF_DEFAULT DATABASE_CONF_HASH

#..............................................................................
# declare_DATABASE_CONF_HASH
#..............................................................................
declare_DATABASE_CONF_HASH() {
  local USAGE='
syntax: declare_DATABASE_CONF_HASH BU [CONFIG_FILE|KEY=VALUE PAIRS]
description: declares DATABASE_CONF_HASH with key-value pairs from
  section [BU] in CONFIG_FILE or from KEY=VALUE PAIRS
'
  [[ $1 != ${1#-} || $# -lt 1 ]] && { echo "$USAGE" 1>&2; return 1; }

  local BU=$1 ; shift
  local INI_CONFIG=$(find_ini_config)

  DATABASE_CONF_HASH=()

  for arg in $@ ; do
    key=${arg%%=*} ; value=${arg#*=}
    # ignore other keys than words
    [[ -f "$arg" ]] || echo $key | egrep -q '^\w+$' || continue
    # arg must be either file or key=value pair
    [[ -f "$arg" || $key != $arg ]] || continue
    # key=value pair found
    [[ -f "$arg" ]] || { DATABASE_CONF_HASH[$key]="$value"; continue; }
    # file with key=value pair found
    for keyvalue in $($INI_CONFIG -f $arg -s $BU -G) ; do
      key=${keyvalue%%=*} ; value=${keyvalue#*=}
      # ignore other keys than words
      echo $key | egrep -q '^\w+$' || continue
      # keyvalue must be key=value pair
      [[ $key = $keyvalue ]] || DATABASE_CONF_HASH[$key]="$value"
    done
    # set passwd= for root
    if [[ ${DATABASE_CONF_HASH[login]} = root && ${DATABASE_CONF_HASH[passwd]-undef} = undef ]]
    then
      if VAL=$(get_ini_value $arg mysqld-root-password 'general definitions') ; then
	DATABASE_CONF_HASH[passwd]="$VAL"
      fi
    fi
    # set MySQLConfigFile=
    if VAL=$(get_ini_value $arg my-cnf 'general definitions') ; then
      [[ -z $VAL ]] || DATABASE_CONF_HASH[MySQLConfigFile]="$VAL"
    fi
  done
}

#..............................................................................
# declare_DATABASE_CONF_DEFAULT
#..............................................................................
declare_DATABASE_CONF_DEFAULT() {
  local USAGE='
syntax: declare_DATABASE_CONF_DEFAULT BU [CONFIG_FILE|KEY=VALUE PAIRS]
description: declares DATABASE_CONF_DEFAULT for BU
used environment: EPAGES_CONFIG
INSTALL_EPAGES_HOST
INSTALL_EPAGES_MYSQL_PASS
INSTALL_EPAGES_MY_CNF
'
  [[ $1 != ${1#-} || $# -lt 1 ]] && { echo "$USAGE" 1>&2; return 1; }

  local BU=$1 ; shift

  DATABASE_CONF_DEFAULT=()

  # set Backup= for base BUs (no CONFIG_FILE|KEY=VALUE -> in DATABASE_CONF_HASH)
  if [[ -z $1 ]]  ; then
    case $BU in
      ASPoolDB|epagesj|PatchSystemInfoDB|Site|Store|System)
	DATABASE_CONF_HASH[Backup]=Backup ;;
    esac
  fi

  # set StoreType= for base BUs
  if [[ $BU = ASPoolDB || $BU = Site || $BU = Store || $BU = System ]] ; then
    DATABASE_CONF_DEFAULT[StoreType]=$BU
  fi

  # set Database=
  if [[ $BU = Backup ]] ; then
    DATABASE_CONF_DEFAULT[Database]=mysql
  elif [[ $BU = ASPoolDB || $BU = epagesj || $BU = PatchSystemInfoDB ]] ; then
    DATABASE_CONF_DEFAULT[Database]=${BU,,}
  else
    DATABASE_CONF_DEFAULT[Database]=${BU,,}db
  fi

  # set login=, BackupHost=
  if [[ $BU = Backup ]] ; then
    DATABASE_CONF_DEFAULT[login]=root
  elif [[ $BU = ASPoolDB ]] ; then
    DATABASE_CONF_DEFAULT[login]=usr_${DATABASE_CONF_DEFAULT[Database]}
    [[ -z ${DATABASE_CONF_HASH[Backup]} ]] || DATABASE_CONF_DEFAULT[BackupHost]=none
  elif [[ $BU = PatchSystemInfoDB ]] ; then
    DATABASE_CONF_DEFAULT[login]=usr_patch
    [[ -z ${DATABASE_CONF_HASH[Backup]} ]] || DATABASE_CONF_DEFAULT[BackupHost]=none
  else
    DATABASE_CONF_DEFAULT[login]=usr_${DATABASE_CONF_DEFAULT[Database]}
    [[ -z ${DATABASE_CONF_HASH[Backup]} ]] || DATABASE_CONF_DEFAULT[BackupHost]=localhost
  fi

  # set active=1
  DATABASE_CONF_DEFAULT[active]=1

  # set passwd=
  if [[ ${DATABASE_CONF_HASH[login]:-${DATABASE_CONF_DEFAULT[login]}} != root ]] ; then
    DATABASE_CONF_DEFAULT[passwd]=epages
  else
    DATABASE_CONF_DEFAULT[passwd]="$INSTALL_EPAGES_MYSQL_PASS"
  fi

  # set MySQLConfigFile=
  if [[ -n $INSTALL_EPAGES_MY_CNF ]] ; then
    DATABASE_CONF_DEFAULT[MySQLConfigFile]="$INSTALL_EPAGES_MY_CNF"
  fi

  # set datasource=dbi:mysql:
  if [[ -z ${DATABASE_CONF_DEFAULT[datasource]} ]] ; then
    DATABASE_CONF_DEFAULT[datasource]="dbi:mysql:host=${INSTALL_EPAGES_HOST:-localhost}"
  fi

}

#..............................................................................
# update_database_conf
#..............................................................................

update_database_conf() {
  local USAGE='
syntax: [EPAGES_CONFIG=...] update_database_conf BU [CONFIG_FILE|KEY=VALUE PAIRS]
description: sets KEY=VALUE pairs for [Connection] in
  /srv/epages/eproot/Shared/Config/Database.d/$BU.conf
  - either use space-separated KEY=VALUE pairs
  - or use KEY=VALUE pairs from section [BU] in CONFIG_FILE
  - second argument is CONFIG_FILE if a) it is an existing
    file and b) only 2 arguments exist
  - if neither CONFIG_FILE nor KEY=VALUE exists then use default values
used environment: EPAGES_CONFIG
'
  [[ $1 != ${1#-} || $# -lt 1 ]] && { echo "$USAGE" 1>&2; return 1; }

  local BU=$1 ; shift

  local CONF_DIR=/srv/epages/eproot/Shared/Config/Database.d
  [[ -n $EPAGES_CONFIG ]] && CONF_DIR=$EPAGES_CONFIG/Database.d

  #----------------------------------------------------------------------------
  # declare -A DATABASE_CONF_HASH
  #----------------------------------------------------------------------------

  declare_DATABASE_CONF_HASH $BU $@

  #----------------------------------------------------------------------------
  # declare -A DATABASE_CONF_DEFAULT
  #----------------------------------------------------------------------------

  declare_DATABASE_CONF_DEFAULT $BU $@

  #----------------------------------------------------------------------------
  # edit Database.d/Store.conf
  #----------------------------------------------------------------------------

  # update [Connection]:passwd= in Database.d/$BU.conf
  EPAGES_CONFIG=${CONF_DIR%/*} update_database_conf_passwd "$BU" \
    "${DATABASE_CONF_HASH[passwd]-${DATABASE_CONF_DEFAULT[passwd]}}"

  unset DATABASE_CONF_HASH[passwd] DATABASE_CONF_DEFAULT[passwd]

  local DB_CONF=$CONF_DIR/$BU.conf
  chown ep_appl:epages $DB_CONF

  # overwrite Database.d/Store.conf with DATABASE_CONF_HASH values
  for key in ${!DATABASE_CONF_HASH[@]} ; do
    set_ini_value --overwrite $DB_CONF $key ${DATABASE_CONF_HASH[$key]} Connection
  done

  # set DATABASE_CONF_DEFAULT values in Database.d/Store.conf (if before empty)
  for key in ${!DATABASE_CONF_DEFAULT[@]} ; do
    set_ini_value $DB_CONF $key ${DATABASE_CONF_DEFAULT[$key]} Connection
  done

  # add host to datasource=
  local VAL=$(get_ini_value $DB_CONF datasource Connection)
  if [[ $VAL = dbi:mysql && -n $INSTALL_EPAGES_HOST ]] ; then
    VAL="dbi:mysql:host=$INSTALL_EPAGES_HOST"
    set_ini_value --overwrite $DB_CONF datasource $VAL Connection
  fi

  return 0
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
