#!/bin/bash

PKG=${0##*/} ; PKG=${PKG%.sh} ; PKG=${PKG/spec/epages}

#==============================================================================
# epages-appsrv.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-epages.sh
export update_epages_environment epages_service
export etc_default enable_ipv6 set_local_ips

. $(dirname $0)/fun-wi.sh
export set_wi set_wi_xml

. $(dirname $0)/fun-account.sh
export chown_epages chgrp_epages

. $(dirname $0)/fun-pkg.sh
export pkg_instance touch_pkg_result pkg_installed mktempdir grep_server

. $(dirname $0)/fun-host.sh
export get_host_name

. $(dirname $0)/fun-java.sh
export test_version

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-appsrv-pre
#..............................................................................
epages-appsrv-pre() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-appsrv-pre INSTANCE RPM_VERSION'
    echo 'description: pre-installation tasks of epages-appsrv package'
    echo 'example: epages-appsrv-pre 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  #----------------------------------------------------------------------------
  # backup WebRoot/ImageDB
  #----------------------------------------------------------------------------

  local BACKUP_IMAGEDB=$(mktempdir $PKG)/ImageDB
  if pkg_installed appsrv ; then
    local WEBROOT=/srv/epages/eproot/Shared/WebRoot
    local PREV_VERSION=$(pkg_installed -V appsrv | tail -1 | sed 's,-.*,,')
    PREV_VERSION=${PREV_VERSION//[^0-9.]/}
    echo $PREV_VERSION > $BACKUP_IMAGEDB
    if [[ ! -d $WEBROOT/ImageDB ]] ; then
      # do not backup WebRoot/ImageDB if not exist
      echo ignore > $BACKUP_IMAGEDB
    elif [[ -d $WEBROOT/.ImageDB ]] ; then
      # do not backup WebRoot/ImageDB if already backuped
      echo ignore > $BACKUP_IMAGEDB
    elif test_version $VERSION -ge 6.17.42 && test_version ${PREV_VERSION:-0} -lt 6.17.42
    then
      # backup if version to install >= 6.17.42 and previous version < 6.17.42
      /bin/cp -rp $WEBROOT/ImageDB $WEBROOT/.ImageDB
    fi
  else
    # do not backup WebRoot/ImageDB if first instance
    [[ -f $BACKUP_IMAGEDB ]] || echo ignore > $BACKUP_IMAGEDB
  fi

  local SKIP_RESTART=$(grep_server appsrv "$EPAGES_SPEC_SKIP_RESTART" "$EPAGES_SKIP_SPEC")
  [[ -n $SKIP_RESTART ]] || epages_service stop AS

  return $(touch_pkg_result $PKG pre 0 "SKIP_RESTART=$SKIP_RESTART")
}

#..............................................................................
# epages-appsrv-post
#..............................................................................
epages-appsrv-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-appsrv-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-appsrv package'
    echo 'example: epages-appsrv-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  local SKIP_RESTART=$(grep_server appsrv "$EPAGES_SPEC_SKIP_RESTART" "$EPAGES_SKIP_SPEC")
  local KEEP_CONFIG=$(grep_server appsrv "$EPAGES_SPEC_KEEP_CONFIG" "$EPAGES_SKIP_SPEC")
  local KEEP_SHARED_CONFIG=$(grep_server appsrv "$EPAGES_SPEC_KEEP_SHARED_CONFIG" "$KEEP_CONFIG")

  [[ -n $KEEP_CONFIG ]] || update_epages_environment -i "$1"

  local ETC_DEFAULT=$(readlink -f $(etc_default))
  local LS_ERROR=$(ls -l $ETC_DEFAULT 2>&1 >/dev/null)
  [[ -n $LS_ERROR ]] && return $(touch_pkg_result $PKG post 1 "$LS_ERROR")
  . $ETC_DEFAULT

  local HAS_ASC=$(strings $EPSERVICE | grep -i cacheserver)
  if [[ -n $HAS_ASC && -z $KEEP_SHARED_CONFIG ]] ; then
    set_wi EXCEPTION_HOST
    [[ $INSTANCE = first ]] && set_wi_xml AS
  elif [[ -z $KEEP_SHARED_CONFIG ]] ; then
    set_wi AS AS2RR EXCEPTION_HOST SECTIONS
    set_local_ips AS
  fi

  if [[ -z $KEEP_CONFIG ]] ; then
    enable_ipv6
     for i in $(grep -l BackupHost=localhost "$EPAGES_CONFIG"/Database.d/*.{conf,initial}) ; do
       [[ -z $KEEP_SHARED_CONFIG ]] || continue
      sed -i "s/\(BackupHost\)=localhost/\1=$(get_host_name AS)/" $i
    done
  fi

  chown_epages AS "$EPAGES_CONFIG/"

  if [[ -z $KEEP_CONFIG ]] ; then
    if [[ $INSTANCE = first ]] ; then
        chgrp_epages -R WS "$EPAGES_LOG/"
        chown_epages -R AS "$EPAGES_LOG/"
        chgrp_epages -R WS "$EPAGES_WEBROOT/"
        chown_epages -R AS_RUN "$EPAGES_WEBROOT/"
    else
      chgrp_epages WS "$EPAGES_WEBROOT/"
      chown_epages AS_RUN "$EPAGES_WEBROOT/"
    fi
  fi

  [[ -n $SKIP_RESTART ]] || epages_service restart AS

  #----------------------------------------------------------------------------
  # restore WebRoot/ImageDB
  #----------------------------------------------------------------------------

  local PREV_VERSION=ignore
  local BACKUP_IMAGEDB=$(mktempdir $PKG)/ImageDB
  local RESTORE_IMAGEDB=
  local WEBROOT=/srv/epages/eproot/Shared/WebRoot

  [[ -f $BACKUP_IMAGEDB ]] && PREV_VERSION=$(cat $BACKUP_IMAGEDB)
  if [[ $INSTANCE = first  || $PREV_VERSION = ignore || ! -d $WEBROOT/.ImageDB ]] ; then
    # do not restore if first instance or no backup
    RESTORE_IMAGEDB=
  elif test_version ${PREV_VERSION:-0} -lt 6.17.42 ; then
    # restore if version to install >= 6.17.42 and previous version < 6.17.42
    test_version $VERSION -ge 6.17.42 && RESTORE_IMAGEDB=1
  fi

  if [[ -z $RESTORE_IMAGEDB ]] ; then
    # return if WebRoot/ImageDB shall not be restored
    return $(touch_pkg_result $PKG post 0 "SKIP_RESTART=$SKIP_RESTART KEEP_CONFIG=$KEEP_CONFIG")
  fi

  if [[ -f /etc/debian_version ]] ; then
    # debian postinstall: files were already updated -> restore now
    if [[ -f $WEBROOT/ImageDB/EP_images.xml ]] ; then
      true
    elif [[ -d $WEBROOT/ImageDB ]] ; then
      /bin/cp -rp $WEBROOT/.ImageDB/* $WEBROOT/ImageDB || exit 0
    else
      /bin/mv $WEBROOT/.ImageDB $WEBROOT/ImageDB || exit 0
    fi
    /bin/rm -rf $WEBROOT/.ImageDB

  else
    # redhat postinstall: files were not yet updated -> restore after update
    TEMP_FILE=$(mktempdir $PKG)-ImageDB.sh
    echo '#!/bin/bash
WEBROOT=/srv/epages/eproot/Shared/WebRoot
[[ -d $WEBROOT/.ImageDB ]] || exit 0
while [[ -d /proc/$1 ]] || ps auxwww | egrep -q "(yum|rpm) .*[e]pages-appsrv"
do [[ $(type -t sleepenh) ]] && sleepenh 0.1 >/dev/null || usleep 100000 ; done
echo

if [[ -f $WEBROOT/ImageDB/EP_images.xml ]] ; then
  true
elif [[ -d $WEBROOT/ImageDB ]] ; then
  /bin/cp -rp $WEBROOT/.ImageDB/* $WEBROOT/ImageDB || exit 0
else
  /bin/mv $WEBROOT/.ImageDB $WEBROOT/ImageDB || exit 0
fi
/bin/rm -rf $WEBROOT/.ImageDB
' > $TEMP_FILE
    chmod 755 $TEMP_FILE
    $TEMP_FILE $$ >> $TEMP_FILE.log 2>&1 &
  fi

  return $(touch_pkg_result $PKG post 0 "SKIP_RESTART=$SKIP_RESTART KEEP_CONFIG=$KEEP_CONFIG")
}

#..............................................................................
# epages-appsrv-preun
#..............................................................................
epages-appsrv-preun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-appsrv-preun INSTANCE RPM_VERSION'
    echo 'description: preun-uninstallation tasks of epages-appsrv package'
    echo 'example: epages-appsrv-preun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  return $(touch_pkg_result $PKG preun 0)
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  pre)   epages-appsrv-pre   $2 $3 ; exit $? ;;
  post)  epages-appsrv-post  $2 $3 ; exit $? ;;
  preun) epages-appsrv-preun $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
