#!/bin/bash

PKG=${0##*/} ; PKG=${PKG%.sh} ; PKG=${PKG/spec/epages}

#==============================================================================
# epages-perl.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-epages.sh
export update_epages_environment epages_service

. $(dirname $0)/fun-pkg.sh
export pkg_instance touch_pkg_result mktempdir grep_server

. $(dirname $0)/fun-perl.sh
export backup_obsolete_perl set_perl_tree what_needs_lib new_perl_binaries

. $(dirname $0)/fun-java.sh
export test_version

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-perl-pre
#..............................................................................
epages-perl-pre() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-pre INSTANCE RPM_VERSION'
    echo 'description: pre-uninstallation tasks of epages-perl package'
    echo 'example: epages-perl-pre 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  local SKIP_RESTART=$(grep_server appsrv "$EPAGES_SPEC_SKIP_RESTART" "$EPAGES_SKIP_SPEC")

  [[ $INSTANCE = first ]] || rm -rf /srv/epages/eproot/Perl/man/man8

  if ! new_perl_binaries $VERSION >/dev/null ; then
    return $(touch_pkg_result $PKG pre 0 "no new perl binaries")
  fi

  # epages Flex AS are stopped in epages-perl.spec
  [[ ! -x /etc/init.d/ePagesConfigServer && -z $SKIP_RESTART ]] && epages_service stop AS

  local BACKUP_FILE_LIST=$(mktempdir $PKG)/$VERSION.obsoleted
  rm -f $BACKUP_FILE_LIST

  # backup obsolete perl files
  PERL_BACKUP=$(backup_obsolete_perl)
  [[ -s $PERL_BACKUP ]] && ln -s $PERL_BACKUP $BACKUP_FILE_LIST

  return $(touch_pkg_result $PKG pre 0)
}

#..............................................................................
# epages-perl-post
#..............................................................................
epages-perl-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-perl package'
    echo 'example: epages-perl-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  local SKIP_RESTART=$(grep_server appsrv "$EPAGES_SPEC_SKIP_RESTART" "$EPAGES_SKIP_SPEC")
  local KEEP_CONFIG=$(grep_server perl "${EPAGES_SPEC_KEEP_CONFIG:+1}" "$EPAGES_SKIP_SPEC")

  [[ -n $KEEP_CONFIG ]] || update_epages_environment -i "$1"

  # adjust selinux file context
  set_selinux4epages

  # versions < 6.17.36 create files from .ept files
  local LIB_LINUX=${EPAGES_PERL:-/srv/epages/eproot/Perl}/lib/linux
  [[ -f "$LIB_LINUX"/Config.pm.ept ]] && test_version $VERSION -lt 6.17.36 && set_perl_tree

  # start appserver
  [[ -n $SKIP_RESTART ]] || epages_service restart AS

  [[ $INSTANCE = first ]] && return $(touch_pkg_result $PKG post 0 "INSTANCE=first")

  if ! new_perl_binaries -f $VERSION ; then
    return $(touch_pkg_result $PKG post 0 "no new perl binaries")
  fi

  # if epages-perl-pre backed up obsolete perl files
  # then check these files for loading libssl/crypto

  local BACKUP_FILE_LIST=$(mktempdir $PKG)/$VERSION.obsoleted

  if [[ ! -s $BACKUP_FILE_LIST ]] ; then
    # return if no backup file list found
    return $(touch_pkg_result $PKG post 0 "no backup: $BACKUP_FILE_LIST")
  fi

  # shorten file list by files existing in current version
  OPENSSL_EX='Crypt/OpenSSL/(Bignum/Bignum|DSA/DSA|RSA/RSA|Random/Random|X509/X509).so'
  OTHER_EX='((Crypt|Net)/SSLeay/SSLeay|DBD/mysql/mysql).so'
  egrep -v "/($OPENSSL_EX|$OTHER_EX|lib32/)" $BACKUP_FILE_LIST > $BACKUP_FILE_LIST.short

  # find binaries which load libssl/crypto (1.0.0)
  local MESSAGE="libssl/crypto files found: $BACKUP_FILE_LIST.short"
  what_needs_lib $BACKUP_FILE_LIST.short libssl.so.1.0.0 libcrypto.so.1.0.0 | grep .

  # return if no such binaries found
  [[ $? = 0 ]] || return $(touch_pkg_result $PKG post 0 "no $MESSAGE")

  # die if outdated binaries found
  echo -e "\nError: additional CPAN modules found that must be recompiled"
  echo "  because OpenSSL has changed."
  echo -e "Without re-compilation ePages won't work correctly.\n"
  return $(touch_pkg_result $PKG post 1 "$MESSAGE")
}

#..............................................................................
# epages-perl-preun
#..............................................................................
epages-perl-preun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-preun INSTANCE RPM_VERSION'
    echo 'description: preun-uninstallation tasks of epages-perl package'
    echo 'example: epages-perl-preun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  return $(touch_pkg_result $PKG preun 0)
}

#..............................................................................
# epages-perl-postun
#..............................................................................
epages-perl-postun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-postun INSTANCE RPM_VERSION'
    echo 'description: postun-uninstallation tasks of epages-perl package'
    echo 'example: epages-perl-postun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  return $(touch_pkg_result $PKG postun 0)
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  pre)     epages-perl-pre    $2 $3 ; exit $? ;;
  post)    epages-perl-post   $2 $3 ; exit $? ;;
  preun)   epages-perl-preun  $2 $3 ; exit $? ;;
  postun)  epages-perl-postun $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
