#!/bin/bash

PKG=${0##*/} ; PKG=${PKG%.sh} ; PKG=${PKG/spec/epages}

#==============================================================================
# epages-release.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/config.sh
export EPAGES_UPGRADE_PATH

. $(dirname $0)/fun-arch.sh
export nss_softokn_is_compatible

. $(dirname $0)/fun-epages.sh
export get_cartridge_version

. $(dirname $0)/fun-repoedit.sh
export enable_repo
export link_yum2zypper unlink_yum2zypper update_repo_arch refresh_repo_by_upgrade_path

. $(dirname $0)/fun-pkg.sh
export pkg_instance pkg_installed touch_pkg_result mktempdir

. $(dirname $0)/fun-httpd.sh
export backup_httpd_confd

. $(dirname $0)/fun-mysql.sh
export create_mysql_directories

. $(dirname $0)/fun-java.sh
export test_version

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-release-post
#..............................................................................
epages-release-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-release-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-release package'
    echo 'example: epages-release-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  if [[ -f /etc/debian_version ]] ; then
    return $(touch_pkg_result $PKG post 0 "debian: skip")
  fi

  local CORE_VERSION=$(get_cartridge_version --from-conf Core)
  [[ -n ${CORE_VERSION//[0-9.]/} ]] && CORE_VERSION=

  if test_version ${CORE_VERSION:-0} -lt 6.17.6 ; then
    local BAK=$(backup_httpd_confd native epages)
    [[ -n $BAK ]] && echo "Creating: $BAK"
  fi

  if [[ $INSTANCE = first && ! -f /etc/debian_version ]] ; then
    [[ -f /etc/yum.conf ]] || { touch /etc/yum.conf; chmod 644 /etc/yum.conf; }
    /var/epages/ini-config.sh -f /etc/yum.conf -s main -k protected_multilib -v false -i
  fi

  if test_version $VERSION -gt 6.17.38 ; then
    mkdir -m 755 -p /srv/epages/etc/sysconfig
    cp -rp /var/epages/conf/* /srv/epages/etc/sysconfig
  fi

  local NSS_TAR=$(mktempdir epages-requires)/nss_softokn.tar

  if [[ -z $SKIP_UPDATE_REPO ]] ; then
    # on CentOS 6, test if nss-softokn and nss-softokn-freebl are compatible
    # see https://bugzilla.redhat.com/show_bug.cgi?id=1182337
    if ! RPM_LIST=$(nss_softokn_is_compatible) ; then
      # if softokn libraries are not compatible:
      #   1. backup softokn libraries in tar file
      #   2. remember to update
      pushd / >/dev/null
      rpm -q $RPM_LIST --list | tar cfT $NSS_TAR - 2>/dev/null
      popd >/dev/null
      echo "
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
The nss packages of this installation must be updated, run:

  yum -y install nss-softokn nss-softokn.i686 nss-softokn-freebl nss-softokn-freebl.i686
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
"
    else
      # otherwise delete tar
      rm -f $NSS_TAR
    fi
  fi

  local REFRESH_REPO
  if [[ $INSTANCE = first ]] ; then
    /var/epages/SetRepository.sh -r "$EPAGES_REPO" -X -i
    REFRESH_REPO="exit 0"
  else
    if [[ -z $SKIP_UPDATE_REPO ]] ; then
      # the repository must be refreshed so that it obeys the upgrade path
      REFRESH_REPO=$(refresh_repo_by_upgrade_path -b yum:$VERSION $EPAGES_UPGRADE_PATH)
    fi
    if test_version ${CORE_VERSION:-0} -ge 6.17.5 ; then
      update_repo_arch setup base arch
    fi
  fi

  if [[ -z "$REFRESH_REPO$SKIP_UPDATE_REPO" ]] && test_version ${CORE_VERSION:-0} -lt 6.17.5
  then
    /var/epages/SetRepository.sh -r "$EPAGES_REPO" -X -i
    REFRESH_REPO='
if [ -f /etc/redhat-release ] ; then
  yum --enablerepo="epages-*" --disablerepo "epages*-6.*" clean metadata 2>/dev/null
elif [ -f /etc/SuSE-release ] ; then
  ( zypper clean -m ; zypper refresh ) 2>/dev/null
elif [ -f /etc/debian_version ] ; then
  apt-get -y update
fi'
  fi

  if [[ -n $REFRESH_REPO ]] ; then
    TEMP_FILE=$(mktempdir $PKG)/SetRepository-f.sh.~$$~

    # can't change existing config files when rpm is running
    # therefore 1st we create a shell script that
    # waits until the rpm program has finished
    echo '#!/bin/sh
while [ -d /proc/$1 ] || ps auxwww | egrep -q "(zypper .*up|yum .*up|rpm .*U).*[e]pages-release"
do [[ $(type -t sleepenh) ]] && sleepenh 0.1 >/dev/null || usleep 100000 ; done
echo
[ -f /etc/debian_version ] || rpm -qa | grep -qi b1d28320 || rpm --import /etc/pki/rpm-gpg/RPM-GPG-KEY-epages
' > $TEMP_FILE
    # and refreshes the repository (by e.g. yum clean all)
    echo  "$REFRESH_REPO" >> $TEMP_FILE
    chmod 755 $TEMP_FILE
    # and 2nd we run that shell script in the background
    $TEMP_FILE $$ > $TEMP_FILE.log 2>&1 &
  fi

  enable_repo -e 0 empty update >/dev/null
  enable_repo -e 1 base >/dev/null

  if pkg_installed common ; then
    enable_repo -e 0 setup >/dev/null
  elif [[ -z $CORE_VERSION ]] ; then
    enable_repo -e 1 setup >/dev/null
  fi

  link_yum2zypper

  [[ -f /etc/SuSE-release ]] && cp -p /var/epages/mongod /etc/init.d
  cp -lf /var/epages/delete-epages.SH /var/epages/delete-epages.sh

  create_mysql_directories

  return $(touch_pkg_result $PKG post 0)
}

#..............................................................................
# epages-release-preun
#..............................................................................
epages-release-preun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-release-preun INSTANCE RPM_VERSION'
    echo 'description: pre-uninstallation tasks of epages-release package'
    echo 'example: epages-release-preun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  if [[ -f /etc/debian_version ]] ; then
    return $(touch_pkg_result $PKG preun 0 "debian: skip")
  fi

  unlink_yum2zypper

  if [[ $INSTANCE = last ]] ; then
    # reset %config(noreplace)
    mv /etc/sysconfig/epages6.rpmsave /etc/sysconfig/epages6 2>/dev/null
    mv /var/epages/mongod.rpmsave /var/epages/mongod 2>/dev/null
    rm -f /etc/yum.repos.d/*epages-{setup,update,empty}*
  fi

  return $(touch_pkg_result $PKG preun 0)
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  post)  epages-release-post  $2 $3 ; exit $? ;;
  preun) epages-release-preun $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
