#========================================================================================
# package      DE_EPAGES::TaxUSSystemDB::API::Table::Zip2TaxUSMeta
# state        public
# description  interface for table Zip2TaxUSMeta
#========================================================================================
package DE_EPAGES::TaxUSSystemDB::API::Table::Zip2TaxUSMeta;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    InsertZip2TaxUSMeta
    UpdateZip2TaxUSMeta
    DeleteZip2TaxUSMeta
    GetAllPKeyZip2TaxUSMetas
    GetAllInfoZip2TaxUSMetas
    GetInfoZip2TaxUSMeta
    ExistsZip2TaxUSMeta
    GetPKeyZip2TaxUSMetasByMetaKey
    GetInfoZip2TaxUSMetasByMetaKey

    GetInfoZip2TaxUSMetaValueByMetaKey
    GetCurrentZip2TaxUSVersion
    GetNextZip2TaxUSVersion
    GetZip2TaxUSUpdateURL
    GetZip2TaxUSFallbackURL
);

## no critic qw(RequireCorrectImports)
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Core::API::TestParameter qw (
    TestDateTime
    TestFloat
    TestInteger
    TestString
);
use DE_EPAGES::TaxUSSystemDB::DAL::Zip2TaxUS;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NoResults_Zip2TaxUSMeta' => 'Zip2TaxUSMeta doesn\'t exist (MetaKey=#MetaKey)',
    };
}
#ERRORDEFINITION END

#========================================================================================
# function     InsertZip2TaxUSMeta
#----------------------------------------------------------------------------------------
# syntax       InsertZip2TaxUSMeta($hZip2TaxUSMeta);
#----------------------------------------------------------------------------------------
# description  Creates an entry of Zip2TaxUSMeta.
#----------------------------------------------------------------------------------------
# input        $hZip2TaxUSMeta | with following keys:
#               <ul>
#                   <li>MetaKey       -                      (primary key)- varchar(255)
#                   <li>MetaValue     -                      (optional)- text
#               </ul> | reference to hash
#========================================================================================
sub InsertZip2TaxUSMeta ($) {
    my ($hZip2TaxUSMeta) = @_;
    DAL_InsertZip2TaxUSMeta($hZip2TaxUSMeta);
    my $MetaKey = $hZip2TaxUSMeta->{'MetaKey'};
    TriggerHook('API_InsertZip2TaxUSMeta', { 'MetaKey' => $MetaKey });
    return;
}

#========================================================================================
# function     UpdateZip2TaxUSMeta
#----------------------------------------------------------------------------------------
# syntax       UpdateZip2TaxUSMeta($MetaKey, $hZip2TaxUSMeta);
# syntax       UpdateZip2TaxUSMeta($hZip2TaxUSMeta); # if $hZip2TaxUSMeta includes MetaKey
#----------------------------------------------------------------------------------------
# description  Updates a(n) Zip2TaxUSMeta if any element was changed.
#----------------------------------------------------------------------------------------
# input        $MetaKey       |                      (primary key) | varchar(255)
# input        $hZip2TaxUSMeta | with following keys:
#               <ul>
#                   <li>MetaValue     -                      (optional)- text
#               </ul> | reference to hash
#========================================================================================
sub UpdateZip2TaxUSMeta ($;$) {
    my ($MetaKey, $hZip2TaxUSMeta) = @_;
    if (ref $MetaKey eq 'HASH') {
        $hZip2TaxUSMeta = $MetaKey;
        $MetaKey = $hZip2TaxUSMeta->{'MetaKey'};
    } else {
        $hZip2TaxUSMeta->{'MetaKey'} = $MetaKey;
    }
    TestString('MetaKey',$MetaKey);
    TriggerHook('API_BeforeUpdateZip2TaxUSMeta', $hZip2TaxUSMeta );
    DAL_UpdateZip2TaxUSMeta($MetaKey, $hZip2TaxUSMeta);
    TriggerHook('API_AfterUpdateZip2TaxUSMeta', $hZip2TaxUSMeta );
    return 1;
}

#========================================================================================
# function     DeleteZip2TaxUSMeta
#----------------------------------------------------------------------------------------
# syntax       DeleteZip2TaxUSMeta($MetaKey);
# syntax       DeleteZip2TaxUSMeta($hZip2TaxUSMeta); # if $hZip2TaxUSMeta includes MetaKey
#----------------------------------------------------------------------------------------
# description  Deletes a(n) Zip2TaxUSMeta.
#----------------------------------------------------------------------------------------
# input        $MetaKey       |                      (primary key) | varchar(255)
#========================================================================================
sub DeleteZip2TaxUSMeta ($) {
    my ($MetaKey) = @_;
    if (ref $MetaKey eq 'HASH') {
        my $hInfo = $MetaKey;
        $MetaKey = $hInfo->{'MetaKey'};
    }
    TestString('MetaKey',$MetaKey);
    TriggerHook('API_DeleteZip2TaxUSMeta', { 'MetaKey' => $MetaKey } );
    DAL_DeleteZip2TaxUSMeta($MetaKey);
    return;
}

#========================================================================================
# function     GetAllPKeyZip2TaxUSMetas
#----------------------------------------------------------------------------------------
# syntax       $aZip2TaxUSMetaIDs = GetAllPKeyZip2TaxUSMetas();
#----------------------------------------------------------------------------------------
# description  Returns all Zip2TaxUSMeta identifiers.
#----------------------------------------------------------------------------------------
# return       $aZip2TaxUSMetaIDs |  | reference to array of varchar(255)
#========================================================================================
sub GetAllPKeyZip2TaxUSMetas () { DAL_GetAllPKeyZip2TaxUSMetas(); }

#========================================================================================
# function     GetAllInfoZip2TaxUSMetas
#----------------------------------------------------------------------------------------
# syntax       $ahZip2TaxUSMetas = GetAllInfoZip2TaxUSMetas();
#----------------------------------------------------------------------------------------
# description  Returns all objects of Zip2TaxUSMeta.
#----------------------------------------------------------------------------------------
# return       $ahZip2TaxUSMetas | with following keys:
#               <ul>
#                   <li>MetaKey       -                      (primary key)- varchar(255)
#                   <li>MetaValue     -                      (optional)- text
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoZip2TaxUSMetas () { DAL_GetAllInfoZip2TaxUSMetas(); }

#========================================================================================
# function     GetInfoZip2TaxUSMeta
#----------------------------------------------------------------------------------------
# syntax       $hZip2TaxUSMeta = GetInfoZip2TaxUSMeta($MetaKey);
#----------------------------------------------------------------------------------------
# description  Returns the Zip2TaxUSMeta by identifier.
#----------------------------------------------------------------------------------------
# input        $MetaKey       |                      (primary key) | varchar(255)
# return       $hZip2TaxUSMeta | with following keys:
#               <ul>
#                   <li>MetaKey       -                      (primary key)- varchar(255)
#                   <li>MetaValue     -                      (optional)- text
#               </ul>| reference to hash
#========================================================================================
sub GetInfoZip2TaxUSMeta ($) {
    my ($MetaKey) = @_;
    TestString('MetaKey',$MetaKey);
    my $hZip2TaxUSMeta = DAL_GetInfoZip2TaxUSMeta($MetaKey);
    Error('NoResults_Zip2TaxUSMeta', { 'MetaKey' => $MetaKey }) unless defined $hZip2TaxUSMeta;
    return $hZip2TaxUSMeta;
}

#========================================================================================
# function     ExistsZip2TaxUSMeta
#----------------------------------------------------------------------------------------
# syntax       $exists = ExistsZip2TaxUSMeta($MetaKey);
#----------------------------------------------------------------------------------------
# description  Returns true if the Zip2TaxUSMeta exists.
#----------------------------------------------------------------------------------------
# input        $MetaKey       |                      (primary key) | varchar(255)
# return       $exists | 1/0  the Zip2TaxUSMeta exists (yes/no) | boolean
#========================================================================================
sub ExistsZip2TaxUSMeta ($) {
    my ($MetaKey) = @_;
    TestString('MetaKey',$MetaKey);
    my $hZip2TaxUSMeta = DAL_GetInfoZip2TaxUSMeta($MetaKey);
    return defined $hZip2TaxUSMeta ? 1 : 0;
}

#========================================================================================
# function     GetPKeyZip2TaxUSMetasByMetaKey
#----------------------------------------------------------------------------------------
# syntax       $aKeys = GetPKeyZip2TaxUSMetasByMetaKey($MetaKey);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of Zip2TaxUSMeta by alternate key MetaKey.
#----------------------------------------------------------------------------------------
# input        $MetaKey |  | varchar(255)
# return       $aKeys |  (primary key) | reference to array of varchar(255)
#========================================================================================
sub GetPKeyZip2TaxUSMetasByMetaKey ($) {
    my ($MetaKey) = @_;
    TestString('MetaKey',$MetaKey);
    return DAL_GetPKeyZip2TaxUSMetasByMetaKey($MetaKey);
}

#========================================================================================
# function     GetInfoZip2TaxUSMetasByMetaKey
#----------------------------------------------------------------------------------------
# syntax       $ahZip2TaxUSMetas = GetInfoZip2TaxUSMetasByMetaKey($MetaKey);
#----------------------------------------------------------------------------------------
# description  Returns objects of Zip2TaxUSMeta by alternate key MetaKey.
#----------------------------------------------------------------------------------------
# input        $MetaKey |  | varchar(255)
# return       $ahZip2TaxUSMetas | with following keys:
#               <ul>
#                   <li>MetaKey       -                      (primary key)- varchar(255)
#                   <li>MetaValue     -                      (optional)- text
#               </ul> | reference to array of hashes
#========================================================================================
sub GetInfoZip2TaxUSMetasByMetaKey ($) {
    my ($MetaKey) = @_;
    TestString('MetaKey',$MetaKey);
    return DAL_GetInfoZip2TaxUSMetasByMetaKey($MetaKey);
}

#========================================================================================
# function     GetInfoZip2TaxUSMetaValueByMetaKey
#----------------------------------------------------------------------------------------
# syntax       my $MetaValue = GetInfoZip2TaxUSMetaValueByMetaKey($Key);
#----------------------------------------------------------------------------------------
# description  Returns the value of a zip2taxusmeta entry by the given key.
#----------------------------------------------------------------------------------------
# input         $MetaKey | key | string
# return       $MetaValue | value | string
#========================================================================================
sub GetInfoZip2TaxUSMetaValueByMetaKey ($) {
    my ($Key) = @_;
    my $ahInfo = GetInfoZip2TaxUSMetasByMetaKey($Key);

    if (scalar @$ahInfo) {
        return $ahInfo->[0]->{'MetaValue'};
    }
    else {
        Error("No zip2taxusmeta key '$Key' found!");
    }
}

#========================================================================================
# function     GetCurrentZip2TaxUSVersion
#----------------------------------------------------------------------------------------
# syntax       my $CurrentVersion = GetCurrentZip2TaxUSVersion();
#----------------------------------------------------------------------------------------
# description  Returns the current stored zip2taxus version.
#----------------------------------------------------------------------------------------
# return       $CurrentVersion | current stored version | string
#========================================================================================
sub GetCurrentZip2TaxUSVersion {
    return GetInfoZip2TaxUSMetaValueByMetaKey('Version');
}

#========================================================================================
# function     GetNextZip2TaxUSVersion
#----------------------------------------------------------------------------------------
# syntax       my $NextVersion = GetNextZip2TaxUSVersion();
#----------------------------------------------------------------------------------------
# description  Returns the next to-be-used zip2taxus version, which is the current date
#               formatted as YYYY-MM.
#----------------------------------------------------------------------------------------
# return       $NextVersion | next to-be-used zip2taxus version | string
#========================================================================================
sub GetNextZip2TaxUSVersion {
    return GetCurrentDateTime()->format_cldr('YYYY-MM');
}

#========================================================================================
# function     GetZip2TaxUSUpdateURL
#----------------------------------------------------------------------------------------
# syntax       my $UpdateURL = GetZip2TaxUSUpdateURL();
#----------------------------------------------------------------------------------------
# description  Returns the URL for fetching the zip2taxus csv from the provider.
#----------------------------------------------------------------------------------------
# return       $UpdateURL | zip2taxus update URL | string
#========================================================================================
sub GetZip2TaxUSUpdateURL {
    return GetInfoZip2TaxUSMetaValueByMetaKey('UpdateURL');
}

#========================================================================================
# function     GetZip2TaxUSFallbackURL
#----------------------------------------------------------------------------------------
# syntax       my $FallbackURL = GetZip2TaxUSFallbackURL();
#----------------------------------------------------------------------------------------
# description  Returns the fallback URL if GetZip2TaxUSUpdateURL() fails
#----------------------------------------------------------------------------------------
# return       $FallbackURL | zip2taxus update URL | string
#========================================================================================
sub GetZip2TaxUSFallbackURL {
    return 'ftp://ftp.epages.com/pub/epages/data/Zip2Tax.csv';
}

1;
