#==============================================================================
# package      DE_EPAGES::TaxUSSystemDB::API::Zip2TaxUS
# state        public
#------------------------------------------------------------------------------
# description  Functions for the zip2taxus database table
#==============================================================================
package DE_EPAGES::TaxUSSystemDB::API::Zip2TaxUS;
use base Exporter;

use strict;
use DE_EPAGES::Core::API::CSV qw ( ParseCSV );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( GetFileContentLines );
use DE_EPAGES::Core::API::Net qw ( GetFile );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( RunOnSystem );
use DE_EPAGES::TaxUSSystemDB::API::Constants qw ( ORIGIN_BASED_US_STATES );
use DE_EPAGES::TaxUSSystemDB::API::Table::Zip2TaxUS qw (
    DeleteZip2TaxUSByVersion
    ExistsZip2TaxUS
    GetInfoZip2TaxUS
    GetInfoZip2TaxUSsByVersion
    GetSalesTaxRangeByState
    InsertZip2TaxUS
);
use DE_EPAGES::TaxUSSystemDB::API::Table::Zip2TaxUSMeta qw (
    ExistsZip2TaxUSMeta
    GetCurrentZip2TaxUSVersion
    GetNextZip2TaxUSVersion
    GetZip2TaxUSUpdateURL
    GetZip2TaxUSFallbackURL
    InsertZip2TaxUSMeta
    UpdateZip2TaxUSMeta
);
use File::Temp;

our @EXPORT_OK = qw(
    UpdateZip2TaxUSMetaTable

    UpdateZip2TaxUSTable
    ImportZip2TaxUSTable

    GetInfoZip2TaxUSByZipCodeCityState
    GetTaxRateOrRangeByZipCodeCityState

    GetInfoZip2TaxUSsByZipCodePart
);

#========================================================================================
# function     UpdateZip2TaxUSMetaTable
#----------------------------------------------------------------------------------------
# syntax       UpdateZip2TaxUSMetaTable();
#----------------------------------------------------------------------------------------
# description  Initializes the zip2taxusmeta table with the given zip2taxus version and
#               update URL.
#               Will be run on store 'System' automatically.
#----------------------------------------------------------------------------------------
# input        $Version | zip2taxus version | string
# input        $UpdateURL | provider update URL | string
#========================================================================================
sub UpdateZip2TaxUSMetaTable {
    my ($Version, $UpdateURL) = @_;

    RunOnSystem(sub {
        if (ExistsZip2TaxUSMeta('Version')) {
            UpdateZip2TaxUSMeta({
                'MetaKey' => 'Version',
                'MetaValue' => $Version
            });
        }
        else {
            InsertZip2TaxUSMeta({
                'MetaKey' => 'Version',
                'MetaValue' => $Version
            });
        }

        unless (ExistsZip2TaxUSMeta('UpdateURL')) {
            InsertZip2TaxUSMeta({
                'MetaKey' => 'UpdateURL',
                'MetaValue' => $UpdateURL
            });
        }
    });
}

#========================================================================================
# function     UpdateZip2TaxUSTable
#----------------------------------------------------------------------------------------
# syntax       UpdateZip2TaxUSTable();
#----------------------------------------------------------------------------------------
# description  Updates the zip2taxus table with the fresh fetched version from the
#               provider. Can be executed without downtime.
#               Will be run on store 'System' automatically.
#----------------------------------------------------------------------------------------
# return       $Message | returns a message if the current version equals the next
#               version (update already done in a given month), otherwise undef | string
#========================================================================================
sub UpdateZip2TaxUSTable {
    RunOnSystem(sub {
        my $CurrentVersion = GetCurrentZip2TaxUSVersion();
        my $NextVersion    = GetNextZip2TaxUSVersion();

        if ($NextVersion eq $CurrentVersion and scalar @{GetInfoZip2TaxUSsByVersion($CurrentVersion)}) {
            my $Message = 'Update of zip2taxus skipped: current version is current month';
            warn($Message);
            return $Message;
        }

        my $TempFile = File::Temp->new();

        my $URL = GetZip2TaxUSUpdateURL();
        my $ResponseCode = GetFile($URL, $TempFile->filename);

        my $UseFallBack;
        if ($ResponseCode) {
            # if we can't download table from origin
            # try fallback
            _tryFallback(sprintf('Error while fetching zip2tax table from %s: %s',
                                 $URL, $ResponseCode),
                         $TempFile->filename);
            $UseFallBack = 1;
        }

        eval { ImportZip2TaxUSTable($TempFile->filename, $NextVersion); };
        if (ExistsError()) {
            # if downloaded table from origin is corrupt
            my $Error = GetError();
            # die if we already used fallback
            Error($Error) if $UseFallBack;
            # otherwise try fallback
            _tryFallback($Error, $TempFile->filename);
            ImportZip2TaxUSTable($TempFile->filename, $NextVersion);
        }

        # switch meta table to new version and update cache
        UpdateZip2TaxUSMeta('Version', {'MetaValue' => $NextVersion});

        # delete rows with old version
        DeleteZip2TaxUSByVersion($CurrentVersion) unless ($NextVersion eq $CurrentVersion);

        File::Temp::cleanup();
    });
}

sub _tryFallback {
    my ($Error, $TempFile) = @_;
    Warning($Error);
    my $URL = GetZip2TaxUSFallbackURL();
    my $ResponseCode = GetFile($URL, $TempFile);
    Error(sprintf('Error while fetching zip2tax table from %s: %s',
                  $URL, $ResponseCode)) if $ResponseCode;
}

#========================================================================================
# function     ImportZip2TaxUSTable
#----------------------------------------------------------------------------------------
# syntax       ImportZip2TaxUSTable($FileName, $Version);
#----------------------------------------------------------------------------------------
# description  Imports a zip2tax-formatted csv file to the zip2taxus table. Existing
#               entries won't be updated.
#               Will be run on store 'System' automatically.
#----------------------------------------------------------------------------------------
# input        $FileName | file path of the csv file | string
# input        $Version | version, defaults to current version | string
#========================================================================================
sub ImportZip2TaxUSTable {
    my ($FileName, $Version) = @_;
    TestString('FileName', $FileName);

    RunOnSystem(sub {
        $Version //= GetCurrentZip2TaxUSVersion();

        my $aFileContent = GetFileContentLines($FileName);

        my $HeadLine = shift @$aFileContent;
        $HeadLine =~ s/"//g;

        my $aFields = ParseCSV($HeadLine, {'sep_char' => ','});
        my $FieldsCount = scalar(@$aFields) - 1;

        foreach my $TaxEntry (@$aFileContent) {
            my $aValues = ParseCSV($TaxEntry, {'sep_char' => ','});
            if (scalar(@$aValues)) {
                my $hParams;
                $hParams->{$aFields->[$_]} = "$aValues->[$_]" foreach (0 .. $FieldsCount);
                $hParams->{'Version'} = $Version;

                unless (ExistsZip2TaxUS($hParams->{'ZipCode'}, $hParams->{'City'}, $hParams->{'State'}, $hParams->{'Version'})) {
                    $hParams->{'SalesTaxRate'} = $hParams->{'SalesTaxRate'} / 1000 if $hParams->{'SalesTaxRate'} >= 100;
                    $hParams->{'ZipCode'} = sprintf('%05d', $hParams->{'ZipCode'});

                    InsertZip2TaxUS($hParams);
                }
            }
        }
    });
}


#========================================================================================
# function     GetTaxRateOrRangeByZipCodeCityState
#----------------------------------------------------------------------------------------
# syntax       GetTaxRateOrRangeByZipCodeCityState($hParams);
#----------------------------------------------------------------------------------------
# description  Gets the SalesTaxRate for primary keys ZipCode, City, State. if the
#               state is origin based.
#               Gets the SalesTaxRate range for the State
#               ZipCode, City, State. if the state is destimation based.
#               Will be run on store 'System' automatically.
#----------------------------------------------------------------------------------------
# input        $hParams | with following keys:
#               <ul>
#                   <li>ZipCode       - (part of primary key)- varchar(50)
#                   <li>City          - (part of primary key)- varchar(50)
#                   <li>State         - (part of primary key)- varchar(50)
#               </ul> | reference to hash
#----------------------------------------------------------------------------------------
# return       $InfoZip2Tax | with following keys:
#               <ul>
#                   <li>SalesTaxRateMin       -  float
#                   <li>SalesTaxRateMax       -  float
#                   <li>OriginBased      -  bit
#               </ul>| reference to array of hashes
#========================================================================================
sub GetTaxRateOrRangeByZipCodeCityState {
    my ($hParams) = @_;
    TestString('ZipCode', $hParams->{'ZipCode'});
    TestString('City',    $hParams->{'City'});
    TestString('State',   $hParams->{'State'});

    my $hInfo = {};
    my $OriginBased = ElementOf($hParams->{'State'}, ORIGIN_BASED_US_STATES) ? 1 : 0;

    if ($OriginBased) {
        # we have one tax value
        my $hValues = GetInfoZip2TaxUSByZipCodeCityState($hParams);

        if ($hValues->{'SalesTaxRate'}) {
            $hInfo->{'SalesTaxRateMin'} = $hValues->{'SalesTaxRate'};
            $hInfo->{'SalesTaxRateMax'} = $hInfo->{'SalesTaxRateMin'};
        }
        else {
            $hInfo->{'SalesTaxRateMin'} = undef;
            $hInfo->{'SalesTaxRateMax'} = undef;
        }
    }
    else {
        # we have two tax values, get the range
        RunOnSystem(sub {
            $hInfo = GetSalesTaxRangeByState($hParams->{'State'})->[0];
        });
    }

    $hInfo->{'OriginBased'} = $OriginBased;
    return $hInfo;
}

#========================================================================================
# function     GetInfoZip2TaxUSByZipCodeCityState
#----------------------------------------------------------------------------------------
# syntax       GetInfoZip2TaxUSByZipCodeCityState($hParams);
#----------------------------------------------------------------------------------------
# description  Gets Info for primary keys ZipCode, City, State.
#               Will be run on store 'System' automatically.
#----------------------------------------------------------------------------------------
# input        $hParams | with following keys:
#               <ul>
#                   <li>ZipCode       - (part of primary key)- varchar(50)
#                   <li>City          - (part of primary key)- varchar(50)
#                   <li>State         - (part of primary key)- varchar(50)
#               </ul> | reference to hash
# return       $hZip2TaxUS | with following keys:
#               <ul>
#                   <li>z2t_ID        -                      (optional)- int
#                   <li>ZipCode       -                      (part of primary key)- varchar(50)
#                   <li>SalesTaxRate  -                      (optional)- float
#                   <li>RateState     -                      (optional)- float
#                   <li>ReportingCodeState -                      (optional)- varchar(50)
#                   <li>RateCounty    -                      (optional)- float
#                   <li>ReportingCodeCounty -                      (optional)- varchar(50)
#                   <li>RateCity      -                      (optional)- float
#                   <li>ReportingCodeCity -                      (optional)- varchar(50)
#                   <li>RateSpecialDistrict -                      (optional)- float
#                   <li>ReportingCodeSpecialDistrict -                      (optional)- varchar(50)
#                   <li>City          -                      (part of primary key)- varchar(50)
#                   <li>PostOffice    -                      (optional)- varchar(50)
#                   <li>State         -                      (part of primary key)- varchar(50)
#                   <li>County        -                      (optional)- varchar(50)
#                   <li>ShippingTaxable -                      (optional)- bit
#                   <li>PrimaryRecord -                      (optional)- bit
#                   <li>Version       -                      (part of primary key)- char(7)
#               </ul>| reference to hash
#========================================================================================
sub GetInfoZip2TaxUSByZipCodeCityState {
    my ($hParams) = @_;
    TestString('ZipCode', $hParams->{'ZipCode'});
    TestString('City',    $hParams->{'City'});
    TestString('State',   $hParams->{'State'});

    #shorten the zipcode to the 1st 5 digits because some cartridges uses extensions like 65801-1234
    my $ZipCode = substr($hParams->{'ZipCode'},0,5);
    my $hInfo = {};

    RunOnSystem(sub {
        if (ExistsZip2TaxUS($ZipCode, $hParams->{'City'}, $hParams->{'State'}, GetCurrentZip2TaxUSVersion())) {
            $hInfo = GetInfoZip2TaxUS($ZipCode, $hParams->{'City'}, $hParams->{'State'}, GetCurrentZip2TaxUSVersion());
        }
    });

    return $hInfo;
}

#========================================================================================
# function     GetInfoZip2TaxUSsByZipCodePart
#----------------------------------------------------------------------------------------
# syntax       $ahZip2TaxUSs = GetInfoZip2TaxUSsByZipCodePart($ZipCode);
#----------------------------------------------------------------------------------------
# description  Returns objects of Zip2TaxUS by alternate key ZipCode.
#               Will be run on store 'System' automatically.
#----------------------------------------------------------------------------------------
# input        $ZipCode |  | varchar(50)
# return       $ahZip2TaxUSs | with following keys:
#               <ul>
#                   <li>z2t_ID        -                      (optional)- int
#                   <li>ZipCode       -                      (part of primary key)- varchar(50)
#                   <li>SalesTaxRate  -                      (optional)- float
#                   <li>RateState     -                      (optional)- float
#                   <li>ReportingCodeState -                      (optional)- varchar(50)
#                   <li>RateCounty    -                      (optional)- float
#                   <li>ReportingCodeCounty -                      (optional)- varchar(50)
#                   <li>RateCity      -                      (optional)- float
#                   <li>ReportingCodeCity -                      (optional)- varchar(50)
#                   <li>RateSpecialDistrict -                      (optional)- float
#                   <li>ReportingCodeSpecialDistrict -                      (optional)- varchar(50)
#                   <li>City          -                      (part of primary key)- varchar(50)
#                   <li>PostOffice    -                      (optional)- varchar(50)
#                   <li>State         -                      (part of primary key)- varchar(50)
#                   <li>County        -                      (optional)- varchar(50)
#                   <li>ShippingTaxable -                      (optional)- bit
#                   <li>PrimaryRecord -                      (optional)- bit
#               </ul> | reference to array of hashes
#========================================================================================
sub GetInfoZip2TaxUSsByZipCodePart {
    my ($ZipCode) = @_;
    my $ahZip2TaxUSs;

    #shorten the zipcode to the 1st 5 digits because some cartridges uses extensions like 65801-1234
    $ZipCode = substr($ZipCode,0,5);

    RunOnSystem(sub {
        $ahZip2TaxUSs = DE_EPAGES::TaxUSSystemDB::API::Table::Zip2TaxUS::GetInfoZip2TaxUSsByZipCodePart($ZipCode);
    });

    return $ahZip2TaxUSs;
}

1;
