#!/bin/bash

#==============================================================================
# install-RPM.sh
#==============================================================================

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

LANG=C # use base language for standard output of some commands
LC_ALL=C

#------------------------------------------------------------------------------
# arguments
#------------------------------------------------------------------------------

usage() {
  [[ -n $1 ]] && echo -e "Error:\n  $@"
  echo "Usage:"
  echo "  ENABLE_REPO=... DISABLE_REPO=... $0 [OPTIONS] RPM1/DEB1 RPM2/DEB2 ..."
  echo "Options:"
  echo "  --update-only: only update the installed RPM/DEB"
  echo "  --update: update or install the RPM/DEB"
  echo "  (none): only install RPM/DEB if not installed yet"
  echo "  --workaround-fileconflicts: use workaround for file conflicts (yum only)"
  echo "Environment variables:"
  echo "  ENABLE/DISABLE_REPO: space sep. list of repos to use/skip"
  echo "  use one/more of: epages-base epages-setup epages-update epages-hotfix"
  echo "Examples:"
  echo "  ENABLE_REPO=epages-setup $0 --update epages-perl epages-nodejs"
  echo "  ENABLE_REPO=epages-base $0 --update-only epages-requires epages-webconf"
  exit 1
}

TEMP=$(getopt -o '' --long update-only,update,workaround-fileconflicts -n $0 -- "$@") || usage
eval set -- "$TEMP"

DO=install
unset WORKAROUND_FILECONFLICTS
while true ; do
  case "$1" in
    --update-only) DO=update ; shift ;;
    --update) DO=install_or_update ; shift ;;
    --workaround-fileconflicts) WORKAROUND_FILECONFLICTS=1 ; shift ;;
    --) shift ; break ;;
    *) echo $@ ; usage ;;
  esac
done
# other arguments are packages to install
[[ -z $1 ]] && usage

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/functions.sh
export started_by_rpm

. $(dirname $0)/functions/fun-repoedit.sh
export enable_apt_repo restore_apt_repo

. $(dirname $0)/functions/fun-pkg.sh
export mktempdir

#------------------------------------------------------------------------------
# execute
#------------------------------------------------------------------------------

# if install-RPM.sh was executed by a packaging tool we can't run packaging tools
started_by_rpm $$ && exit 0

#..............................................................................
# find operating system and its packaging tools
#..............................................................................

case $EPAGES_KERNEL in
  *d) unset PERL5LIB ; INSTALLER=apt-get ;; # debian
  *r) INSTALLER=yum ;; # redhat
  *) echo "$0: operating system not supported: $(uname -a)" ; exit 1 ;;
esac

#..............................................................................
# enable or disable repos depending on envvars ENABLE_REPO and DISABLE_REPO
#..............................................................................

unset APT_LIST_BACKUP CLEAN_ALL ENABLE DISABLE

for i in $ENABLE_REPO ; do
  case $INSTALLER in
    yum) ENABLE="$ENABLE --enablerepo $i" ;;
    apt-get) APT_LIST_BACKUP="$(enable_apt_repo 1 ${i/base/setup}) $APT_LIST_BACKUP" ;;
  esac
done
for i in $DISABLE_REPO ; do
  case $INSTALLER in
    yum) DISABLE="$DISABLE --disablerepo $i" ;;
    apt-get) APT_LIST_BACKUP="$(enable_apt_repo 0 ${i/base/setup}) $APT_LIST_BACKUP" ;;
  esac
done

#..............................................................................
# update packages
#..............................................................................

RESULT=SUCCESS
for PKG in $@ ; do
  IS_INSTALLED=
  if [[ $INSTALLER = apt-get ]] ; then
    dpkg --status $PKG >/dev/null 2>&1 && IS_INSTALLED=1
  else
    rpm --quiet -q $PKG >/dev/null 2>&1 && IS_INSTALLED=1
  fi
  if [[ -n $IS_INSTALLED ]] ; then
    [[ $DO = install ]] && continue
    # workaround: sometimes file conflicts occur
    if [[ -n $WORKAROUND_FILECONFLICTS && $INSTALLER = yum ]] ; then
      yum check-update -y $ENABLE $DISABLE 2>/dev/null | awk "/^$PKG\./{exit 123}"
      [[ $? = 123 ]] || continue # up-to-date
      case $PKG in
        epages-j|epages-nodejs) rpm -e --nodeps $PKG ; CLEAN_ALL=1 ;;
        epages-*) true ;;
        *) rpm -e --nodeps $PKG ; CLEAN_ALL=1 ;;
      esac
    fi
  else
    [[ $DO = update ]] && continue
  fi

  rm -f $(mktempdir $PKG)/post.*
  echo "$INSTALLER -y $ENABLE $DISABLE install $PKG"
  eval $INSTALLER -y $ENABLE $DISABLE install $PKG
  [[ $? = 0 ]] && RESULT=SUCCESS || RESULT=FAILURE

  [[ -f $(mktempdir $PKG)/post.success ]] && RESULT=SUCCESS
  [[ -f $(mktempdir $PKG)/post.failure ]] && RESULT=FAILURE
  echo "FINISHED: $INSTALLER install $PKG -> $RESULT"

  [[ $RESULT = SUCCESS ]] || break
done

#..............................................................................
# restore repositories
#..............................................................................

if [[ $INSTALLER = apt-get && -n $APT_LIST_BACKUP ]] ; then
  for backup in $APT_LIST_BACKUP ; do restore_apt_repo $backup ; done
elif [[ $INSTALLER = yum && -n $CLEAN_ALL ]] ; then
  yum clean all
fi

[[ $RESULT = SUCCESS ]] && exit 0 || exit 1
