#!/bin/bash

#=======================================================================
# postinstall/appsrv.sh
#=======================================================================

if [ ! -w /etc/passwd ] ; then
  echo "$0: only 'root' can execute this program"
  exit 1
fi

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  [[ -n $EPAGES_INSTALLATION_MOCK ]] && echo "$@" || echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

#-----------------------------------------------------------------------
# define variables and functions
#-----------------------------------------------------------------------

[[ -n $1 ]] && EPAGES_INSTALL_CONF=$(readlink -m $1) || EPAGES_INSTALL_CONF=

# do nothing if standby
if [[ $EPAGES_INSTALL_CONF =~ /standby ]] ; then
  ECHO $LINENO "EPAGES_SPEC_SKIP_RESTART=1 (/standby in $EPAGES_INSTALL_CONF)"
  exit 0
fi

PERL_ENV=$PERL
EPAGES_CONFIG_ENV=$EPAGES_CONFIG

CONF_DIR=$(cd $(dirname $0)/../conf && pwd)
HOTFIX_DIR=$(cd $(dirname $0)/../hotfix 2>/dev/null && pwd)
BIN_DIR=$(cd $(dirname $0)/../bin && pwd)
FUN_DIR=$(cd $(dirname $0)/../functions && pwd)

. $FUN_DIR/fun-epages.sh
export etc_default find_ini_config set_ini_value get_ini_value get_cartridge_fullversion

. $FUN_DIR/fun-java.sh
export test_version jre_home java_executable

. $FUN_DIR/fun-pkg.sh
export created_directories mktempdir

. $FUN_DIR/fun-arch.sh
export absolute_path

. $(etc_default)
. /etc/sysconfig/epages6

EPAGES=${EPAGES:-/srv/epages/eproot}
EPAGES_CONFIG=${EPAGES_CONFIG:-/srv/epages/eproot/Shared/Config}
EPAGES_CONFIG=${EPAGES_CONFIG_ENV:-$EPAGES_CONFIG}
EPAGES_CARTRIDGES=${EPAGES_CARTRIDGES:-$EPAGES/Cartridges}
PERL=${PERL:-$EPAGES/Perl/bin/perl}
PERL=${PERL_ENV:-$PERL}

set -e

pushd $EPAGES_CARTRIDGES/DE_EPAGES >/dev/null

$PERL Makefile.PL
chown ep_appl:epages makefile

if [[ $EPAGES_INSTALLATION_MOCK =~ '.' ]] ; then
  VERSION=$EPAGES_INSTALLATION_MOCK
else
  VERSION=$(get_cartridge_version Core)
fi

#-----------------------------------------------------------------------
# read patchhost.conf
#-----------------------------------------------------------------------

PATCHHOST_CONF=
if [[ -f $EPAGES_INSTALL_CONF ]] ; then
  PATCHHOST_CONF=$(grep '^\s*patchhost\s*=' $EPAGES_INSTALL_CONF 2>/dev/null | head -1)
fi
if [[ -n $PATCHHOST_CONF ]] ; then
  PATCHHOST_CONF=$(absolute_path ${PATCHHOST_CONF#*=} $EPAGES_INSTALL_CONF) || true
fi
if [[ -f $PATCHHOST_CONF ]] ; then
  ECHO $LINENO "source $PATCHHOST_CONF"
  source $PATCHHOST_CONF
fi

#-----------------------------------------------------------------------
# hotfixes
#-----------------------------------------------------------------------

if test_version $VERSION -ge 6.17.26 && test_version $VERSION -lt 6.17.35 ; then
  if pushd $HOTFIX_DIR/6.17.35 >/dev/null ; then
    # AD-3132: use workaround for TaxUSSystemDB/API/Zip2TaxUS.pm if versions < 6.17.35
    find DE_EPAGES/TaxUSSystemDB -name '*.pm' | while read file ; do
      ECHO $LINENO "cat $file > $EPAGES_CARTRIDGES/DE_EPAGES/$file"
      [[ -z $EPAGES_INSTALLATION_MOCK ]] || continue
      cat $file > $EPAGES_CARTRIDGES/DE_EPAGES/$file
    done
    popd >/dev/null
  fi
fi

#-----------------------------------------------------------------------
# make install and exit unless install-epages.conf exists
#-----------------------------------------------------------------------

MYSQLD_INSTALL_CONF=
if [[ -f $EPAGES_INSTALL_CONF ]] ; then
  for i in $(grep '^\s*mysqld\s*=' $EPAGES_INSTALL_CONF 2>/dev/null) ; do
    i=$(absolute_path ${i#*=} $EPAGES_INSTALL_CONF) || continue
    [[ $MYSQLD_INSTALL_CONF =~ " $i" ]] && continue || true
    MYSQLD_INSTALL_CONF="$MYSQLD_INSTALL_CONF $i"
  done
fi

INI_CONFIG=$(find_ini_config) || MYSQLD_INSTALL_CONF=
if [[ -z $MYSQLD_INSTALL_CONF ]] ; then
  ECHO $LINENO "make install $EPAGES_INSTALL_MAKE_ARGS"
  [[ -z $EPAGES_INSTALLATION_MOCK ]] || exit 0
  eval make install $EPAGES_INSTALL_MAKE_ARGS
  exit $?
fi

#-----------------------------------------------------------------------
# get store/storetype information
#-----------------------------------------------------------------------

declare -A BASETYPE CARTRIDGES LANGUAGES
declare -A ALL_STORETYPES ALL_LANGUAGES
unset DEFAULT_LANGUAGE

for conf in $MYSQLD_INSTALL_CONF ; do
  while read section ; do
    ST=$(get_ini_value $conf StoreType "$section") || continue
    [[ -n $ST ]] || continue
    ALL_STORETYPES[$ST]=1
    BASETYPE[$ST]=${BASETYPE[$ST]:-$(get_ini_value $conf StoreType.Base "$section")} || true
    CARTRIDGES[$ST]=${CARTRIDGES[$ST]:-$(get_ini_value $conf StoreType.Add "$section")} || true
    LANGUAGES[$ST]=${LANGUAGES[$ST]:-$(get_ini_value $conf StoreType.Lang "$section")} || true
  done < <($INI_CONFIG -f $conf -G)
done

pushd $EPAGES_CONFIG/Database.d >/dev/null
ALL_STORES=$(ls -d *.conf 2>/dev/null | egrep -v '^(System|Site).conf$')
popd >/dev/null

#-----------------------------------------------------------------------
# update Cartridges.xml and StoreTypes.d/[Languages]
#-----------------------------------------------------------------------

created_directories -m 755 $EPAGES_CONFIG/StoreTypes.d/$VERSION | while read dir ; do
  chmod 755 "$dir"
  chown ep_appl:epages "$dir"
done

CAXL=$EPAGES_CARTRIDGES_XML
CAXL=${CAXL:+-x $CAXL}
for ST in ${!ALL_STORETYPES[@]} ; do
  if [[ -n "${BASETYPE[$ST]}${CARTRIDGES[$ST]}" ]] ; then
    ECHO $LINENO "$BIN_DIR/update_cartridges_xml.sh -s $ST -b '${BASETYPE[$ST]}' -a '${CARTRIDGES[$ST]}' $CAXL"
    $BIN_DIR/update_cartridges_xml.sh -s $ST -b "${BASETYPE[$ST]}" -a "${CARTRIDGES[$ST]}" $CAXL
  fi
  # pretend that dialect is already installed because installing dialects by running
  # DE_EPAGES::Installer::API::LanguagePack->installStoreType won't work
  if [[ -n ${LANGUAGES[$ST]} ]] ; then
    touch $EPAGES_CONFIG/StoreTypes.d/$VERSION/$ST.conf
    chmod 644 $EPAGES_CONFIG/StoreTypes.d/$VERSION/$ST.conf
    chown ep_appl:epages $EPAGES_CONFIG/StoreTypes.d/$VERSION/$ST.conf
    for lang in ${LANGUAGES[$ST]} ; do
      DEFAULT_LANGUAGE=${DEFAULT_LANGUAGE:-$lang}
      ALL_LANGUAGES[$lang]=1
      set_ini_value --overwrite $EPAGES_CONFIG/StoreTypes.d/$VERSION/$ST.conf $lang installed Languages
    done
  fi
done

#-----------------------------------------------------------------------
# sortCartridgesXml.pl
#-----------------------------------------------------------------------

if [[ -n "${BASETYPE[$ST]}${CARTRIDGES[$ST]}" ]] ; then
  pushd $EPAGES_CARTRIDGES/DE_EPAGES >/dev/null
  CAXL=$EPAGES_CARTRIDGES_XML
  CAXL=${CAXL:+-file $CAXL}
  ECHO $LINENO "$PERL Installer/Scripts/sortCartridgesXml.pl $CAXL -inplace"
  $PERL Installer/Scripts/sortCartridgesXml.pl $CAXL -inplace
  popd >/dev/null
fi

#-----------------------------------------------------------------------
# get_locales
#-----------------------------------------------------------------------

get_locales () {
  local lang locales
  local LOCALES_CONF
  LOCALES_CONF=$(absolute_path locales.conf $EPAGES_INSTALL_CONF) || {
    LOCALES_CONF=$CONF_DIR/locales.conf
  }
  [[ -f $LOCALES_CONF ]] || return

  for lang in $@ ; do
    locales=$(get_ini_value $LOCALES_CONF Locales $lang) || continue
    echo -n " ${locales//,/ }"
  done
}

#-----------------------------------------------------------------------
# install language files
#-----------------------------------------------------------------------

# EPAGES_LANGUAGES/EPAGES_LOCALES are used by installDatabase.pl later
export EPAGES_LANGUAGES=${!ALL_LANGUAGES[@]}
export EPAGES_LOCALES=$(echo $(get_locales $DEFAULT_LANGUAGE $EPAGES_LANGUAGES))

#  'de' and 'en' already exist in Cartridges/; don't try to download
unset ALL_LANGUAGES[de] ALL_LANGUAGES[en]
languages=${!ALL_LANGUAGES[@]}

unset LEGACY_VERSION
[[ $(echo -e "6.17.46\n$VERSION" | sort -V | head -1) = 6.17.46 ]] || LEGACY_VERSION=1

if [[ -n $LEGACY_VERSION && -n $languages ]] ; then
  MOCK=${EPAGES_INSTALLATION_MOCK:+-unpack-only}
  ECHO $LINENO "$BIN_DIR/installLanguageFiles.pl $MOCK -version $VERSION -lang '$languages'"
  $BIN_DIR/installLanguageFiles.pl $MOCK -version $VERSION -lang "$languages"
fi

#-----------------------------------------------------------------------
# printCartridgesXml.pl
#-----------------------------------------------------------------------

CARTRIDGES_TXT=$(mktempdir epages-appsrv)/cartridges.txt
CAXL=$EPAGES_CARTRIDGES_XML
CAXL=${CAXL:+-xmlfile $CAXL}
ECHO $LINENO "$PERL $BIN_DIR/printCartridgesXml.pl $CAXL > $CARTRIDGES_TXT"
$PERL $BIN_DIR/printCartridgesXml.pl $CAXL > $CARTRIDGES_TXT


#-----------------------------------------------------------------------
# set_pl
#-----------------------------------------------------------------------

set_pl() {
  local path=$1
  local attribute=$2
  local SET=$EPAGES_CARTRIDGES/DE_EPAGES/Object/Scripts/set.pl

  ECHO $LINENO "set_pl $path $attribute"
  [[ -z $EPAGES_INSTALLATION_MOCK ]] || return 0
  [[ $path = /Shops/MasterShop ]] && bu=Master || bu=MultiStore
  $PERL $SET -storename $bu -path $path "$attribute"
}

#-----------------------------------------------------------------------
# make install_multistore_complete and exit if MultiStore is used
#-----------------------------------------------------------------------

SYNC=$EPAGES_CARTRIDGES/DE_EPAGES/ShopConfiguration/Scripts/synccartridges.pl

if echo $ALL_STORES | grep -q '\bMultiStore\b' ; then
  if grep -q install_multistore_complete makefile ; then
    TARGET=install_multistore_complete
  else
    TARGET=install_multistore
    [[ -n $EPAGES_IGNORE_DEMOSHOP ]] || TARGET="$TARGET install_multistore_demo"
  fi
  ECHO $LINENO "make $TARGET"
  [[ -n $EPAGES_INSTALLATION_MOCK ]] || make $TARGET

  for STORE in Site Master MultiStore Store ; do
    ECHO $LINENO "$PERL $SYNC -storename $STORE"
    [[ -n $EPAGES_INSTALLATION_MOCK ]] || $PERL $SYNC -storename $STORE
  done
  host=$(hostname --fqdn)

  # create users
  set_pl /Shops/MasterShop MS_WebServiceUser='{"Site":"/Users/wsadmin","MasterShop":"/Users/wsadmin","eRetail":"/Users/wsadmin","DropShipping":"/Users/wsadmin"}'
  set_pl /Shops/eRetail MS_WebServiceUser='{"Site":"/Users/wsadmin","MasterShop":"/Users/wsadmin","eRetail":"/Users/wsadmin"}'
  set_pl /Shops/DropShipping MS_WebServiceUser='{"Site":"/Users/wsadmin","MasterShop":"/Users/wsadmin","DropShipping":"/Users/wsadmin"}'

  # create URLs
  set_pl /Shops/MasterShop MS_WebServiceURL='{"Site":"http://$host/epages/Site.soap","MasterShop":"http://$host/epages/MasterShop.soap","eRetail":"http://$host/epages/eRetail.soap","DropShipping":"http://$host/epages/DropShipping.soap"}'
  set_pl /Shops/eRetail MS_WebServiceURL='{"Site":"http://$host/epages/Site.soap","MasterShop":"http://$host/epages/MasterShop.soap","eRetail":"http://$host/epages/eRetail.soap"}'
  set_pl /Shops/DropShipping MS_WebServiceURL='{"Site":"http://$host/epages/Site.soap","MasterShop":"http://$host/epages/MasterShop.soap","DropShipping":"http://$host/epages/DropShipping.soap"}'

  # create passwords
  set_pl /Shops/MasterShop MS_WebServicePassword='{"Site":"wsadmin","MasterShop":"wsadmin","eRetail":"wsadmin","DropShipping":"wsadmin"}'
  set_pl /Shops/eRetail MS_WebServicePassword='{"Site":"wsadmin","MasterShop":"wsadmin","eRetail":"wsadmin"}'
  set_pl /Shops/DropShipping MS_WebServicePassword='{"Site":"wsadmin","MasterShop":"wsadmin","DropShipping":"wsadmin"}'

  exit 0
fi

#-----------------------------------------------------------------------
# install_business_unit
#-----------------------------------------------------------------------

INSTALLDB=$EPAGES_CARTRIDGES/DE_EPAGES/ShopConfiguration/Scripts/installDatabase.pl
DOWNLOAD_CTMPLS=$EPAGES_CARTRIDGES/DE_EPAGES/Presentation/Scripts/downloadCtmpls.pl
INSTALL_CTMPLS=$EPAGES_CARTRIDGES/DE_EPAGES/Presentation/Scripts/installCtmplsFromBase.pl

install_business_unit () {
  local STORE=$1
  local STORETYPE=$2

  export EPAGES_LANGUAGES=${LANGUAGES[$STORETYPE]}
  export EPAGES_LOCALES=$(echo $(get_locales $EPAGES_LANGUAGES))
  ECHO $LINENO "EPAGES_LOCALES='$EPAGES_LOCALES'"
  ECHO $LINENO "EPAGES_LANGUAGES='$EPAGES_LANGUAGES' $PERL $INSTALLDB -storename $STORE"
  if [[ -z $EPAGES_INSTALLATION_MOCK ]] ; then
    $PERL $INSTALLDB -storename $STORE
  fi

  # do not run synccartridges.pl on stores without DE_EPAGES::WebService
  grep -q "^$STORETYPE .*\bDE_EPAGES::WebService\b" $CARTRIDGES_TXT || return 0

  ECHO $LINENO "$PERL $SYNC -storename $STORE"
  if [[ -z $EPAGES_INSTALLATION_MOCK ]] ; then
    $PERL $SYNC -storename $STORE
  fi
  if [[ -n $DROP_CTMPLS ]] ; then
    if grep -q '^cleanfiles_ctmpls:$' makefile ; then
      ECHO $LINENO "make cleanfiles_ctmpls"
      [[ -n $EPAGES_INSTALLATION_MOCK ]] || make cleanfiles_ctmpls
    fi
    return
  fi
  [[ -f $DOWNLOAD_CTMPLS ]] || return
  ECHO $LINENO "$PERL $DOWNLOAD_CTMPLS -storename $STORE -version $VERSION"
  if [[ -z $EPAGES_INSTALLATION_MOCK ]] ; then
    $PERL $DOWNLOAD_CTMPLS -storename $STORE -version $VERSION
  fi
  [[ -f $INSTALL_CTMPLS ]] || return
  ECHO $LINENO "$PERL $INSTALL_CTMPLS -nooutput -storename $STORE -version $VERSION"
  if [[ -z $EPAGES_INSTALLATION_MOCK ]] ; then
    $PERL $INSTALL_CTMPLS -nooutput -storename $STORE -version $VERSION
  fi
}

#-----------------------------------------------------------------------
# install stores
#-----------------------------------------------------------------------

INSERTSTORE=$EPAGES_CARTRIDGES/DE_EPAGES/ShopConfiguration/Scripts/store.pl
REMOTESEARCH=$EPAGES_CARTRIDGES/DE_EPAGES/RemoteSearch/Scripts/remoteSearch.pl
IMPORT=$EPAGES_CARTRIDGES/DE_EPAGES/Object/Scripts/import.pl
APPSTORE_IMPORT=$EPAGES_CARTRIDGES/DE_EPAGES/AppStore/Scripts/importScripts.pl

for STORE in System.conf Site.conf ; do
  [[ -f $EPAGES_CONFIG/Database.d/$STORE ]] || continue
  DATASOURCE=$(get_ini_value $EPAGES_CONFIG/Database.d/$STORE datasource Connection) || continue
  [[ $DATASOURCE =~ ^dbi:mysql ]] || continue
  STORETYPE=$(get_ini_value $EPAGES_CONFIG/Database.d/$STORE StoreType Connection) || continue
  [[ -n $STORETYPE ]] || continue
  install_business_unit ${STORE%.conf} $STORETYPE
done

for STORE in $ALL_STORES ; do
  # datasource must be dbi:mysql
  DATASOURCE=$(get_ini_value $EPAGES_CONFIG/Database.d/$STORE datasource Connection) || continue
  [[ $DATASOURCE =~ ^dbi:mysql ]] || continue
  # StoreType mustn't be empty and not ASPoolDB (already done) or epagesj
  STORETYPE=$(get_ini_value $EPAGES_CONFIG/Database.d/$STORE StoreType Connection) || continue
  [[ -z $STORETYPE || $STORETYPE = ASPoolDB || $STORETYPE = epagesj ]] && continue || true
  STORE=${STORE%.conf}

  # do not run ShopConfiguration/Scripts/store.pl on stores without DE_EPAGES::WebService
  if [[ -z ${ALL_STORETYPES[FlexStore]} ]] && grep -q "^$STORETYPE .*\bDE_EPAGES::WebService\b" $CARTRIDGES_TXT ; then
    ECHO $LINENO "$PERL $INSERTSTORE -action insert -storename $STORE"
    if [[ -z $EPAGES_INSTALLATION_MOCK ]] ; then
      $PERL $INSERTSTORE -action insert -storename $STORE
    fi
  fi

  install_business_unit $STORE $STORETYPE

  # do not run remoteSearch.pl on stores without DE_EPAGES::RemoteSearch
  if grep -q "^$STORETYPE .*\bDE_EPAGES::RemoteSearch\b" $CARTRIDGES_TXT ; then
    ECHO $LINENO "$PERL $REMOTESEARCH -storename $STORE -state on"
    if [[ -z $EPAGES_INSTALLATION_MOCK ]] ; then
      $PERL $REMOTESEARCH -storename $STORE -state on || true
    fi
  fi

  if grep -q "^$STORETYPE .*\bDE_EPAGES::AppStore\b" $CARTRIDGES_TXT ; then
    MATe=$(get_ini_value $EPAGES_CONFIG/epages.conf TokenMATe "DE_EPAGES::AppStore") || continue
    [[ -n $MATe ]] || continue
    ECHO $LINENO "$PERL $APPSTORE_IMPORT -storename $STORE -nooutput -randomsleep 5"
    if [[ -z $EPAGES_INSTALLATION_MOCK ]] ; then
      $PERL $APPSTORE_IMPORT -storename $STORE -nooutput -randomsleep 5
    fi
  fi
done

#-----------------------------------------------------------------------
# install storetypes
#-----------------------------------------------------------------------

for STORETYPE in ${!ALL_STORETYPES[@]} ; do
  # do not install storetypes on storetypes without DE_EPAGES::(Shop|Store)
  egrep -q "^$STORETYPE .*\bDE_EPAGES::(Shop|Store)\b" $CARTRIDGES_TXT || continue
  ECHO $LINENO "make install_storetype STORE_TYPE=$STORETYPE"
  [[ -z $EPAGES_INSTALLATION_MOCK ]] || continue
  make install_storetype STORE_TYPE=$STORETYPE
done

#-----------------------------------------------------------------------
# set Providers/Distributor
#-----------------------------------------------------------------------

export PROVIDER_PATH=${PROVIDER_PATH:-/Providers/Distributor}
XMLDIR=$EPAGES_CARTRIDGES/DE_EPAGES/ShopConfiguration/Database/XML

if [[ -f $EPAGES_CONFIG/Database.d/Site.conf ]] ; then
  for xml in Distributor DistributorStore ; do
    # don't use DistributorStore for FlexSite
    [[ -z ${ALL_STORETYPES[FlexSite]} || $xml = Distributor ]] || continue
    ECHO $LINENO "$PERL $IMPORT -storename Site $XMLDIR/$xml.xml"
    [[ -z $EPAGES_INSTALLATION_MOCK ]] || continue
    $PERL $IMPORT -storename Site $XMLDIR/$xml.xml
  done
  ECHO $LINENO "$PERL $IMPORT -storename Site -path $PROVIDER_PATH $XMLDIR/DistributorShopTypes.xml"
  if [[ -z $EPAGES_INSTALLATION_MOCK ]] ; then
    $PERL $IMPORT -storename Site -path $PROVIDER_PATH $XMLDIR/DistributorShopTypes.xml
  fi

  for STORE in $ALL_STORES ; do
    # StoreType mustn't be empty and not ASPoolDB (already done) or epagesj
    STORETYPE=$(get_ini_value $EPAGES_CONFIG/Database.d/$STORE StoreType Connection) || continue
    [[ -n $STORETYPE ]] || continue
    STORE=${STORE%.conf}
    # do not run import.pl for stores without DE_EPAGES::Styles
    if grep -q "^$STORETYPE .*\bDE_EPAGES::Styles\b" $CARTRIDGES_TXT ; then
      STOREIMPORT_XML=$(mktempdir epages-appsrv)/$STORE-import.xml
      cat <<EOF > $STOREIMPORT_XML
<?xml version="1.0" encoding="iso-8859-1"?>
<epages>
  <Object Alias="Providers" reference="1">
    <Provider Alias="Distributor" reference="1">
      <ProviderStore Store="$STORE" delete="1" />
    </Provider>
  </Object>
</epages>
EOF
      ECHO $LINENO "$PERL $IMPORT -storename Site -path / $STOREIMPORT_XML"
      if [[ -z $EPAGES_INSTALLATION_MOCK ]] ; then
	$PERL $IMPORT -storename Site -path / $STOREIMPORT_XML
      fi
    fi
  done
fi

#-----------------------------------------------------------------------
# install demoshop
#-----------------------------------------------------------------------

export DEMOSHOP_ALIAS=${DEMOSHOP_ALIAS:-DemoShop}
export DEMOSHOP_SHOPTYPE=${DEMOSHOP_SHOPTYPE:-Demo}
export PROVIDER_USER=${PROVIDER_USER:-admin}
export PROVIDER_PASSWD=${PROVIDER_PASSWD:-admin}
export STORE=${DEMOSHOP_STORE:-Store}

if [[ -z $EPAGES_IGNORE_DEMOSHOP ]] && echo $ALL_STORES | grep -q "\b$STORE\b" ; then
  ECHO $LINENO "make install_demoshop STORE=$STORE"
  [[ -n $EPAGES_INSTALLATION_MOCK ]] || make install_demoshop STORE=$STORE || true
fi

#-----------------------------------------------------------------------
# flex
#-----------------------------------------------------------------------

if [[ -n ${ALL_STORETYPES[FlexStore]} ]] ; then
  ECHO $LINENO "make FlexStore"
  [[ -n $EPAGES_INSTALLATION_MOCK ]] || make FlexStore || true
fi

if [[ -n ${ALL_STORETYPES[FlexSite]} ]] ; then
  ECHO $LINENO "make FlexSite"
  [[ -n $EPAGES_INSTALLATION_MOCK ]] || make FlexSite || true
fi

#-----------------------------------------------------------------------
# final tasks
#-----------------------------------------------------------------------

ECHO $LINENO "make start_cron"
[[ -n $EPAGES_INSTALLATION_MOCK ]] || make start_cron || true

exit 0
