#!/bin/bash

#=======================================================================
# postinstall/j.sh
#=======================================================================

if [ ! -w /etc/passwd ] ; then
  echo "$0: only 'root' can execute this program"
  exit 1
fi

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  [[ -n $EPAGES_INSTALLATION_MOCK ]] && echo "$@" || echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

#-----------------------------------------------------------------------
# define variables and functions
#-----------------------------------------------------------------------

J_CONFIGFILE=$1

EPAGESJ_CONFIG_ENV=$EPAGESJ_CONFIG

FUN_DIR=$(cd $(dirname $0)/../functions && pwd)

. $FUN_DIR/fun-epages.sh
export fun_epages_sourced
export -f find_ini_config get_ini_value set_ini_value etc_default
. $FUN_DIR/fun-j.sh
export fun_j_sourced
export -f start_epagesj
. $FUN_DIR/fun-java.sh
export fun_java_sourced
export -f start_mongodb java_executable jre_home mongod_name test_version

. $(etc_default)

INI_CONFIG=$(find_ini_config) || exit 1

EPAGES=${EPAGES:-/srv/epages/eproot}
EPAGESJ=${EPAGESJ:-$EPAGES/j}
EPAGESJ_CONFIG=${EPAGESJ_CONFIG:-$EPAGESJ/conf}
EPAGESJ_CONFIG=${EPAGESJ_CONFIG_ENV:-$EPAGESJ_CONFIG}

#-----------------------------------------------------------------------
# create ssh key pair to connect to the epages-j console
#-----------------------------------------------------------------------

set -e

pushd $EPAGESJ_CONFIG >/dev/null
unset SECRET_KEY_UPDATED

if type ssh-keygen >/dev/null 2>&1 && [[ -f $J_CONFIGFILE ]] ; then
  if SECRET_KEY=$(get_ini_value $J_CONFIGFILE secret.key application-secret-key) ; then

    # create ssh key pair unless exists
    if [[ -n $EPAGES_INSTALLATION_MOCK ]]  ; then
      id_rsa_found=
    elif ls id_rsa id_rsa.pub >/dev/null 2>&1 ; then
      id_rsa_found=1
    else
      ssh-keygen -f id_rsa -P ''
      id_rsa_found=
    fi
    if [[ ! -f authorized_key.pem || -z $id_rsa_found ]] ; then
      ECHO $LINENO "openssl rsa -in id_rsa -pubout -outform pem > authorized_key.pem"
      if [[ -z $EPAGES_INSTALLATION_MOCK ]]  ; then
	openssl rsa -in id_rsa -pubout -outform pem > authorized_key.pem
      fi
    fi

    # update secret.key in application-secret-key.conf
    ASKS=application-secret-key.sample.conf
    if [[ -z $EPAGES_INSTALLATION_MOCK && ! -f ${ASKS//.sample/} && -f $ASKS ]] ; then
      cp -p $ASKS ${ASKS//.sample/}
    fi
    touch ${ASKS//.sample/}
    ECHO $LINENO "set_ini_value --overwrite ${ASKS//.sample/} secret.key '$SECRET_KEY'"
    if [[ -z $EPAGES_INSTALLATION_MOCK ]]  ; then
      set_ini_value --overwrite ${ASKS//.sample/} secret.key "$SECRET_KEY"
    fi
    SECRET_KEY_UPDATED=1
  fi
fi

#-----------------------------------------------------------------------
# wait_for_j
#-----------------------------------------------------------------------

set +e

wait_for_j() {
  local WAIT=$1
  EPAGESJ_HOST=${EPAGESJ_HOST:-localhost}
  EPAGESJ_PORT=${EPAGESJ_PORT:-8088}

  local RESULT=0
  local START=$(date +%s)
  ECHO $LINENO curl http://$EPAGESJ_HOST:$EPAGESJ_PORT/rs/
  echo -n .
  while sleep 3 ; do
    echo -n .
    # stop after $WAIT minutes
    if [[ $(($(date +%s) - $START)) -gt $(($WAIT * 60)) ]] ; then
      JVERSION=$(curl http://$EPAGESJ_HOST:$EPAGESJ_PORT/rs/ 2>/dev/null) || RESULT=$JVERSION
      [[ $JVERSION =~ revision ]] || RESULT=$JVERSION
      break
    fi
    JVERSION=$(curl http://$EPAGESJ_HOST:$EPAGESJ_PORT/rs/ 2>/dev/null) || continue
    [[ $JVERSION =~ revision ]] || continue
    break
  done
  echo
  [[ $RESULT = 0 ]] && return 0 || { echo $RESULT ; return 1; }
}

#-----------------------------------------------------------------------
# start epages-j
#-----------------------------------------------------------------------

if [[ -f /etc/init.d/epages6 ]] ; then
  ECHO $LINENO "/etc/init.d/epages6 perm j"
  [[ -n $EPAGES_INSTALLATION_MOCK ]] || /etc/init.d/epages6 perm j
else
  ECHO $LINENO "chown -R epagesj:epages '$EPAGESJ'"
  [[ -n $EPAGES_INSTALLATION_MOCK ]] || chown -R epagesj:epages "$EPAGESJ"
fi

[[ -n $EPAGES_INSTALLATION_MOCK || ! -f id_rsa.pub ]] || chmod a+r id_rsa.pub

if [[ $J_CONFIGFILE =~ /standby ]] ; then
  ECHO $LINENO "EPAGES_SPEC_SKIP_RESTART=1 (/standby in $J_CONFIGFILE)"
  exit 0
fi

if [[ -z $EPAGES_INSTALLATION_MOCK ]] ; then
  ECHO $LINENO start_mongodb
  start_mongodb
  ECHO $LINENO start_epagesj epages-solr epages-solr-slave
  start_epagesj epages-solr epages-solr-slave

  if [[ -f /var/epages/bin/mongo_enable_auth.sh ]] && /var/epages/bin/mongo_can_enable_auth.sh ; then
    sleep 2
    ECHO $LINENO /var/epages/bin/mongo_enable_auth.sh
    /var/epages/bin/mongo_enable_auth.sh
  fi

  export SKIP_MONGO_ENABLE_AUTH=1
  ECHO $LINENO SKIP_MONGO_ENABLE_AUTH=1 start_epagesj epagesj
  start_epagesj epagesj
  wait_for_j 4
fi

exit 0
