#!/bin/bash

#=======================================================================
# postinstall/mysqld.sh
#=======================================================================

if [ ! -w /etc/passwd ] ; then
  echo "$0: only 'root' can execute this program"
  exit 1
fi

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  [[ -n $EPAGES_INSTALLATION_MOCK ]] && echo "$@" || echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

#-----------------------------------------------------------------------
# define variables and functions
#-----------------------------------------------------------------------

MYSQLD_CONFIGFILE=$1

# do nothing if standby
if [[ $MYSQLD_CONFIGFILE =~ /standby ]] ; then
  ECHO $LINENO "EPAGES_SPEC_SKIP_RESTART=1 (/standby in $MYSQLD_CONFIGFILE)"
  exit 0
fi

PERL_ENV=$PERL
EPAGES_CONFIG_ENV=$EPAGES_CONFIG

FUN_DIR=$(cd $(dirname $0)/../functions && pwd)

. $FUN_DIR/fun-epages.sh
export fun_epages_sourced
export -f find_ini_config get_ini_value set_ini_value etc_default
. $FUN_DIR/fun-mysql.sh
export fun_mysql_sourced
export -f set_mysql_root_password reset_mysql_root_password guess_mysql_root_password run_sql

. /etc/default/epages6

INI_CONFIG=$(find_ini_config) || exit 1

EPAGES=${EPAGES:-/srv/epages/eproot}
EPAGES_CONFIG=${EPAGES_CONFIG:-/srv/epages/eproot/Shared/Config}
EPAGES_CONFIG=${EPAGES_CONFIG_ENV:-$EPAGES_CONFIG}
DATABASE_D=$EPAGES_CONFIG/Database.d
EPAGES_CARTRIDGES=${EPAGES_CARTRIDGES:-$EPAGES/Cartridges}
PERL=${PERL:-$EPAGES/Perl/bin/perl}
PERL=${PERL_ENV:-$PERL}

#------------------------------------------------------------------------------
# define root database
#------------------------------------------------------------------------------

set -e

if echo $EPAGES_ROOT_DATABASES | grep -q '\bBackup\b' ; then
  # if exist use Backup
  ROOT=Backup
elif echo $EPAGES_ROOT_DATABASES | grep -q "\bBackup-$(hostname --short)\b" ; then
  # else if exist use Backup-$(hostname --short)
  ROOT=Backup-$(hostname --short)
elif [[ -n $EPAGES_ROOT_DATABASES ]] ; then
  # else if not empty use first item
  ROOT=$(echo $EPAGES_ROOT_DATABASES)
  ROOT=${ROOT%% *}
elif [[ -f $DATABASE_D/Backup-$(hostname --short).conf ]] ; then
  # else if exist Backup.conf use Backup
  ROOT=Backup-$(hostname --short)
elif [[ -f $DATABASE_D/Backup.conf ]] ; then
  # else if exist Backup.conf use Backup
  ROOT=Backup
else
  # else use Backup-$(hostname --short)
  ROOT=Backup-$(hostname --short)
fi

# die unless perl scripts exist
pushd $EPAGES_CARTRIDGES/DE_EPAGES >/dev/null
CREATEDB_PL=$EPAGES_CARTRIDGES/DE_EPAGES/Database/Scripts/createDB.pl
STORECONF_PL=$EPAGES_CARTRIDGES/DE_EPAGES/ShopConfiguration/Scripts/createStoreConf.pl
RUN_SQL_PL=$EPAGES_CARTRIDGES/DE_EPAGES/Database/Scripts/RunSQL.pl
ls $STORECONF_PL $CREATEDB_PL $RUN_SQL_PL >/dev/null

#------------------------------------------------------------------------------
# change database
#------------------------------------------------------------------------------

# set_mysql_root_password is executed in either:
# - postinstall of package:
#   /var/epages/bin/SetupMySQL.sh -> EPAGES_MYSQL_ROOT_SCRIPT=SetupMySQL.sh
# - after package installation (here):
#   /var/epages/postinstall/mysqld.sh -> EPAGES_MYSQL_ROOT_SCRIPT=postinstall/mysqld.sh

if [[ $EPAGES_MYSQL_ROOT_SCRIPT != postinstall/mysqld.sh ]] ; then
  # if root login fails then set_mysql_root_password here
  ECHO $LINENO "${PERL_ENV:-$PERL} $RUN_SQL_PL -storename $ROOT -nooutput -sql 'SELECT @@version'"
  ${PERL_ENV:-$PERL} $RUN_SQL_PL -storename $ROOT -nooutput -sql 'SELECT @@version' >/dev/null 2>&1
  [[ $? = 0 ]] || EPAGES_MYSQL_ROOT_SCRIPT=postinstall/mysqld.sh
fi

if [[ $EPAGES_MYSQL_ROOT_SCRIPT = postinstall/mysqld.sh ]] ; then
  MYSQL_PASS=${INSTALL_EPAGES_MYSQL_PASS-$EPAGES_INSTALL_MYSQL_PASS}
  MY_CNF=${INSTALL_EPAGES_MY_CNF-$EPAGES_INSTALL_MY_CNF}
  export EPAGES_INSTALL_MY_CNF=$MY_CNF
  export EPAGES_INSTALL_MYSQL_PASS=$MYSQL_PASS

  # set_mysql_root_password
  ECHO $LINENO "set_mysql_root_password XXXXXX"
  set_mysql_root_password "$MYSQL_PASS" || true

  # delete empty users
  STATEMENT="DELETE FROM mysql.user WHERE user = ''; FLUSH PRIVILEGES;"
  ECHO $LINENO "run_sql -p XXXXXX '$STATEMENT'"
  run_sql -p "$MYSQL_PASS" "$STATEMENT" || exit 1

  # grant all privileges on *.* to root
  STATEMENT="GRANT ALL ON *.* TO 'root' %s WITH GRANT OPTION;\n"
  [[ -z $MYSQL_PASS ]] && IDENTIFIED_BY= || IDENTIFIED_BY="IDENTIFIED BY '$MYSQL_PASS'"
  ECHO $LINENO "run_sql -p XXXXXX '$STATEMENT'"
  run_sql -p "$MYSQL_PASS" "$(printf "$STATEMENT" "$IDENTIFIED_BY")" || exit 1

  # database engine must be InnoDB
  STATEMENT="SHOW ENGINES;"
  ECHO $LINENO "run_sql -p XXXXXX '$STATEMENT'"
  if ! run_sql -p "$MYSQL_PASS" "$STATEMENT" | grep -Pq '^InnoDB\s+(YES|DEFAULT)\s+' ; then
    echo "ERROR: SHOW ENGINES does not show InnoDB."
    exit 1
  fi

  # die unless root connection works
  ECHO $LINENO "${PERL_ENV:-$PERL} $RUN_SQL_PL -storename $ROOT -nooutput -sql 'SELECT @@version'"
  ${PERL_ENV:-$PERL} $RUN_SQL_PL -storename $ROOT -nooutput -sql 'SELECT @@version'
fi

#------------------------------------------------------------------------------
# create databases
#------------------------------------------------------------------------------

# unluckily, shell functions reset $EPAGES_CONFIG
EPAGES_CONFIG=${EPAGES_CONFIG_ENV:-$EPAGES_CONFIG}

ECHO $LINENO "EPAGES_CONFIG=$EPAGES_CONFIG"

if [[ -f $MYSQLD_CONFIGFILE ]] ; then
  $INI_CONFIG -f $MYSQLD_CONFIGFILE -G
else
  echo Site$'\n'Store$'\n'System$'\n'ASPoolDB$'\n'epagesj
fi | while read STORE ; do
  [[ -z $STORE ]] && continue || true
  [[ $STORE =~ ' ' ]] && continue || true

  DB_CONF=$EPAGES_CONFIG/Database.d/$STORE.conf
  if [[ -f $DB_CONF ]] ; then
    # ignore backup connections
    [[ mysql != $(get_ini_value $DB_CONF Database Connection) ]] || continue
    # update Backup= (if found and outdated)
    if BACKUP=$(get_ini_value $DB_CONF Backup Connection) ; then
      if [[ $BACKUP != $ROOT ]] ; then
	ECHO $LINENO set_ini_value --overwrite $DB_CONF Backup $ROOT Connection
	set_ini_value --overwrite $DB_CONF Backup $ROOT Connection
      fi
    fi
  fi

  ECHO $LINENO "${PERL_ENV:-$PERL} $CREATEDB_PL -storename $STORE"
  [[ -z $EPAGES_INSTALLATION_MOCK ]] || continue
  ${PERL_ENV:-$PERL} $CREATEDB_PL -storename $STORE
done

set +e

ECHO $LINENO "/etc/init.d/epages6 perm admin"
/etc/init.d/epages6 perm admin >/dev/null 2>&1 

exit 0
