#!/srv/epages/eproot/Perl/bin/perl

use strict;
use Getopt::Long;
use Config::IniFiles;
use File::Basename qw ( dirname );
use Sys::Hostname qw( hostname );

use DE_EPAGES::Core::API::PerlTools qw(GetIPAddresses);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::File qw ( AbsolutePath RealPath WriteFile );

sub firstElem {
    my ($Item) = @_;
    return undef unless defined $Item;
    return $Item->[0] if ref $Item eq 'ARRAY';
    return $Item;
}

sub absolutePath {
    my ($Path, $RelativeTo) = @_;

    return RealPath(AbsolutePath($Path)) if $Path =~ m,^/,;
    return RealPath(AbsolutePath($Path)) unless $RelativeTo;

    $RelativeTo = RealPath(AbsolutePath($RelativeTo));
    return RealPath(AbsolutePath($Path, dirname($RelativeTo)));
}

sub getIPFromEpagesInstallConf {
    my ($EpagesHash, $EpagesSection, $HostList) = @_;
    my $IP;

    push @$HostList, $EpagesHash->{$EpagesSection}{hostname} if
        defined $EpagesHash->{$EpagesSection}{hostname};

    if ($EpagesHash->{$EpagesSection}{ip}) {
        $IP = $EpagesHash->{$EpagesSection}{ip};
        push @$HostList, $IP;
    } elsif ($EpagesHash->{$EpagesSection}{hostname}) {
        my $IpHash = GetIPAddresses($EpagesHash->{$EpagesSection}{hostname});
        $IP = $IpHash->{4}[0] // $EpagesHash->{$EpagesSection}{hostname};
        push @$HostList, @{$IpHash->{4}} if defined $IpHash->{4};
    } else {
        # replace localhost by hostname()
        $EpagesSection = hostname() if $EpagesSection eq 'localhost';

        my $IpHash = GetIPAddresses($EpagesSection);
        $IP = $IpHash->{4}[0] // $EpagesSection;
        push @$HostList, @{$IpHash->{4}} if defined $IpHash->{4};
        push @$HostList, $EpagesSection if defined $IpHash->{4};
    }
    return $IP;
}

sub getServerConfig {
    my ($EpagesInstallConf) = @_;
    my ($CacheServer, $RequestRouter, $Pool, $Global, $URLRewrite) = ({},{},{},{},{});

    tie my %EpagesHash, 'Config::IniFiles', (-file => $EpagesInstallConf)
        or die "$EpagesInstallConf: $!";

    my $HostList = [];
    my ($FirstAppSrvIP, $FirstAscSrvIP, $FirstReqSrvIP);
    foreach my $EpagesSection (keys %EpagesHash) {
        next if $EpagesSection =~ /\s/;
        my $IP = getIPFromEpagesInstallConf(\%EpagesHash, $EpagesSection, $HostList);
        if ($EpagesHash{$EpagesSection}{ascsrv}) {
            my $ServerConf = absolutePath(firstElem($EpagesHash{$EpagesSection}{ascsrv}),
                                          $EpagesInstallConf);
            $FirstAscSrvIP //= $IP;
            defineCacheServer($CacheServer, $IP, $ServerConf);
        }
        if ($EpagesHash{$EpagesSection}{reqsrv}) {
            my $ServerConf = absolutePath(firstElem($EpagesHash{$EpagesSection}{reqsrv}),
                                          $EpagesInstallConf);
            $FirstReqSrvIP //= $IP;
            defineRequestRouter($RequestRouter, $IP, $ServerConf);
        }
        if ($EpagesHash{$EpagesSection}{appsrv}) {
            my $ServerConf = absolutePath(firstElem($EpagesHash{$EpagesSection}{appsrv}),
                                          $EpagesInstallConf);
            $FirstAppSrvIP //= $IP;
            definePool($Pool, $IP, $ServerConf, $Global, $URLRewrite);
        }
    }

    # define DefaultPool if no pool in configuration found
    if (not scalar keys %$Pool and $FirstAppSrvIP) {
        defineDefaultPool($Pool, $FirstAppSrvIP);
    }
    # define DefaultRequestRouter if no request router in configuration found
    if (not scalar keys %$RequestRouter and $FirstReqSrvIP) {
        defineDefaultRequestRouter($RequestRouter, $FirstReqSrvIP);
    }
    # define DefaultCacheServer if no cache server in configuration found
    if (not scalar keys %$CacheServer and $FirstAscSrvIP) {
        defineDefaultCacheServer($CacheServer, $FirstAscSrvIP);
    }

    fixTempNames($RequestRouter, $Pool);

    $Pool->{ExceptionHost} = join(' ', @$HostList) if scalar @$HostList;
    return ($CacheServer, $RequestRouter, $Pool, $Global, $URLRewrite);
}

sub defineDefaultPool {
    my ($Pool, $IP) = @_;
    $Pool->{"tempname $IP"}{$IP}{ports} = '10045-10046';
    $Pool->{"tempname $IP"}{$IP}{maxmemory} = 250;
    $Pool->{maxmemory} //= 250;
}

sub defineDefaultRequestRouter {
    my ($RequestRouter, $IP) = @_;
    $RequestRouter->{"tempname $IP"}{host} = $IP;
    $RequestRouter->{"tempname $IP"}{monitor_port} = 10041;
    $RequestRouter->{"tempname $IP"}{mc_port} = 10042;
    $RequestRouter->{"tempname $IP"}{port} = 10043;
    $RequestRouter->{max_cache_items} //= 2000;
}

sub defineDefaultCacheServer {
    my ($CacheServer, $IP) = @_;
    $CacheServer->{$IP}{host} = $IP;
    $CacheServer->{$IP}{port} = '17277';
}

sub nextRR {
    my ($RequestRouter, $TempName) = @_;

    # 1. find matching RequestRouter
    if (defined $RequestRouter->{$TempName}) {
        return $RequestRouter->{$TempName}{name} if
            defined $RequestRouter->{$TempName}{name};
    }

    # 2. find DefaultRequestRouter
    my $DefaultRR = 'DefaultRequestRouter';
    return $DefaultRR if defined $RequestRouter->{$DefaultRR};
    foreach (keys %$RequestRouter) {
        next if $_ eq 'max_cache_items';
        next unless defined $RequestRouter->{$_}{name};
        return $DefaultRR if $RequestRouter->{$_}{name} eq $DefaultRR;
    }

    # 3. find first available RequestRouter
    foreach (sort keys %$RequestRouter) {
        return $_ unless /^(tempname |max_cache_items$)/;
    }
    foreach (sort keys %$RequestRouter) {
        next if $_ eq 'max_cache_items';
        return $RequestRouter->{$_}{name} if defined $RequestRouter->{$_}{name};
    }
}

sub fixTempNames {
    my ($RequestRouter, $Pool) = @_;

    # fix RequestRouter names
    my %RequestRouterNames;
    $RequestRouterNames{$_} = 1 foreach keys %$RequestRouter;
    my $NextRequestRouter = 'DefaultRequestRouter';

    foreach my $TempName (keys %$RequestRouter) {
        next unless $TempName =~ /^tempname /;
        if (defined $RequestRouterNames{$NextRequestRouter}) {
            my $Count = 1;
            $Count++ while (defined $RequestRouterNames{"RequestRouter$Count"});
            $NextRequestRouter = "RequestRouter$Count";
        }
        $RequestRouterNames{$NextRequestRouter} = 1;
        $RequestRouter->{$TempName}{name} = $NextRequestRouter;
    }

    # fix Pool names
    my %PoolNames;
    $PoolNames{$_} = 1 foreach keys %$Pool;
    my $NextPool = 'DefaultPool';

    foreach my $TempName (keys %$Pool) {
        next unless $TempName =~ /^tempname /;
        if (defined $PoolNames{$NextPool}) {
            my $Count = 1;
            $Count++ while (defined $PoolNames{"Pool$Count"});
            $NextPool = "Pool$Count";
        }
        $PoolNames{$NextPool} = 1;
        $Pool->{$TempName}{name} = $NextPool;
        $Pool->{$TempName}{requestrouters} .= ','. nextRR($RequestRouter, $TempName);
    }
}

sub defineCacheServer {
    my ($CacheServer, $IP, $ServerConf)  = @_;

    # $CacheServer->{ID}{host};
    # $CacheServer->{ID}{port};
    # $CacheServer->{ID}{bind};
    # $CacheServer->{ID}{enable_cache};

    if (not -s $ServerConf and $ServerConf !~ /standby/) {
        defineDefaultCacheServer($CacheServer, $IP);
        return;
    }

    tie my %ServerHash, 'Config::IniFiles', (-file => $ServerConf) or die "$ServerConf: $!";
    foreach my $Section (keys %ServerHash) {
        my $Host = firstElem($ServerHash{$Section}{host}) // $IP;
        $CacheServer->{"$Host $Section"}{host} = $Host;
        $CacheServer->{"$Host $Section"}{port} = $Section;
        $CacheServer->{"$Host $Section"}{$_} =
            firstElem($ServerHash{$Section}{$_}) foreach qw(enable_cache bind);
    }
}

sub defineRequestRouter {
    my($RequestRouter, $IP, $ServerConf) = @_;

    # $RequestRouter->{ID}{name};
    # $RequestRouter->{ID}{host};
    # $RequestRouter->{ID}{monitor_port};
    # $RequestRouter->{ID}{mc_port};
    # $RequestRouter->{ID}{port};
    # $RequestRouter->{max_cache_items};

    if (not -s $ServerConf and $ServerConf !~ /standby/) {
        defineDefaultRequestRouter($RequestRouter, $IP);
        return;
    }

    tie my %ServerHash, 'Config::IniFiles', (-file => $ServerConf) or die "$ServerConf: $!";
    foreach my $Section (keys %ServerHash) {
        next unless scalar keys %{$ServerHash{$Section}};
        my $Host = firstElem($ServerHash{$Section}{host});
        $RequestRouter->{$Section}{host} = $Host // $IP;
        $RequestRouter->{$Section}{$_} =
            firstElem($ServerHash{$Section}{$_})
            foreach qw(monitor_port mc_port port max_cache_items);
        next unless defined $ServerHash{$Section}{max_cache_items};
        $RequestRouter->{max_cache_items} //=
            firstElem($ServerHash{$Section}{max_cache_items});
        $RequestRouter->{max_cache_items} = firstElem($ServerHash{$Section}{max_cache_items})
            if $RequestRouter->{max_cache_items} <
            firstElem($ServerHash{$Section}{max_cache_items});
    }
}

sub definePool {
    my ($Pool, $IP, $ServerConf, $Global, $URLRewrite) = @_;

    # $Pool->{ExceptionHost};
    # $Pool->{maxmemory};
    # $Pool->{ID}{host}{ports};
    # $Pool->{ID}{host}{maxmemory};
    # $Pool->{ID}{host}{priority};
    # $Pool->{ID}{name};
    # $Pool->{ID}{requestrouters};
    # $Pool->{ID}{servers_per_site};
    # $Pool->{ID}{servers_per_client};
    # $Pool->{ID}{servers_per_extension};

    if (not -s $ServerConf and $ServerConf !~ /standby/) {
        defineDefaultPool($Pool, $IP);
        return;
    }

    tie my %ServerHash, 'Config::IniFiles', (-file => $ServerConf) or die "$ServerConf: $!";
    foreach my $Section (keys %ServerHash) {
        next unless scalar keys %{$ServerHash{$Section}};
        if ($Section eq 'GLOBAL') {
            $Global->{$_} = firstElem($ServerHash{$Section}{$_})
                foreach keys %{$ServerHash{$Section}};
            next;
        }
        if ($Section eq 'URLRewrite') {
            $URLRewrite->{$_} = firstElem($ServerHash{$Section}{$_})
                foreach keys %{$ServerHash{$Section}};
            next;
        }
        my $Host = $ServerHash{$Section}{host};
        $Host //= $IP;
        $Pool->{$Section}{$Host}{$_} =
            firstElem($ServerHash{$Section}{$_}) foreach qw(ports maxmemory priority);
        $Pool->{$Section}{"servers_per_$_"} =
            firstElem($ServerHash{$Section}{$_}) foreach qw(site client extension);
        $Pool->{$Section}{requestrouters} .= ','.
            firstElem($ServerHash{$Section}{requestrouters});
        next unless defined $ServerHash{$Section}{maxmemory};
        $Pool->{maxmemory} //= firstElem($ServerHash{$Section}{maxmemory});
        $Pool->{maxmemory} = firstElem($ServerHash{$Section}{maxmemory})  if
            $Pool->{maxmemory} < firstElem($ServerHash{$Section}{maxmemory});
    }
}

sub setWebInterfaceConfDefaults {
    my ($Global, $URLRewrite, $RequestRouter, $Pool) = @_;

    my $ExceptionHost = $Pool->{ExceptionHost} // 'localhost';
    my $MaxMemory = $Pool->{maxmemory} // 250;
    my $MaxCacheItems = $RequestRouter->{max_cache_items} // 1000;

    if (not scalar keys %$Global) {
        $Global = {
            COOKIES => 'SessionID,PreviewStyle,UserGUID,PopUp,ClassicView,ShopInit,PerformanceBar,MerchantSF,SecToken',
            FREEREQUESTPERIOD => 5,
            KEEPALIVE => 0,
            MANUAL_MONITOR_PRIORITY => 'NORMAL',
            MAXCACHEITEMS => $MaxCacheItems,
            MAXCONTENTLENGTH => 102400,
            MAXMEMORY => $MaxMemory,
            MAXREQUEST => 10000,
            MONITOR_PRIORITY => 'ABOVE_NORMAL',
            MONITOR_TIMEOUT => 15,
            OPTIONS => '-w',
            PRIORITY => 'ABOVE_NORMAL',
            RETRIES => 30,
            TIMEOUT => 30,
        };
    }

    if (not scalar keys %$URLRewrite) {
        $URLRewrite = {
            BaseURI => '/',
            ExceptionHost => $ExceptionHost,
            ExceptionPrefix => '/epages/ /WebRoot/ /rs/ /robots.txt /favicon.ico /monitor.pl /ShopExport/',
            ExceptionRegex => '',
            StoreClosedConf => '',
            StoreClosedPage => '',
            StoreClosedPrefix => '',
            epages => '/epages/',
        };
    }
    return ($Global, $URLRewrite);
}

sub writeWebInterfaceConf {
    my ($Global, $UrlRewrite) = @_;
    my $ConfigFile = "$ENV{EPAGES_CONFIG}/WebInterface.conf";
    my $Content;

    if (scalar keys %$Global) {
        $Content .= "[GLOBAL]\n";
        $Content .= "$_=$Global->{$_}\n" foreach sort keys %$Global;
        $Content .= "\n";
    }

    if (scalar keys %$UrlRewrite) {
        $Content .= "[URLRewrite]\n";
        $Content .= "$_=$UrlRewrite->{$_}\n" foreach sort keys %$UrlRewrite;
        $Content .= "\n";
    }
    WriteFile($ConfigFile, \$Content, { create_directory=>1 });
}

sub appServerList {
    my ($aIP) = @_;

    my %IPs;
    foreach my $IPRange (@$aIP) {
        # $IPRange = 1,3-6
        foreach my $Item (split(/\s*,\s*/, $IPRange)) {
            if ($Item !~ /-/) {
                # $Item = 1
                $IPs{$Item} = 1;
                next;
            }
            # $Item = 3-6
            my ($Start, $End) = split(/\s*-\s*/, $Item);
            # $Start = 3 ; $End = 6
            do {
                $IPs{$Start++} = 1;
            } until ($Start > $End);
        }
    }

    my $appServerList;
    $appServerList .= ",$_" foreach sort { $a <=> $b } keys %IPs;
    $appServerList =~ s/^,//;
    return $appServerList;
}

sub appendWebInterfaceConfSections {
    my ($RequestRouter, $Pool) = @_;
    my $ConfigFile = "$ENV{EPAGES_CONFIG}/WebInterface.conf";
    my ($Content, %RRhost, %RRport, %ASport, %IPs);

    # add RequestRouter IPs to $RRport
    foreach my $ID (keys %$RequestRouter) {
        next if $ID eq 'max_cache_items';
        my $RR = $RequestRouter->{$ID}{host};
        $IPs{$RR} = 1;
        my $Name = $RequestRouter->{$ID}{name} // $ID;
        $RRhost{$Name} = $RR;
        $RRport{$RR}{$_} = $RequestRouter->{$ID}{$_} foreach qw(port mc_port monitor_port);
    }

    # add AppServer IPs to $ASport and 'dedicated' to $RRport
    foreach my $Name (keys %$Pool) {
        next if $Name ~~ [qw(ExceptionHost maxmemory)];
        my $Dedicated;
        foreach my $AS (sort keys %{$Pool->{$Name}}) {
            next if $AS =~ /^servers_per_/ or $AS ~~ [qw(name requestrouters)];
            next unless defined $Pool->{$Name}{$AS}{ports};
            $Dedicated .= ",$AS";
            $IPs{$AS} = 1;
            push @{$ASport{$AS}}, $Pool->{$Name}{$AS}{ports};
        }
        next unless defined $Pool->{$Name}{requestrouters};
        $Dedicated =~ s/^,//;
        foreach my $RR (split(/\s*,\s*/, $Pool->{$Name}{requestrouters})) {
            next unless $RR;
            $RRport{$RRhost{$RR}}{dedicated} = $Dedicated;
        }
    }

    my %IPMask;
    foreach my $IP (sort keys %IPs) {
        $Content .= "[$IP]\n";
        if (defined $ASport{$IP}) {
            $Content .= 'APPLICATIONSERVER='. appServerList($ASport{$IP})."\n";
        }
        if (defined $RRport{$IP}) {
            $Content .= "REQUESTROUTER=$RRport{$IP}{port}\n"
                if defined $RRport{$IP}{port};
            $Content .= "MESSAGECENTER=$RRport{$IP}{mc_port}\n"
                if defined $RRport{$IP}{mc_port};
            $Content .= "MONITOR=$RRport{$IP}{monitor_port}\n"
                if defined $RRport{$IP}{monitor_port};
            $Content .= "DEDICATED=$RRport{$IP}{dedicated}\n"
                if defined $RRport{$IP}{dedicated};
        }
        $Content .= "\n";
        $IP =~ s/(.*)\..*/$1\.\*/;
        $IPMask{$IP} = 1;
        $IPMask{"::ffff:$IP"} = 1;
    }

    $Content .= "[$_]\n" foreach sort keys %IPMask;
    WriteFile($ConfigFile, \$Content, { append=>1, create_directory=>1 });
}

sub writeServerConfigXML {
    my ($CacheServer, $RequestRouter, $Pool) = @_;

    my $ConfigFile = "$ENV{EPAGES_CONFIG}/ServerConfig.xml";
    my $Content = "<config>\n\n";

    # <cacheservers/>
    $Content .= "  <cacheservers>\n";
    foreach (sort keys %$CacheServer) {
        $Content .= "    <server host='$CacheServer->{$_}{host}' port='$CacheServer->{$_}{port}'";
        $Content .= " enable_cache='$CacheServer->{$_}{enable_cache}'"
            if defined $CacheServer->{$_}{enable_cache};
        $Content .= " bind='$CacheServer->{$_}{bind}'"
            if defined $CacheServer->{$_}{bind};
        $Content .= " />\n";
    }
    $Content .= "  </cacheservers>\n\n";

    # <requestrouters/>
    if (defined $RequestRouter->{max_cache_items}) {
        $Content .= "  <requestrouters max_cache_items='$RequestRouter->{max_cache_items}'>\n";
    } else {
        $Content .= "  <requestrouters>\n";
    }
    foreach my $ID (sort keys %$RequestRouter) {
        next if $ID eq 'max_cache_items';
        my $Name = $RequestRouter->{$ID}{name} // $ID;
        $Content .= "    <requestrouter name='$Name'";
        $Content .= " $_='$RequestRouter->{$ID}{$_}'" foreach qw(host port monitor_port mc_port);
        $Content .= " />\n";
    }
   $Content .= "  </requestrouters>\n\n";

    # <pools/>
    $Content .= "  <pools>\n\n";


    foreach my $ID (sort keys %$Pool) {
        next if $ID ~~ [qw(ExceptionHost maxmemory)];
        my $Name = $Pool->{$ID}{name} // $ID;
        $Content .= (' ' x 4). "<pool name='$Name'>\n";

        # <pools><requestrouters/></pools>
        my %hRR;
        $Content .= (' ' x 6). "<requestrouters>\n";
        foreach my $RequestRouter (sort split(/\s*,\s*/, $Pool->{$ID}{requestrouters})) {
            next unless $RequestRouter =~ /\S/;
            next if $hRR{$RequestRouter};
            $hRR{$RequestRouter} = 1;
            $Content .= (' ' x 8). "<requestrouter>$RequestRouter</requestrouter>\n"
        }
        $Content .= (' ' x 6). "</requestrouters>\n";

        # <pools><appservers/></pools>
        $Content .= (' ' x 6). "<appservers>\n";
        foreach my $AS (sort keys %{$Pool->{$ID}}) {
            next if $AS =~ /^servers_per_/ or $AS ~~ [qw(name requestrouters)];
            next unless defined $Pool->{$ID}{$AS}{ports};
            $Content .= (' ' x 8). "<appserver host='$AS'";
            $Content .= " $_='$Pool->{$ID}{$AS}{$_}'" foreach qw(ports maxmemory);
            $Content .= " priority='$Pool->{$ID}{$AS}{priority}'"
                if defined $Pool->{$ID}{$AS}{priority};
            $Content .= " />\n";
        }
        $Content .= (' ' x 6). "</appservers>\n";

        # <pools><limits/></pools>
        my $Attributes;
        $Attributes = " servers_per_site='$Pool->{$ID}{servers_per_site}'"
            if defined $Pool->{$ID}{servers_per_site};
        $Attributes .= " servers_per_client='$Pool->{$ID}{servers_per_client}'"
            if defined $Pool->{$ID}{servers_per_client};

        if (defined $Attributes or defined $Pool->{$ID}{servers_per_extension}) {
            $Content .= (' ' x 6). "<limits$Attributes>\n";
            foreach (split(/\s*,\s*/, $Pool->{$ID}{servers_per_extension})) {
                my ($Ext, $Limit) = split(/=/, $_);
                my $SEXT = 'servers_per_extension';
                $Content .= (' ' x 8). "<$SEXT extension='$Ext'>$Limit</$SEXT>\n";
            }
            $Content .= (' ' x 6). "</limits>\n";
        }
        $Content .= (' ' x 4). "</pool>\n\n";
    }
    $Content .= "  </pools>\n\n";
    $Content .= "</config>\n";
    WriteFile($ConfigFile, \$Content, { create_directory=>1 });
}

#----------------------------------------------------------------------------
# Main
#----------------------------------------------------------------------------

sub Main {
    local $| = 1;
    my %Options = ();
    my @GetOpt = qw(help);

    GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};
    usage() unless @ARGV;

    my $EpagesInstallConf = $ARGV[0];
    return unless -s $EpagesInstallConf;

    my ($CacheServer, $RequestRouter, $Pool, $Global, $URLRewrite)
        = getServerConfig($EpagesInstallConf);

    # set default values for [GLOBAL] and [URLRewrite] in WebInterface.conf
    ($Global, $URLRewrite) =
        setWebInterfaceConfDefaults($Global, $URLRewrite, $RequestRouter, $Pool);

    # create [GLOBAL] and [URLRewrite] in WebInterface.conf
    writeWebInterfaceConf($Global, $URLRewrite);

    my $NewConfiguration = 0;
    if (not -d "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/WebInterface/Scripts")
    {
        # we assume new configuration if WebInterface/API doesn't exist
        $NewConfiguration = 1;
    }
    elsif (-f "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/WebInterface/Scripts/generateServerConfig.pl")
    {
        # or WebInterface/Scripts/generateServerConfig.pl exists
        $NewConfiguration = 1;
    }

    if ($NewConfiguration) {
        # create ServerConfig.xml
        writeServerConfigXML($CacheServer, $RequestRouter, $Pool);
    } else {
        # add sections to WebInterface.conf
        appendWebInterfaceConfSections($RequestRouter, $Pool);
    }
}

sub usage {
    my ($ErrorMsg) = @_;
    print STDERR "Error:\n    $ErrorMsg\n\n" if $ErrorMsg;
    print STDERR <<END_USAGE;
Description:
    Creates WebInterface.conf and ServerConfig.xml from information
    in install-epages.conf (or uses default values unless exist).

Usage:
    perl $0 /path/to/install-epages.conf

Flags:
    -help      show the command line options

Example:
    $0 /srv/epages/etc/sysconfig/install-epages.conf
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;
