#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods
##############################################################################

# die if shunit2 not found
type shunit2 && SHUNIT2=$(type -p shunit2) || exit 1

# test script directory (/var/epages/t)
TEST_DIR=$(cd $(dirname $0) && pwd)
TEST_FILE=$(basename $0)

# script to test
SCRIPT_TO_TEST=${TEST_DIR%/*}/bin/${TEST_FILE%-test.sh}.sh

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  if [[ ${TEST_DIR%/*} != /var/epages ]] ; then
    mkdir -m 755 -p /var/epages/functions
    cp -rp ${TEST_DIR%/*}/functions/*.sh /var/epages/functions
  fi
  . ${TEST_DIR%/*}/functions/fun-mysql.sh
  export my_cnf

  echo '-> export EPAGES_INSTALLATION_MOCK=1'
  TEST_CONFD=/tmp/${0##*/}.~$$~/Shared/Config
}

function oneTimeTearDown() {
  if [[ -n $KEEP_TEST_CONF ]] ; then
    echo "test_configdir: $TEST_CONFD"
  else
    rm -rf /tmp/${0##*/}.~$$~ /root/.mysql_secret
  fi
}

##############################################################################
# define functions                                                           #
##############################################################################

function test_script_syntax() {
  # script must have correct syntax
  bash -n $SCRIPT_TO_TEST
  assertTrue "[FAILED] bash -n $SCRIPT_TO_TEST" $? || exit 1
}

function test_script_executable() {
  # script must be executable
  test -x $SCRIPT_TO_TEST
  assertTrue "[FAILED] test -x $SCRIPT_TO_TEST" $?
  chmod 755 $SCRIPT_TO_TEST
  echo
}

function test_basic() {
  export EPAGES_INSTALLATION_MOCK=1

  export EPAGES_CONFIG=$TEST_CONFD
  echo "-> export EPAGES_CONFIG=$TEST_CONFD"
  export EPAGES_ROOT_DATABASES=
  echo "-> export EPAGES_ROOT_DATABASES="

  echo "-> INSTALL_EPAGES_ASPOOLDB= $SCRIPT_TO_TEST"
  export INSTALL_EPAGES_ASPOOLDB=
  $SCRIPT_TO_TEST
  EXPECTED=$TEST_CONFD/Database.d/Backup-$(hostname --short).conf
  test -h ${EXPECTED%/*}/Backup.conf
  assertTrue "[FAILED] test -h ${EXPECTED%/*}/Backup.conf" $?
  ls $(readlink -m ${EXPECTED%/*}/Backup.conf) >/dev/null
  assertTrue "[FAILED] ls ${EXPECTED%/*}/Backup.conf" $?
  test -f $EXPECTED
  assertTrue "[FAILED] test -f $EXPECTED" $?
  grep -q '^passwd=$' $EXPECTED
  assertTrue "[FAILED] grep -q '^passwd=$' $EXPECTED" $?
  grep -q '^login=root$' $EXPECTED
  assertTrue "[FAILED] grep -q '^login=root$' $EXPECTED" $?
  grep -q '^datasource=dbi:mysql:' $EXPECTED
  assertTrue "[FAILED] grep -q '^datasource=dbi:mysql:' $EXPECTED" $?
  grep -q '^MySQLConfigFile=' $EXPECTED
  assertFalse "[FAILED] grep -q '^MySQLConfigFile=' $EXPECTED" $?

  # ASPoolDB.conf is not created if INSTALL_EPAGES_ASPOOLDB empty
  test -f ${EXPECTED%/*}/ASPoolDB.conf
  assertFalse "[FAILED] test -f ${EXPECTED%/*}/ASPoolDB.conf" $?

  echo "-> unset INSTALL_EPAGES_ASPOOLDB ; $SCRIPT_TO_TEST"
  unset INSTALL_EPAGES_ASPOOLDB
  $SCRIPT_TO_TEST

  # ASPoolDB.conf is created if INSTALL_EPAGES_ASPOOLDB is unset
  test -f ${EXPECTED%/*}/ASPoolDB.conf
  assertTrue "[FAILED] test -f ${EXPECTED%/*}/ASPoolDB.conf" $?
  echo
}

function test_aspooldb() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD

  export INSTALL_EPAGES_ASPOOLDB=1
  echo "-> export INSTALL_EPAGES_ASPOOLDB=1"
  export INSTALL_EPAGES_MY_CNF=$(my_cnf)
  echo "-> export INSTALL_EPAGES_MY_CNF=$(my_cnf)"
  export EPAGES_ROOT_DATABASES=Backup
  echo "-> export EPAGES_ROOT_DATABASES=Backup"
  rm -f $TEST_CONFD/Database.d/*

  echo "-> $SCRIPT_TO_TEST"
  $SCRIPT_TO_TEST
  local EXPECTED="ASPoolDB.conf Backup.conf "
  local FOUND=$(find $TEST_CONFD/Database.d/* -printf "%f ")
  assertEquals "[FAILED] $TEST_CONFD/Database.d: ASPoolDB.conf Backup.conf" "$EXPECTED" "$FOUND"

  EXPECTED=$TEST_CONFD/Database.d/Backup.conf
  test -h $EXPECTED
  assertFalse "[FAILED] test -h $EXPECTED" $?
  grep -q '^login=root$' $EXPECTED
  assertTrue "[FAILED] grep -q '^login=root$' $EXPECTED" $?
  grep -q '^datasource=dbi:mysql:' $EXPECTED
  assertTrue "[FAILED] grep -q '^datasource=dbi:mysql:' $EXPECTED" $?
  grep -q "^MySQLConfigFile=$(my_cnf)$" $EXPECTED
  assertTrue "[FAILED] grep -q '^MySQLConfigFile=$(my_cnf)$' $EXPECTED" $?

  EXPECTED=$TEST_CONFD/Database.d/ASPoolDB.conf
  grep -q '^login=' $EXPECTED
  assertFalse "[FAILED] grep -q '^login=' $EXPECTED" $?
  grep -q '^datasource=dbi:mysql:' $EXPECTED
  assertTrue "[FAILED] grep -q '^datasource=dbi:mysql:' $EXPECTED" $?
  grep -q "^MySQLConfigFile=$(my_cnf)$" $EXPECTED
  assertTrue "[FAILED] grep -q '^MySQLConfigFile=$(my_cnf)' $EXPECTED" $?
  echo
}

function test_password() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD

  export INSTALL_EPAGES_MYSQL_PASS=epages
  echo "-> export INSTALL_EPAGES_MYSQL_PASS=epages"
  export EPAGES_ROOT_DATABASES=Backup
  echo "-> export EPAGES_ROOT_DATABASES=Backup"

  # no /root/.mysql_secret
  # -> password is not changed by SetupMySQL.sh
  echo "-> rm -f /root/.mysql_secret"
  rm -f $TEST_CONFD/Database.d/* /root/.mysql_secret
  echo "-> $SCRIPT_TO_TEST"
  $SCRIPT_TO_TEST
  local EXPECTED="ASPoolDB.conf Backup.conf "
  local FOUND=$(find $TEST_CONFD/Database.d/* -printf "%f ")
  assertEquals "[FAILED] $TEST_CONFD/Database.d: ASPoolDB.conf Backup.conf" "$EXPECTED" "$FOUND"

  EXPECTED=$TEST_CONFD/Database.d/Backup.conf
  test -h $EXPECTED
  assertFalse "[FAILED] test -h $EXPECTED" $?
  grep -q '^passwd=$' $EXPECTED
  assertTrue "[FAILED] grep -q '^passwd=$' $EXPECTED" $?
  echo

  # has /root/.mysql_secret with current (empty) password
  # -> password is changed by SetupMySQL.sh
  echo "-> echo 'password:' > /root/.mysql_secret"
  echo 'password:' > /root/.mysql_secret
  rm -f $TEST_CONFD/Database.d/*
  echo "-> $SCRIPT_TO_TEST"
  $SCRIPT_TO_TEST
  local EXPECTED="ASPoolDB.conf Backup.conf "
  local FOUND=$(find $TEST_CONFD/Database.d/* -printf "%f ")
  assertEquals "[FAILED] $TEST_CONFD/Database.d: ASPoolDB.conf Backup.conf" "$EXPECTED" "$FOUND"

  EXPECTED=$TEST_CONFD/Database.d/Backup.conf
  test -h $EXPECTED
  assertFalse "[FAILED] test -h $EXPECTED" $?
  grep -q '^passwd=epages$' $EXPECTED
  assertTrue "[FAILED] grep -q '^passwd=epages$' $EXPECTED" $?
  echo

  # has /root/.mysql_secret with current (epages) password
  # -> password is restored by SetupMySQL.sh
  export INSTALL_EPAGES_MYSQL_PASS=
  echo "-> echo 'password:epages' > /root/.mysql_secret"
  echo 'password:epages' > /root/.mysql_secret
  rm -f $TEST_CONFD/Database.d/*
  echo "-> $SCRIPT_TO_TEST"
  $SCRIPT_TO_TEST
  local EXPECTED="ASPoolDB.conf Backup.conf "
  local FOUND=$(find $TEST_CONFD/Database.d/* -printf "%f ")
  assertEquals "[FAILED] $TEST_CONFD/Database.d: ASPoolDB.conf Backup.conf" "$EXPECTED" "$FOUND"

  EXPECTED=$TEST_CONFD/Database.d/Backup.conf
  test -h $EXPECTED
  assertFalse "[FAILED] test -h $EXPECTED" $?
  grep -q '^passwd=$' $EXPECTED
  assertTrue "[FAILED] grep -q '^passwd=$' $EXPECTED" $?
  echo
}

function test_root_databases() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD

  export INSTALL_EPAGES_MYSQL_PASS=
  echo "-> export INSTALL_EPAGES_MYSQL_PASS="
  export INSTALL_EPAGES_ASPOOLDB=
  echo "-> export INSTALL_EPAGES_ASPOOLDB="
  export INSTALL_EPAGES_MY_CNF=
  echo "-> export INSTALL_EPAGES_MY_CNF="
  export EPAGES_ROOT_DATABASES='Backup Backup2'
  echo "-> export EPAGES_ROOT_DATABASES='Backup Backup2'"
  rm -f $TEST_CONFD/Database.d/*

  echo "-> $SCRIPT_TO_TEST"
  $SCRIPT_TO_TEST
  local EXPECTED="Backup2.conf Backup.conf "
  local FOUND=$(find $TEST_CONFD/Database.d/* -printf "%f ")
  assertEquals "[FAILED] $TEST_CONFD/Database.d: Backup.conf Backup2.conf" "$EXPECTED" "$FOUND"

  for ROOT in Backup Backup2 ; do
    EXPECTED=$TEST_CONFD/Database.d/$ROOT.conf
    assertTrue "[FAILED] grep -q '^passwd=$' $EXPECTED" $?
    grep -q '^login=root$' $EXPECTED
    assertTrue "[FAILED] grep -q '^login=root$' $EXPECTED" $?
    grep -q '^datasource=dbi:mysql:' $EXPECTED
    assertTrue "[FAILED] grep -q '^datasource=dbi:mysql:' $EXPECTED" $?
    grep -q '^MySQLConfigFile=' $EXPECTED
    assertFalse "[FAILED] grep -q '^MySQLConfigFile=' $EXPECTED" $?
  done
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
