#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods                                #
##############################################################################

# install if shunit2 not found
PATH=$PATH:/usr/share/shunit2
if ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/debian_version ]] ; then
  apt-get install shunit2
elif ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/redhat-release ]] ; then
  rpm -hiv http://download.fedoraproject.org/pub/epel/6/i386/shunit2-2.1.6-3.el6.noarch.rpm
fi

# die if shunit2 not found
SHUNIT2=$(type -p shunit2) || exit 1

# test script directory: /var/epages/t
TEST_DIR=$(cd $(dirname $0) && pwd)
TEST_FILE=$(basename $0)

# script to test: /var/epages/functions/${TEST_FILE%-test.sh}.sh
SCRIPT_TO_TEST=${TEST_DIR%/*}/functions/${TEST_FILE%-test.sh}.sh

. $SCRIPT_TO_TEST

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  TMP_DIR=/tmp/${0##*/}.~$$~
  TEST_OUTPUT=$TMP_DIR/test.out
  TEST_CONF=$TMP_DIR/test.conf
  TEST_MYSQL_CONF=$TMP_DIR/test-mysqld.conf
  INSTALL_EPAGES_CONF=$TMP_DIR/conf/install-epages.conf.sample
  mkdir -p -m 755 $TMP_DIR
  rm -rf $TMP_DIR/*
  cp -rp ${TEST_DIR%/*}/conf $TMP_DIR
}

function oneTimeTearDown() {
  if [[ -n $KEEP_TEST_CONF ]] ; then
    echo "test_configdir: $TMP_DIR"
  else
    rm -rf $TMP_DIR
  fi
}

##############################################################################
# test functions                                                             #
##############################################################################

function test_find_host_section() {
  local MYSECTION

  test -r $INSTALL_EPAGES_CONF
  assertTrue "[FAILED] test -r $INSTALL_EPAGES_CONF" $? || exit 1

  # find localhost
  MYSECTION=$(find_host_section $INSTALL_EPAGES_CONF)
  assertEquals "[FAILED] Find localhost failed" "localhost" "$MYSECTION"
 
  # find hostname -s
  HOSTNAME=$(hostname -s)
  echo -e "[$HOSTNAME]" >> $INSTALL_EPAGES_CONF 
  MYSECTION=$(find_host_section $INSTALL_EPAGES_CONF)
  assertEquals "[FAILED] Find hostname -s failed" "$HOSTNAME" "$MYSECTION"
 
  # find hostname -f
  HOSTNAME=$(hostname -f)
  echo -e "[$HOSTNAME]" >> $INSTALL_EPAGES_CONF
  MYSECTION=$(find_host_section $INSTALL_EPAGES_CONF)
  assertEquals "[FAILED] Find hostname -f failed" "$HOSTNAME" "$MYSECTION"
}

function test_get_install_host() {
  local MYSECTION

  test -r $INSTALL_EPAGES_CONF
  assertTrue "[FAILED] test -r $INSTALL_EPAGES_CONF" $? || exit 1

  # find localhost
  local CMD="get_install_host $INSTALL_EPAGES_CONF localhost"
  echo "-> $CMD"
  INSTALL_EPAGES_HOST=$(get_install_host $INSTALL_EPAGES_CONF localhost)
  assertTrue "[FAILED] $CMD" $?
  local CMD="INSTALL_EPAGES_HOST=$INSTALL_EPAGES_HOST"
  echo "-> $CMD"
  [[ -n $INSTALL_EPAGES_HOST && -z ${INSTALL_EPAGES_HOST//[0-9.]/} ]]
  assertTrue "[FAILED] $CMD" $?
}

function test_find_required_packages_mysql_conf() {

  # Copy install-epages.conf.sample
  test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample
  assertTrue "[FAILED] test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample" $? || exit 1
  cp ${TEST_DIR%/*}/conf/install-epages.conf.sample $TEST_CONF
  test -r $TEST_CONF
  assertTrue "[FAILED] test -r $TEST_CONF" $? || exit 1

  # Copy conf/mysqld/simple.conf
  test -r ${TEST_DIR%/*}/conf/mysqld/simple.conf
  assertTrue "[FAILED] test -r ${TEST_DIR%/*}/conf/mysqld/simple.conf" $? || exit 1
  cp ${TEST_DIR%/*}/conf/mysqld/simple.conf $TEST_MYSQL_CONF
  test -r $TEST_MYSQL_CONF
  assertTrue "[FAILED] test -r $TEST_MYSQLD_CONF" $? || exit 1

  # Databases found: epagesj/ASPoolDB
  echo "-> find_required_packages $TEST_CONF $TEST_MYSQL_CONF"
  find_required_packages $TEST_CONF $TEST_MYSQL_CONF
  assertTrue "[FAILED] databases found: epagesj/ASPoolDB." $?

  # Database missing: epagesj
  sed -ri "/[epagesj]/d" $TEST_MYSQL_CONF
  find_required_packages $TEST_CONF $TEST_MYSQL_CONF >$TEST_OUTPUT
  grep -q "epages-j requires database: epagesj" $TEST_OUTPUT
  assertTrue "[FAILED] database missing: epagesj" $?

  # Database missing: epagesj/ASPoolDB
  sed -ri "/[ASPoolDB]/d" $TEST_MYSQL_CONF
  find_required_packages $TEST_CONF $TEST_MYSQL_CONF >$TEST_OUTPUT
  grep -q "epages-j requires database: epagesj" $TEST_OUTPUT && \
  grep -q "epages-reqsrv/epages-ascsrv require database: ASPoolDB" $TEST_OUTPUT
  assertTrue "[FAILED] database missing: epagesj/ASPoolDB" $?
}

function test_find_required_packages_databases() {

  # Copy install-epages.conf.sample
  test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample
  assertTrue "[FAILED] test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample" $? || exit 1
  cp ${TEST_DIR%/*}/conf/install-epages.conf.sample $TEST_CONF
  test -r $TEST_CONF
  assertTrue "[FAILED] test -r $TEST_CONF" $? || exit 1

  # Databases missing: epagesj/ASPoolDB
  echo "-> find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  grep -q "epages-j requires database: epagesj" $TEST_OUTPUT && \
  grep -q "epages-reqsrv/epages-ascsrv require database: ASPoolDB" $TEST_OUTPUT
  assertTrue "[FAILED] [myhost]:mysqld= -> databases missing: epagesj/ASPoolDB." $?

  # Databases found epagesj/ASPoolDB
  echo "-> find_required_packages $TEST_CONF 1 > $TEST_OUTPUT"
  find_required_packages $TEST_CONF 1 >$TEST_OUTPUT
  test -s $TEST_OUTPUT
  assertFalse "[FAILED] [myhost]:mysqld=1 -> databases found: epagesj/ASPoolDB" $?

  # Database missing: epagesj
  sed -ri "/general definitions/a installed_databases=ASPoolDB" $TEST_CONF
  echo "-> installed_databases=ASPoolDB find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  grep -q "epages-j requires database: epagesj" $TEST_OUTPUT
  assertTrue "[FAILED] installed_databases=ASPoolDB -> database missing: epagesj" $?
  grep -q "epages-reqsrv/epages-ascsrv require database: ASPoolDB" $TEST_OUTPUT
  assertFalse "[FAILED] installed_databases=ASPoolDB -> database found: ASPoolDB" $?

  # Database missing: ASPoolDB
  sed -ri "/installed_databases=ASPoolDB/d" $TEST_CONF
  sed -ri "/general definitions/a installed_databases=epagesj" $TEST_CONF
  echo "-> installed_databases=epagesj find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  grep -q "epages-reqsrv/epages-ascsrv require database: ASPoolDB" $TEST_OUTPUT
  assertTrue "[FAILED] installed_databases=epagesj -> database missing: ASPoolDB" $?
  grep -q "epages-j requires database: epagesj" $TEST_OUTPUT
  assertFalse "[FAILED] installed_databases=epagesj -> database found: epagesj" $?

  # Databases found epagesj/ASPoolDB
  sed -ri "/installed_databases=ASPoolDB/d" $TEST_CONF
  sed -ri "/general definitions/a installed_databases=ASPoolDB,epagesj" $TEST_CONF
  echo "-> installed_databases=ASPoolDB,epagesj find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  test -s $TEST_OUTPUT
  assertFalse "[FAILED] installed_databases=ASPoolDB,epagesj -> databases found" $?
}

function test_find_required_packages_patchhost() {

  # Copy install-epages.conf.sample
  test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample
  assertTrue "[FAILED] test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample" $? || exit 1
  cp ${TEST_DIR%/*}/conf/install-epages.conf.sample $TEST_CONF
  test -r $TEST_CONF
  assertTrue "[FAILED] test -r $TEST_CONF" $? || exit 1

  # Set host with allservices
  echo "[host1]" >> $TEST_CONF
  echo "appsrv=1" >> $TEST_CONF
  echo "ascsrv=1" >> $TEST_CONF
  echo "reqsrv=1" >> $TEST_CONF
  echo "mysqld=1" >> $TEST_CONF
  echo "webconf=1" >> $TEST_CONF
  echo "j=1" >> $TEST_CONF
  echo "mongo=1" >> $TEST_CONF

  # Nothing installed
  echo "-> find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  grep -q "requires: appsrv on host1" $TEST_OUTPUT && \
  grep -q "requires: ascsrv on host1" $TEST_OUTPUT && \
  grep -q "requires: reqsrv on host1" $TEST_OUTPUT && \
  grep -q "requires: mysqld on host1" $TEST_OUTPUT && \
  grep -q "requires: webconf on host1" $TEST_OUTPUT && \
  grep -q "requires: j on host1" $TEST_OUTPUT && \
  grep -q "requires: mongo on host1" $TEST_OUTPUT
  assertTrue "[FAILED] Nothing installed." $?

  # Set installed_appsrv
  echo "installed_appsrv=1" >> $TEST_CONF
  echo "-> installed_appsrv=1 find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  grep -q "requires: appsrv on host1" $TEST_OUTPUT
  assertFalse "[FAILED] Installed appsrv." $?

  # Set installed_ascsrv
  echo "installed_ascsrv=1" >> $TEST_CONF
  echo "-> installed_ascsrv=1 find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  grep -q "requires: ascsrc on host1" $TEST_OUTPUT
  assertFalse "[FAILED] Installed ascsrv" $?

  # Set installed_reqsrv
  echo "installed_reqsrv=1" >> $TEST_CONF
  echo "-> installed_reqsrv=1 find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  grep -q "requires: reqsrv on host1" $TEST_OUTPUT
  assertFalse "[FAILED] Installed reqsrv." $?

  # Set installed_mysqld
  echo "installed_mysqld=1" >> $TEST_CONF
  echo "-> installed_mysqld=1 find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  grep -q "requires: mysqld on host1" $TEST_OUTPUT
  assertFalse "[FAILED] Installed mysqld." $?

  # Set installed_webconf
  echo "installed_webconf=1" >> $TEST_CONF
  echo "-> installed_webconf=1 find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  grep -q "requires: webconf on host1" $TEST_OUTPUT && \
  assertFalse "[FAILED] Installed webconf." $?

  # Set installed_j
  echo "installed_j=1" >> $TEST_CONF
  echo "-> installed_j=1 find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF >$TEST_OUTPUT
  grep -q "requires: j on host1" $TEST_OUTPUT && \
  assertFalse "[FAILED] Installed j." $?

  # Set installed_mongo
  echo "installed_mongo=1" >> $TEST_CONF
  echo "-> installed_mongo=1 find_required_packages $TEST_CONF >$TEST_OUTPUT"
  find_required_packages $TEST_CONF 1
  assertTrue "[FAILED] Everything installed." $?
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
