#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods                                #
##############################################################################

# install if shunit2 not found
PATH=$PATH:/usr/share/shunit2
if ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/debian_version ]] ; then
  apt-get install shunit2
elif ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/redhat-release ]] ; then
  rpm -hiv http://download.fedoraproject.org/pub/epel/6/i386/shunit2-2.1.6-3.el6.noarch.rpm
fi

# die if shunit2 not found
SHUNIT2=$(type -p shunit2) || exit 1

# test script directory: /var/epages/t
TEST_DIR=$(cd $(dirname $0) && pwd)
TEST_FILE=$(basename $0)

# script to test: /var/epages/functions/${TEST_FILE%-test.sh}.sh
SCRIPT_TO_TEST=${TEST_DIR%/*}/functions/${TEST_FILE%-test.sh}.sh

. $SCRIPT_TO_TEST

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function setUp() {
  chmod 755 /var/epages/ini-config.sh ${TEST_DIR%/*}/ini-config.sh 2>/dev/null
  TEST_OUTPUT=/tmp/${0##*/}
  TEST_CONF=${TEST_OUTPUT%.sh}.conf.~$$~
}

function TearDown() {
  chmod 755 /var/epages/ini-config.sh ${TEST_DIR%/*}/ini-config.sh 2>/dev/null
}

##############################################################################
# test functions                                                             #
##############################################################################

function test_find_ini_config() {
  local FOUND EXPECTED

  # find nothing
  FOUND=$(find_ini_config --help 2>&1)
  assertFalse "[FAILED] find_ini_config --help" $?
  # return usage
  echo $FOUND | grep -q 'syntax: find_ini_config'
  assertTrue "[FAILED] find_ini_config --help" $?

  EXPECTED=${TEST_DIR%/*}/ini-config.sh
  # find ../ini-config.sh
  FOUND=$(find_ini_config)
  assertTrue "[FAILED] find_ini_config" $?
  assertEquals "[FAILED] find_ini_config" "$EXPECTED" "$FOUND"

  if [[ ${TEST_DIR%/*} != /var/epages ]] ; then
    chmod 644 ${TEST_DIR%/*}/ini-config.sh
    EXPECTED=/var/epages/ini-config.sh
    # find /var/epages/ini-config.sh
    FOUND=$(find_ini_config)
    assertTrue "[FAILED] find_ini_config" $?
    assertEquals "[FAILED] find_ini_config" "$EXPECTED" "$FOUND"
  fi

  chmod 644 /var/epages/ini-config.sh
  # find nothing
  FOUND=$(find_ini_config 2>&1)
  assertFalse "[FAILED] find_ini_config" $?
  # return error
  echo $FOUND | grep -q 'executable not found'
  assertTrue "[FAILED] find_ini_config" $?
}

function test_get_ini_value() {
  local FOUND EXPECTED

  cat > $TEST_CONF <<EOF
[1st section]
abc = ""
def= ""
ghi = 
jkl=
# this line ends with a space ==
mno= mno 
pqr = 'pqr'
; this line ends with a space too
stu=" stu " 
; and this too = 1
stu1="stu1" 
; and this too = 2
stu2 =" stu2" 
; and this too = 3
stu3 ="stu3 " 
vwx=vwx
 1 2 3 = " 1 2 3"
 4 5 6= " 4 5 6"
7 8 9 = "7 8 9"
" 1 2 3 4 " = " a b c d"
 " 5 6 " = " ef "
 "7 8" = " ef "
"7 8" = " ef "
"9 0"= " ef "
EOF

  # at least 2 arguments required
  FOUND=$(get_ini_value $TEST_CONF 2>/dev/null)
  assertFalse "[FAILED] get_ini_value $TEST_CONF" $?

  # not more than 3 arguments required
  FOUND=$(get_ini_value $TEST_CONF abc 1st section 2>/dev/null)
  assertFalse "[FAILED] get_ini_value $TEST_CONF abc 1st section" $?

  # if first argument starts with -
  FOUND=$(get_ini_value --help 2>&1)
  assertFalse "[FAILED] get_ini_value --help" $?
  # return usage
  echo $FOUND | grep -q 'syntax: get_ini_value'
  assertTrue "[FAILED] get_ini_value --help" $?

  EXPECTED=''
  # defined, but empty (drop quotes)
  for i in abc def ghi jkl ; do
    # get_ini_value $TEST_CONF $i '1st section'
    FOUND=$(get_ini_value $TEST_CONF $i '1st section')
    assertTrue "[FAILED] get_ini_value $TEST_CONF $i '1st section'" $?
    assertEquals "[FAILED] get_ini_value $TEST_CONF $i '1st section'" "$EXPECTED" "$FOUND"
  done

  # not defined
  FOUND=$(get_ini_value $TEST_CONF xyz '1st section')
  assertFalse "[FAILED] get_ini_value $TEST_CONF $i '1st section'" $?
  assertEquals "[FAILED] get_ini_value $TEST_CONF $i '1st section'" "$EXPECTED" "$FOUND"
  FOUND=$(get_ini_value $TEST_CONF xyz section)
  assertFalse "[FAILED] get_ini_value $TEST_CONF $i section" $?
  assertEquals "[FAILED] get_ini_value $TEST_CONF $i section" "$EXPECTED" "$FOUND"

  # defined and not empty (drop quotes and leading/trailing spaces)
  for i in mno pqr " stu " stu1 " stu2" "stu3 " vwx ; do
    # get_ini_value $TEST_CONF $i '1st section'
    FOUND=$(get_ini_value $TEST_CONF $i '1st section')
    assertTrue "[FAILED] get_ini_value $TEST_CONF $i '1st section'" $?
    assertEquals "[FAILED] get_ini_value $TEST_CONF $i '1st section'" "$i" "$FOUND"
  done

  # keys with spaces (drop trailing spaces)
  for i in " 1 2 3" " 4 5 6" "7 8 9" ; do
    # get_ini_value $TEST_CONF $i '1st section'
    FOUND=$(get_ini_value $TEST_CONF "$i" '1st section')
    assertTrue "[FAILED] get_ini_value $TEST_CONF '$i' '1st section'" $?
    assertEquals "[FAILED] get_ini_value $TEST_CONF '$i' '1st section'" "$i" "$FOUND"
  done

  # keys with spaces and quotes (you should never do this!)
  # 1--" 1 2 3 4 " = " a b c d"
  EXPECTED=' a b c d'
  FOUND=$(get_ini_value $TEST_CONF '" 1 2 3 4 " ' '1st section')
  assertTrue "[FAILED] get_ini_value $TEST_CONF '\" 1 2 3 4 \" ' '1st section'" $?
  assertEquals "[FAILED] get_ini_value $TEST_CONF '\" 1 2 3 4 \" ' '1st section'" "$EXPECTED" "$FOUND"
  # 2-- " 5 6 " = "  ef"
  EXPECTED=' ef '
  FOUND=$(get_ini_value $TEST_CONF ' " 5 6 " ' '1st section')
  assertTrue "[FAILED] get_ini_value $TEST_CONF ' \" 5 6 \" ' '1st section'" $?
  assertEquals "[FAILED] get_ini_value $TEST_CONF ' \" 5 6 \" ' '1st section'" "$EXPECTED" "$FOUND"
  # 3-- "7 8" = " ef "
  FOUND=$(get_ini_value $TEST_CONF ' "7 8" ' '1st section')
  assertTrue "[FAILED] get_ini_value $TEST_CONF ' \"7 8\" ' '1st section'" $?
  assertEquals "[FAILED] get_ini_value $TEST_CONF ' \"7 8\" ' '1st section'" "$EXPECTED" "$FOUND"
  # 4--"7 8" = " ef "
  FOUND=$(get_ini_value $TEST_CONF '"7 8" ' '1st section')
  assertTrue "[FAILED] get_ini_value $TEST_CONF '\"7 8\" ' '1st section'" $?
  assertEquals "[FAILED] get_ini_value $TEST_CONF '\"7 8\" ' '1st section'" "$EXPECTED" "$FOUND"
  # 5--"9 0"= " ef "
  FOUND=$(get_ini_value $TEST_CONF '"9 0"' '1st section')
  assertTrue "[FAILED] get_ini_value $TEST_CONF '\"9 0\"' '1st section'" $?
  assertEquals "[FAILED] get_ini_value $TEST_CONF '\"9 0\"' '1st section'" "$EXPECTED" "$FOUND"

  [[ -n $KEEP_TEST_CONF ]] || rm -f $TEST_CONF
}

function test_ini_set_ini_value() {
  local OPT FOUND EXPECTED
  echo -e "[1st section]\nabc=" > $TEST_CONF.ini

  # at least 3 arguments required
  set_ini_value $TEST_CONF.ini abc >/dev/null 2>&1
  assertFalse "[FAILED] set_ini_value $TEST_CONF.ini abc" $?

  # not more than 5 arguments required
  set_ini_value $TEST_CONF.ini abc abc 1st section >/dev/null 2>&1
  assertFalse "[FAILED] set_ini_value $TEST_CONF.ini abc abc 1st section" $?

  # do not update empty value with --add
  OPT=' --add'
  echo "-> set_ini_value$OPT $TEST_CONF.ini abc abc '1st section'"
  set_ini_value $OPT $TEST_CONF.ini abc abc '1st section'
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc abc '1st section'" $?
  EXPECTED= ; FOUND=$(get_ini_value $TEST_CONF.ini abc '1st section')
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc abc" "$EXPECTED" "$FOUND"

  # update empty value without option
  OPT=
  echo "-> set_ini_value$OPT $TEST_CONF.ini abc abc '1st section'"
  set_ini_value $OPT $TEST_CONF.ini abc abc '1st section'
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc abc '1st section'" $?
  EXPECTED=abc ; FOUND=$(get_ini_value $TEST_CONF.ini abc '1st section')
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc abc" "$EXPECTED" "$FOUND"

  # update non-empty value with --overwrite
  OPT=' --overwrite'
  echo "-> set_ini_value$OPT $TEST_CONF.ini abc abc '1st section'"
  set_ini_value $OPT $TEST_CONF.ini abc 1 '1st section'
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc 1 '1st section'" $?
  EXPECTED=1 ; FOUND=$(get_ini_value $TEST_CONF.ini abc '1st section')
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc 1" "$EXPECTED" "$FOUND"

  # do not update non-empty value without option
  OPT=
  echo "-> set_ini_value$OPT $TEST_CONF.ini abc abc '1st section'"
  set_ini_value $OPT $TEST_CONF.ini abc abc '1st section'
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc abc '1st section'" $?
  EXPECTED=1 ; FOUND=$(get_ini_value $TEST_CONF.ini abc '1st section')
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc abc" "$EXPECTED" "$FOUND"

  # other options than --add or --overwrite return usage
  OPT=' --help'
  echo "-> set_ini_value$OPT $TEST_CONF.ini abc 1 '1st section'"
  set_ini_value $OPT $TEST_CONF.ini abc 1 '1st section' >/dev/null 2>&1
  assertFalse "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc 1 '1st section'" $?

  # error if configfile not found
  OPT=' --help'
  echo "-> set_ini_value /for/bar/kwats abc 1 section"
  set_ini_value /for/bar/kwats abc 1 section >/dev/null 2>&1
  assertFalse "[FAILED] set_ini_value /for/bar/kwats abc 1 section" $?

  # add missing section with --add
  OPT=' --add'
  echo "-> set_ini_value$OPT $TEST_CONF.ini abc '' section2"
  set_ini_value $OPT $TEST_CONF.ini abc '' section2
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc '' section2" $?
  grep -q section2 $TEST_CONF.ini
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc '' section2" $?
  EXPECTED= ; FOUND=$(get_ini_value $TEST_CONF.ini abc section2)
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc ''" "$EXPECTED" "$FOUND"

  # add missing key with --add
  OPT=' --add'
  echo "-> set_ini_value$OPT $TEST_CONF.ini def '' section2"
  set_ini_value $OPT $TEST_CONF.ini def '' section2
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.ini def '' section2" $?
  grep -q def $TEST_CONF.ini
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.ini def '' section2" $?
  EXPECTED= ; FOUND=$(get_ini_value $TEST_CONF.ini def section2)
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.ini def ''" "$EXPECTED" "$FOUND"

  # update missing section value without option
  OPT=
  echo "-> set_ini_value$OPT $TEST_CONF.ini abc abc section2"
  set_ini_value $OPT $TEST_CONF.ini abc abc section2
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc abc section2" $?
  EXPECTED=abc ; FOUND=$(get_ini_value $TEST_CONF.ini abc section2)
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.ini abc abc" "$EXPECTED" "$FOUND"

  [[ -n $KEEP_TEST_CONF ]] || rm -f $TEST_CONF.ini
}

function test_env_set_ini_value() {
  local OPT FOUND EXPECTED
  echo "abc=" > $TEST_CONF.env

  # do not update empty value with --add
  OPT=' --add'
  echo "-> set_ini_value$OPT $TEST_CONF.env abc abc"
  set_ini_value $OPT $TEST_CONF.env abc abc
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.env abc abc" $?
  EXPECTED= ; FOUND=$(get_ini_value $TEST_CONF.env abc)
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.env abc abc" "$EXPECTED" "$FOUND"

  # update empty value without option
  OPT=
  echo "-> set_ini_value$OPT $TEST_CONF.env abc abc"
  set_ini_value $OPT $TEST_CONF.env abc abc
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.env abc abc" $?
  EXPECTED=abc ; FOUND=$(get_ini_value $TEST_CONF.env abc)
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.env abc abc" "$EXPECTED" "$FOUND"

  # update non-empty value with --overwrite
  OPT=' --overwrite'
  echo "-> set_ini_value$OPT $TEST_CONF.env abc abc"
  set_ini_value $OPT $TEST_CONF.env abc 1
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.env abc 1" $?
  EXPECTED=1 ; FOUND=$(get_ini_value $TEST_CONF.env abc)
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.env abc 1" "$EXPECTED" "$FOUND"

  # do not update non-empty value without option
  OPT=
  echo "-> set_ini_value$OPT $TEST_CONF.env abc abc"
  set_ini_value $OPT $TEST_CONF.env abc abc
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.env abc abc" $?
  EXPECTED=1 ; FOUND=$(get_ini_value $TEST_CONF.env abc)
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.env abc abc" "$EXPECTED" "$FOUND"

  # add missing key with --add
  OPT=' --add'
  echo "-> set_ini_value$OPT $TEST_CONF.env def ''"
  set_ini_value $OPT $TEST_CONF.env def ''
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.env def ''" $?
  grep -q def $TEST_CONF.env
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.env def ''" $?
  EXPECTED= ; FOUND=$(get_ini_value $TEST_CONF.env def)
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.env def ''" "$EXPECTED" "$FOUND"

  # update empty value without option
  OPT=
  echo "-> set_ini_value$OPT $TEST_CONF.env def def"
  set_ini_value $OPT $TEST_CONF.env def def
  assertTrue "[FAILED] set_ini_value$OPT $TEST_CONF.env def def" $?
  EXPECTED=def ; FOUND=$(get_ini_value $TEST_CONF.env def)
  assertEquals "[FAILED] set_ini_value$OPT $TEST_CONF.env def def" "$EXPECTED" "$FOUND"

  [[ -n $KEEP_TEST_CONF ]] || rm -f $TEST_CONF.env
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
