#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods                                #
##############################################################################

# install if shunit2 not found
PATH=$PATH:/usr/share/shunit2
if ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/debian_version ]] ; then
  apt-get install shunit2
elif ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/redhat-release ]] ; then
  rpm -hiv http://download.fedoraproject.org/pub/epel/6/i386/shunit2-2.1.6-3.el6.noarch.rpm
fi

# die if shunit2 not found
SHUNIT2=$(type -p shunit2) || exit 1

# test script directory: /var/epages/t
TEST_DIR=$(cd $(dirname $0) && pwd)
TEST_FILE=$(basename $0)

# script to test: /var/epages/functions/${TEST_FILE%-test.sh}.sh
SCRIPT_TO_TEST=${TEST_DIR%/*}/functions/${TEST_FILE%-test.sh}.sh

. ${TEST_DIR%/*}/functions/fun-epages.sh
export epages_status epages_service

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  chmod 755 /var/epages/ini-config.sh ${TEST_DIR%/*}/ini-config.sh 2>/dev/null
  TMP_DIR=/tmp/${0##*/}.~$$~
  TMP_CONF=$TMP_DIR/Config
  TEST_CONF=$TMP_DIR/${0##*/}.conf
  mkdir -p -m 755 $TMP_DIR
}

function oneTimeTearDown() {
  chmod 755 /var/epages/ini-config.sh ${TEST_DIR%/*}/ini-config.sh 2>/dev/null
  [[ -n $KEEP_TEST_CONF ]] || rm -rf $TMP_DIR
}

##############################################################################
# test functions                                                             #
##############################################################################

function test_update_database_conf_passwd() {
  export EPAGES_CONFIG=$TMP_CONF
  local CMD="update_database_conf_passwd root abcde"
  echo "-> $CMD"
  update_database_conf_passwd root abcde
  grep -q passwd=abcde $TMP_CONF/Database.d/root.conf
  assertTrue "[FAILED] $CMD" $?

  CMD="update_database_conf_passwd Backup"
  echo "-> $CMD"
  update_database_conf_passwd Backup
  grep -q ^passwd= $TMP_CONF/Database.d/Backup.conf
  assertTrue "[FAILED] $CMD" $?
}

function test_set_mysql_root_password() {
  . /etc/default/epages6
  epages_status DB
  local MYSQL_STATUS=$?
  local STATUS_RUNNING=0
  local STATUS_STOPPED=1
  local STATUS_MISSING=2
  export EPAGES_ROOT_DATABASES=Backup

  if [[ $MYSQL_STATUS = $STATUS_MISSING ]] ; then
    return
  elif [[ $MYSQL_STATUS = $STATUS_STOPPED ]] ; then
    echo  -e "\n-> epages_service start DB"
    epages_service start DB
  fi

  # password 'epages' in $TMP_CONF/Database.d/Backup.conf
  local CMD="set_mysql_root_password epages (Backup.conf)"
  echo -e "\n-> $CMD"
  EPAGES_CONFIG=$TMP_CONF set_mysql_root_password epages
  grep -q passwd=epages $TMP_CONF/Database.d/Backup.conf
  assertTrue "[FAILED] $CMD" $?
  echo exit | mysql -uroot -pepages
  assertTrue "[FAILED] $CMD" $?

  # no password in $TMP_CONF/Database.d/Backup.conf
  CMD="set_mysql_root_password (Backup.conf)"
  echo -e "\n-> $CMD"
  EPAGES_CONFIG=$TMP_CONF set_mysql_root_password
  grep -q  ^passwd= $TMP_CONF/Database.d/Backup.conf
  assertTrue "[FAILED] $CMD" $?
  echo exit | mysql -uroot
  assertTrue "[FAILED] $CMD" $?

  # password 'epages' in $TMP_CONF/Database.d/Backup-$(hostname --short.conf
  unset EPAGES_ROOT_DATABASES
  rm -f $TMP_CONF/Database.d/Backup.conf
  CMD="set_mysql_root_password epages (Backup-$(hostname --short).conf)"
  echo -e "\n-> $CMD"
  EPAGES_CONFIG=$TMP_CONF set_mysql_root_password epages
  grep -q passwd=epages $TMP_CONF/Database.d/Backup-$(hostname --short).conf
  assertTrue "[FAILED] $CMD" $?
  echo exit | mysql -uroot -pepages
  assertTrue "[FAILED] $CMD" $?

  # password 'epages' in $EPAGES_CONFIG/Database.d/Backup.conf
  CMD="set_mysql_root_password epages ($EPAGES_CONFIG/Database.d/Backup.conf)"
  echo -e "\n-> $CMD"
  EPAGES_ROOT_DATABASES=Backup set_mysql_root_password epages
  grep -q  passwd=epages $EPAGES_CONFIG/Database.d/Backup.conf
  assertTrue "[FAILED] $CMD" $?
  echo exit | mysql -uroot -pepages
  assertTrue "[FAILED] $CMD" $?

  # password 'abcde' for EPAGES_ROOT_DATABASES='Backup Backup2'
  CMD="set_mysql_root_password abcde (EPAGES_ROOT_DATABASES='Backup Backup2')"
  echo -e "\n-> $CMD"
  EPAGES_ROOT_DATABASES='Backup Backup2' set_mysql_root_password abcde
  grep -q  passwd=abcde $EPAGES_CONFIG/Database.d/Backup.conf
  assertTrue "[FAILED] $CMD" $?
  grep -q  passwd=abcde $EPAGES_CONFIG/Database.d/Backup2.conf
  assertTrue "[FAILED] $CMD" $?
  echo exit | mysql -uroot -pabcde
  assertTrue "[FAILED] $CMD" $?

  # no password in $EPAGES_CONFIG/Database.d/Backup.conf
  CMD="set_mysql_root_password ($EPAGES_CONFIG/Database.d/Backup.conf)"
  echo -e "\n-> $CMD"
  EPAGES_ROOT_DATABASES=Backup set_mysql_root_password
  grep -q  ^passwd= $EPAGES_CONFIG/Database.d/Backup.conf
  assertTrue "[FAILED] $CMD" $?
  echo exit | mysql -uroot
  assertTrue "[FAILED] $CMD" $?
}

function test_run_sql() {

  # password 'epages' in $EPAGES_CONFIG/Database.d/Backup.conf
  local CMD="EPAGES_ROOT_DATABASES=Backup set_mysql_root_password epages"
  echo -e "\n-> $CMD"
  EPAGES_ROOT_DATABASES=Backup set_mysql_root_password epages
  CMD="run_sql -pepages 'select 1;'"
  echo "-> $CMD"
  local EXPECTED=11
  local FOUND=$(run_sql -pepages 'select 1;' | tr -d '\n')
  assertTrue "[FAILED] $CMD" $?
  assertEquals "[FAILED] $CMD" "$EXPECTED" "$FOUND"

  # without semicolon
  CMD="run_sql -pepages 'select 1'"
  echo "-> $CMD"
  run_sql -pepages 'select 1' >/dev/null
  assertTrue "[FAILED] $CMD" $?

  # wrong password
  CMD="run_sql -pKWATS 'select 1;'"
  echo "-> $CMD"
  run_sql -pKWATS 'select 1;'
  assertFalse "[FAILED] $CMD" $?

  # no password in $EPAGES_CONFIG/Database.d/Backup.conf
  local CMD="EPAGES_ROOT_DATABASES=Backup set_mysql_root_password"
  echo -e "\n-> $CMD"
  EPAGES_ROOT_DATABASES=Backup set_mysql_root_password
  local CMD="run_sql 'select 1;'"
  echo "-> $CMD"
  run_sql 'select 1;' >/dev/null
  assertTrue "[FAILED] $CMD" $?
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
