#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods                                #
##############################################################################

# install if shunit2 not found
PATH=$PATH:/usr/share/shunit2
if ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/debian_version ]] ; then
  apt-get install shunit2
elif ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/redhat-release ]] ; then
  rpm -hiv http://download.fedoraproject.org/pub/epel/6/i386/shunit2-2.1.6-3.el6.noarch.rpm
fi

# die if shunit2 not found
SHUNIT2=$(type -p shunit2) || exit 1

# test script directory: /var/epages/t
TEST_DIR=$(cd $(dirname $0) && pwd)

# script to test: /var/epages/functions/fun-pkg.sh
SCRIPT_TO_TEST=${TEST_DIR%/*}/functions/fun-pkg.sh

. $SCRIPT_TO_TEST

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function setUp() {
  DIR=/tmp/pid.~$$~/1/2
  rm -rf /tmp/pid.~$$~ /tmp/epages/pid.~$$~
}

function oneTimeTearDown() {
  rm -rf /tmp/*.~$$~ /tmp/epages/pid.~$$~
}

##############################################################################
# test functions                                                             #
##############################################################################

function test_created_directories() {
  local FOUND EXPECTED
  FOUND=$(created_directories $DIR | tr '\n' ' ')
  assertTrue "[FAILED] created_directories $DIR" $?
  EXPECTED="/tmp/pid.~$$~ ${DIR%/*} $DIR "
  assertEquals "[FAILED] created_directories $DIR" "$EXPECTED" "$FOUND"

  FOUND=$(created_directories $DIR/1 | tr '\n' ' ')
  assertTrue "[FAILED] created_directories $DIR/1" $?
  EXPECTED="$DIR/1 "
  assertEquals "[FAILED] created_directories $DIR/1" "$EXPECTED" "$FOUND"
  # default mode for new directories is 755
  assertEquals "[FAILED] stat -c %a $DIR/1" 755 $(stat -c %a $DIR/1)

  FOUND=$(created_directories -m 700 $DIR/1 | tr '\n' ' ')
  assertTrue "[FAILED] created_directories -m 700 $DIR/1" $?
  EXPECTED=""
  assertEquals "[FAILED] created_directories -m 700 $DIR/1" "$EXPECTED" "$FOUND"
  # mode does not change if directory already exist
  assertEquals "[FAILED] stat -c %a $DIR/1" 755 $(stat -c %a $DIR/1)

  FOUND=$(created_directories -m 700 $DIR/1/2 | tr '\n' ' ')
  assertTrue "[FAILED] created_directories -m 700 $DIR/1/2" $?
  EXPECTED="$DIR/1/2 "
  assertEquals "[FAILED] created_directories -m 700 $DIR/1/2" "$EXPECTED" "$FOUND"
  # set mode to 700 for new directory
  assertEquals "[FAILED] stat -c %a $DIR/1/2" 700 $(stat -c %a $DIR/1/2)

  # 0 arguments not allowed
  created_directories 2>/dev/null
  assertFalse "[FAILED] created_directories" $?
  # just -m not allowed
  created_directories -m 2>/dev/null
  assertFalse "[FAILED] created_directories -m" $?
  # 2 arguments not allowed
  created_directories -m x 2>/dev/null
  assertFalse "[FAILED] created_directories -m x" $?
  # 2 arguments not allowed
  created_directories x y 2>/dev/null
  assertFalse "[FAILED] created_directories x y" $?
  # mkdir: cannot create directory '/proc/x': Operation not permitted
  created_directories /proc/x 2>/dev/null
  assertFalse "[FAILED] created_directories /proc/x" $?
}

function test_mktempdir() {
  local FOUND EXPECTED
  FOUND=$(mktempdir pid.~$$~)
  assertTrue "[FAILED] mktempdir pid.~$$~" $?
  EXPECTED=/tmp/epages/pid.~$$~
  assertEquals "[FAILED] mktempdir pid.~$$~" "$EXPECTED" "$FOUND"
  # set mode to 1777 for new directory
  assertEquals "[FAILED] stat -c %a $DIR/1/2" 1777 $(stat -c %a $EXPECTED)

  FOUND=$(mktempdir pid.~$$~/1/2)
  assertTrue "[FAILED] mktempdir pid.~$$~/1/2" $?
  EXPECTED=/tmp/epages/pid.~$$~/1/2
  assertEquals "[FAILED] mktempdir pid.~$$~/1/2" "$EXPECTED" "$FOUND"
  # set mode to 1777 for new directory
  assertEquals "[FAILED] stat -c %a $DIR/1" 1777 $(stat -c %a ${EXPECTED%/*})

  # 0 arguments not allowed
  mktempdir 2>/dev/null
  assertFalse "[FAILED] mktempdir" $?
  # options starting with '-' not allowed
  mktempdir -m 2>/dev/null
  assertFalse "[FAILED] mktempdir -m" $?
  # 2 arguments not allowed
  mktempdir -m x 2>/dev/null
  assertFalse "[FAILED] mktempdir -m x" $?
  # 2 arguments not allowed
  mktempdir x y 2>/dev/null
  assertFalse "[FAILED] mktempdir x y" $?
}

function test_grep_server() {
  local GREP_SERVER
  # abc has neither server list nor global setting -> nope
  GREP_SERVER=$(grep_server abc)
  assertEquals "[FAILED] grep_server abc" "" "$GREP_SERVER"
  # abc has server list with unknown item (like 1) -> yeah
  GREP_SERVER=$(grep_server abc 1)
  assertEquals "[FAILED] grep_server abc 1" 1 "$GREP_SERVER"
  # abc has global setting -> yeah
  GREP_SERVER=$(grep_server abc '' 1)
  assertEquals "[FAILED] grep_server '' 1" 1 "$GREP_SERVER"
  # abc belongs to server list -> yeah
  GREP_SERVER=$(grep_server abc 'abc def ghi')
  assertEquals "[FAILED] grep_server abc 'abc def ghi'" 1 "$GREP_SERVER"
  # abc does not belong to server list -> nope
  GREP_SERVER=$(grep_server abc 'appsrv ascsrv reqsrv')
  assertEquals "[FAILED] grep_server abc 'appsrv ascsrv reqsrv'" "" "$GREP_SERVER"
  # abc does not belong to server list but has global setting -> yeah
  GREP_SERVER=$(grep_server abc 'appsrv ascsrv reqsrv' 1)
  assertEquals "[FAILED] grep_server 'appsrv ascsrv reqsrv' 1" 1 "$GREP_SERVER"
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
