#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods
##############################################################################

# die if shunit2 not found
type shunit2 && SHUNIT2=$(type -p shunit2) || exit 1

# test script directory (/var/epages/t)
TEST_DIR=$(cd $(dirname $0) && pwd)

# script to test (/var/epages/install-epages.sh)
SCRIPT_TO_TEST=${TEST_DIR%/*}/install-epages.sh

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function setUp() {
  rm -f /tmp/0 /tmp/1 /tmp/standby

  # source /var/epages/functions/fun-*.sh
  . ${TEST_DIR%/*}/functions/fun-epages.sh
  export find_ini_config get_ini_value
  . ${TEST_DIR%/*}/functions/fun-arch.sh
  export absolute_path

  TEST_OUTPUT=/tmp/${0##*/}
  TEST_CONF=${TEST_OUTPUT%.sh}.conf.~$$~
  TEST_OUTPUT=${TEST_OUTPUT%.sh}.txt.~$$~
}

function tearDown() {
  rm -f $TEST_CONF.1
}

function oneTimeTearDown() {
  if [[ -n $KEEP_TEST_CONF ]] ; then
    echo "test_configfile: $TEST_CONF"
    echo "test_outputfile: $TEST_OUTPUT"
  else
    rm -rf $TEST_CONF $TEST_OUTPUT $TEST_OUTPUT.*
  fi
}

##############################################################################
# define functions                                                           #
##############################################################################

function test_script_syntax() {
  # script must have correct syntax
  bash -n $SCRIPT_TO_TEST
  assertTrue "[FAILED] bash -n $SCRIPT_TO_TEST" $? || exit 1
}

function test_script_executable() {
  # script must be executable
  test -x $SCRIPT_TO_TEST
  assertTrue "[FAILED] test -x $SCRIPT_TO_TEST" $?
  chmod 755 $SCRIPT_TO_TEST
}

function test_configfile_empty() {
  # args: --configfile $TEST_CONF
  local CMD="$SCRIPT_TO_TEST --configfile /tmp/my.conf --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock 2>&1 | grep -q 'no servers found'
  assertTrue "[FAILED] $SCRIPT_TO_TEST --configfile $TEST_CONF --mock" $?
  rm -f /tmp/my.conf
}

function test_configfile_missing() {
  local CMD="$SCRIPT_TO_TEST --configfile --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --configfile --mock 2>&1  | grep -q 'Option requires 1 argument'
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_missing_singlehost() {
  # args: --configfile $TEST_CONF
  rm -f $TEST_CONF || exit 1
  local CMD="$SCRIPT_TO_TEST --singlehost --configfile /proc/my.conf --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --singlehost --configfile /proc/my.conf --mock 2>&1 | grep -q 'cannot write'
  local RESULT=(${PIPESTATUS[@]})
  assertFalse "[FAILED] $CMD" ${RESULT[0]}
  assertTrue "[FAILED] grep 'cannot write'" ${RESULT[1]}
}

function test_configfile_missing_distributed() {
  # args: --configfile $TEST_CONF
  rm -f $TEST_CONF || exit 1
  local CMD="$SCRIPT_TO_TEST --distributed --configfile /proc/my.conf --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --distributed --configfile /proc/my.conf --mock 2>&1 | grep -q 'cannot write'
  local RESULT=(${PIPESTATUS[@]})
  assertFalse "[FAILED] $CMD" ${RESULT[0]}
  assertTrue "[FAILED] grep 'cannot write'" ${RESULT[1]}
}

function test_configfile_singlehost() {
  # conf: patchhost=1
  test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample
  assertTrue "[FAILED] test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample" $? || exit 1
  cp ${TEST_DIR%/*}/conf/install-epages.conf.sample $TEST_CONF
  test -r $TEST_CONF
  assertTrue "[FAILED] test -r $TEST_CONF" $? || exit 1

  # install-epages.conf.sample contains singlehost installation
  local CMD="$SCRIPT_TO_TEST --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock 2>&1 | grep -q 'install epages'
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_no_ascsrv() {
  # conf: ascsrv=0
  sed -ri 's,^(ascsrv)=.*,\1=0,' $TEST_CONF

  # install-epages.conf.sample contains distributed installation
  # because no ascsrc
  local EXPECTED='appsrv j mysqld reqsrv webconf'
  local CMD="$SCRIPT_TO_TEST --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock | grep -q "INCLUDE=$EXPECTED"
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_no_ascsrv_old() {
  # install-epages.conf.sample contains singlehost installation
  # because version has no ascsrv
  local CMD="$SCRIPT_TO_TEST --version 6.16.3 --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --version 6.16.3 --configfile $TEST_CONF --mock | grep -q 'install epages'
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_no_j_old() {
  # conf: j=0
  sed -ri 's,^(j)=.*,\1=0,' $TEST_CONF

  # install-epages.conf.sample contains distributed installation
  # because no j
  local EXPECTED='appsrv mysqld reqsrv webconf'
  local CMD="$SCRIPT_TO_TEST --version 6.16.3 --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --version 6.16.3 --configfile $TEST_CONF --mock | grep -q "INCLUDE=$EXPECTED"
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_no_j_veryold() {
  # install-epages.conf.sample contains singlehost installation
  # because version has no j
  local CMD="$SCRIPT_TO_TEST --version 6.13.0 --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --version 6.13.0 --configfile $TEST_CONF --mock | grep -q 'install epages'
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_distributed() {
  # conf: patchhost=0
  test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample
  assertTrue "[FAILED] test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample" $? || exit 1
  cp ${TEST_DIR%/*}/conf/install-epages.conf.sample $TEST_CONF
  test -r $TEST_CONF
  assertTrue "[FAILED] test -r $TEST_CONF" $? || exit 1

  local EXPECTED='appsrv ascsrv j mysqld reqsrv webconf'
  sed -ri 's,^(patchhost)=.*,\1=0,' $TEST_CONF
  assertTrue "[FAILED] sed -ri 's,^(patchhost)=.*,\1=0,' $TEST_CONF" $? || exit 1

  # with patchhost=0 configuration changes to distributed installation
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock | grep -q "INCLUDE=$EXPECTED"
  assertTrue "[FAILED] $SCRIPT_TO_TEST: patchhost=0" $?
}

function test_configfile_standby() {
  # conf: *srv=standby
  test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample
  assertTrue "[FAILED] test -r ${TEST_DIR%/*}/conf/install-epages.conf.sample" $? || exit 1
  sed -r 's,^([a-z].*)=.*/.*,\1=standby,' ${TEST_DIR%/*}/conf/install-epages.conf.sample >$TEST_CONF
  test -r $TEST_CONF
  assertTrue "[FAILED] test -r $TEST_CONF" $? || exit 1

  local EXPECTED='appsrv ascsrv j mysqld reqsrv webconf'
  sed -ri 's,^(patchhost)=.*,\1=0,' $TEST_CONF
  assertTrue "[FAILED] sed -ri 's,^(patchhost)=.*,\1=0,' $TEST_CONF" $? || exit 1

  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock | grep -q '/standby'
  assertTrue "[FAILED] $SCRIPT_TO_TEST: *srv=standby" $?
}

function test_configfile_general_definitions() {
  # conf: version=6.17.40 repo=foobar mysqld-root-password=abcde
  sed -ri 's,^(version)=.*,\1=6.17.40,' $TEST_CONF
  sed -ri 's,^(repo)=.*,\1=foobar,' $TEST_CONF
  sed -ri 's,^(mysqld-root-password)=.*,\1=abcde,' $TEST_CONF
  sed -ri 's,^(mysqld)=.*,\1=1,' $TEST_CONF
  sed -ri 's,(my-cnf)=,;\1=,' $TEST_CONF

  # with patchhost=0 configuration changes to distributed installation
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q '/var/epages/SetRepository.sh -b 6.17.40 -r foobar' $TEST_OUTPUT && \
  grep -q '^INSTALL_EPAGES_MYSQL_PASS=abcde$' $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: INSTALL_EPAGES_MYSQL_PASS=abcde" $?
  grep -q '/var/epages/SetRepository.sh -b 6.17.40 -r foobar' $TEST_OUTPUT && \
  # my-cnf not set -> INSTALL_EPAGES_MY_CNF not set
  grep -q '^INSTALL_EPAGES_MY_CNF=$' $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: INSTALL_EPAGES_MY_CNF=" $?
}

function test_configfile_general_definitions_my_cnf() {
  # conf: version=6.17.40 repo=foobar my-cnf=123
  sed -ri 's,^;(my-cnf)=.*,\1=123,' $TEST_CONF

  # my-cnf set -> INSTALL_EPAGES_MY_CNF set
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q '^INSTALL_EPAGES_MY_CNF=123$' $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: [general definitions] my-cnf=123" $?
}

function test_configfile_general_definitions_my_cnf_empty() {
  # conf: version=6.17.40 repo=foobar my-cnf=
  sed -ri 's,^(my-cnf)=.*,\1=,' $TEST_CONF

  # my-cnf empty -> INSTALL_EPAGES_MY_CNF empty
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q '^INSTALL_EPAGES_MY_CNF=$' $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: [general definitions] my-cnf=" $?
}

function test_configfile_cmdline_general_definitions() {
  # args --version 6.17.41 --repo otherepo
  local CMD="$SCRIPT_TO_TEST --version 6.17.41 --repo otherepo --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --version 6.17.41 --repo otherepo --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q '/var/epages/SetRepository.sh -b 6.17.41 -r otherepo' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_set_uid() {
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >/dev/null
  local EXPECTED FOUND
  for user in eprunapp epagesj ep_appl eprunweb ; do
    # if user exists then configfile must contain its UID
    grep -q "^$user:" /etc/passwd || continue
    EXPECTED=$(awk -F= "/^$user=/"'{print $2}' $TEST_CONF)
    FOUND=$(id -u $user)
    assertEquals "[FAILED] grep $user= $TEST_CONF -> id -u $user" "$EXPECTED" "$FOUND"
  done
}

function test_configfile_unix_users() {
  # conf: epages=eprunapp=epagesj=ep_appl=111 
  for i in epages ep_appl eprunapp epagesj ep_web eprunweb ; do
    sed -ri "s,^($i)=.*,\1=111," $TEST_CONF
  done

  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  for i in EP_GID AS_UID AS_RUNUID EPAGESJ_UID WS_GID WS_UID WS_RUNUID ; do
    grep -q "/etc/sysconfig/epages6 -k EPAGES_INSTALL_$i -v '111'" $TEST_OUTPUT
    assertTrue "[FAILED] $SCRIPT_TO_TEST: EPAGES_INSTALL_$i=111" $?
  done
}

function test_configfile_void_gid() {
  # conf: epages=sss
  sed -ri 's,^(epages)=.*,\1=sss,' $TEST_CONF

  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q 'ID must be a number' $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: epages=sss" $?
}

function test_configfile_void_uid() {
  # conf: ep_appl=sss
  sed -ri 's,^(ep_appl)=.*,\1=sss,' $TEST_CONF

  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q 'ID must be a number' $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: ep_appl=sss" $?
}

function test_configfile_localhost() {
  # conf: ip=1.2.3.4 hostname=shop.example
  cp ${TEST_DIR%/*}/conf/install-epages.conf.sample $TEST_CONF
  sed -ri 's,^(ip)=.*,\1=1.2.3.4,' $TEST_CONF
  sed -ri 's,^(hostname)=.*,\1=shop.example,' $TEST_CONF
  sed -ri 's,^(patchhost)=.*,\1=0,' $TEST_CONF

  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q "/etc/sysconfig/epages6 -k EPAGES_INSTALL_AS_HOST -v 'shop.example'" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: hostname=shop.example" $?
  grep -q "/etc/sysconfig/epages6 -k EPAGES_INSTALL_AS_HOSTIP -v '1.2.3.4'" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: ip=1.2.3.4" $?
  grep -q "^INSTALL_EPAGES_MYSQL_CONNECTION=1.2.3.4" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: hostname=shop.example" $?
  grep -q "^INSTALL_EPAGES_MYSQL_HOST=shop.example" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: ip=1.2.3.4" $?
}

function test_configfile_distributed_parts() {
  # conf: appsrv=0 j=0 mongo=arbiter nlb=1
  local EXPECTED='ascsrv mysqld reqsrv webconf'
  sed -ri 's,^(appsrv)=.*,\1=0,' $TEST_CONF
  sed -ri 's,^(j)=.*,\1=0,' $TEST_CONF
  sed -ri 's,^(mongo)=.*,\1=arbiter,' $TEST_CONF
  sed -ri 's,^(nlb)=.*,\1=1,' $TEST_CONF

  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q "INCLUDE=$EXPECTED" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: INCLUDE=$EXPECTED" $?
  grep -q "install mongo" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: install mongo" $?
  grep -q "install nginx" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: install nginx" $?
}

function test_configfile_cmdline_distributed_parts() {
  # args --distributed 'j mysqld'
  local EXPECTED='j mysqld'
  local CMD="$SCRIPT_TO_TEST --distributed '$EXPECTED' --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --distributed "$EXPECTED" --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q "INCLUDE=$EXPECTED" $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_cmdline_distributed_no_j() {
  # args --distributed 'epages -j'
  local CMD="$SCRIPT_TO_TEST --distributed 'epages -j' --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --distributed 'epages -j' --configfile $TEST_CONF --mock >$TEST_OUTPUT
  local EXPECTED='appsrv ascsrv mysqld reqsrv webconf'
  grep -q "INCLUDE=$EXPECTED" $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_cmdline_singlehost() {
  # args --singlehost
  local CMD="$SCRIPT_TO_TEST --singlehost --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --singlehost --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q 'install epages' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_missing_nfsserver() {
  # conf: fileserver=client
  sed -ri 's,^(fileserver)=.*,\1=client,' $TEST_CONF
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  local EXPECTED='no fileserver=server for .localhost.:fileserver=client'
  grep -q "$EXPECTED" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: $EXPECTED" $?
}

function test_configfile_cmdline_nfsclient() {
  # args: --fileserver nfsserver
  local CMD="$SCRIPT_TO_TEST --fileserver nfsserver --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --fileserver nfsserver --configfile $TEST_CONF --mock >$TEST_OUTPUT
  local EXPECTED='/var/epages/epages-fs.sh -mount nfsserver'
  grep -q "$EXPECTED" $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_cmdline_nfsserver() {
  # args: --fileserver
  local CMD="$SCRIPT_TO_TEST --fileserver --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --fileserver --configfile $TEST_CONF --mock >$TEST_OUTPUT
  local EXPECTED='/var/epages/epages-fs.sh -share'
  grep -q '^FILESERVER=$' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_configfile_nfsserver() {
  # conf: fileserver=server
  sed -ri 's,^(fileserver)=.*,\1=server,' $TEST_CONF
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  local EXPECTED='/var/epages/epages-fs.sh -share'
  grep -q "$EXPECTED" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: $EXPECTED" $?
}

function test_configfile_mysql_password_empty() {
  # conf: mysqld-root-password=
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q "/etc/sysconfig/epages6 -k EPAGES_INSTALL_MYSQL_PASS -v '' -i" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: mysqld-root-password=" $?
}

function test_configfile_mysql_password_missing() {
  # conf: #mysqld-root-password=
  sed -ri 's,^(mysqld-root-password=.*),#\1,' $TEST_CONF
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  # without mysqld-root-password -> EPAGES_INSTALL_MYSQL_PASS must not be set
  ! grep -q EPAGES_INSTALL_MYSQL_PASS $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: #mysqld-root-password=" $?
}

function test_configfile_mysql_password_ignored() {
  # conf: mysqld=0
  sed -ri 's,^(mysqld)=.*,\1=0,' $TEST_CONF
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  # with mysqld=0 -> EPAGES_INSTALL_MYSQL_PASS must not be set
  ! grep -q EPAGES_INSTALL_MYSQL_PASS $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: mysqld=0" $?
}

function test_configfile_unique_fileserver() {
  # conf: fileserver=server can be set only once
  echo -e "[otherhost]\nip=3.4.5.6\nhostname=shop1\nappsrv=1\nfileserver=server" >> $TEST_CONF
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  local EXPECTED='fileserver=server can be set only once'
  grep -q "$EXPECTED" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: $EXPECTED" $?
}

function test_configfile_unique_patchhost() {
  # conf: patchhost=1 can be set only once
  local LINES=$(wc -l $TEST_CONF | awk '{print $1}')
  head -$(($LINES - 1)) $TEST_CONF > $TEST_CONF.1 && cat $TEST_CONF.1 > $TEST_CONF
  sed -ri 's,^(fileserver)=.*,\1=client,' $TEST_CONF
  sed -ri 's,^(patchhost)=.*,\1=1,' $TEST_CONF
  echo -e "fileserver=server\npatchhost=1" >> $TEST_CONF

  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  local EXPECTED='patchhost=1 can be set only once'
  grep -q "$EXPECTED" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: $EXPECTED" $?
}

function test_configfile_otherhost_nfsserver() {
  # conf: [otherhost]
  local LINES=$(wc -l $TEST_CONF | awk '{print $1}')
  head -$(($LINES - 1)) $TEST_CONF > $TEST_CONF.1 && cat $TEST_CONF.1 > $TEST_CONF
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  local EXPECTED='FILESERVER=shop1'
  grep -q "^$EXPECTED$" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: $EXPECTED" $?
}

function test_configfile_hostname() {
  # conf: [$(hostname)] has higher prio than [localhost]
  sed -ri "s,otherhost,$(hostname)," $TEST_CONF
  $SCRIPT_TO_TEST --configfile $TEST_CONF --mock >$TEST_OUTPUT
  grep -q "^MYSECTION=$(hostname)$" $TEST_OUTPUT && grep -q "^FILESERVER=" $TEST_OUTPUT \
  grep -q "^DISTRIBUTED=appsrv$" $TEST_OUTPUT
  assertTrue "[FAILED] $SCRIPT_TO_TEST: [$(hostname)]" $?
}

function test_mysqld_complex_config() {
  local TEST_CONFD=/tmp/${0##*/}.~$$~
  rm -rf $TEST_CONFD
  cp -rp ${TEST_DIR%/*}/conf $TEST_CONFD

  local TEST_CONF=$TEST_CONFD/install-epages.conf.sample
  test -r $TEST_CONF
  assertTrue "[FAILED] test -r $TEST_CONF" $? || exit 1

  sed -i 's,mysqld/simple.conf,mysqld/complex.conf,' $TEST_CONF

  type get_ini_value >/dev/null 2>&1
  assertTrue "[FAILED] type get_ini_value" $? || return

  local MYSQLD_CONF=$(get_ini_value $TEST_CONF mysqld localhost)
  local EXPECTED=mysqld/complex.conf
  assertEquals "[FAILED] get_ini_value $TEST_CONF mysqld localhost" $EXPECTED $MYSQLD_CONF

  MYSQLD_CONF=$(absolute_path $MYSQLD_CONF $TEST_CONF)
  assertTrue "[FAILED] absolute_path $MYSQLD_CONF $TEST_CONF" $? || return
  local EXPECTED=$TEST_CONFD/mysqld/complex.conf
  assertEquals "[FAILED] absolute_path $MYSQLD_CONF $TEST_CONF" $EXPECTED $MYSQLD_CONF
  test -r $MYSQLD_CONF
  assertTrue "[FAILED] test -r $MYSQLD_CONF" $? || return

  local CMD="$SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock > $TEST_OUTPUT 2>&1
  grep -q 'install epages-mysqld' $TEST_OUTPUT
  assertTrue "[FAILED] install epages-mysqld" $?
  grep -q 'EPAGES_INSTALL_STORES .* Store StoreenUS StoreptBR' $TEST_OUTPUT
  assertTrue "[FAILED] EPAGES_INSTALL_STORES='Store StoreenUS StoreptBR'" $?

  cat > $TEST_OUTPUT.expected <<EOF
[Site]
StoreType=Site
Key=1234567890abcdef
Backup=Backup
StoreType.Add=DE_EPAGES::Diagnostics
StoreType.Lang=en-US ca cs da de es fi fr it nl no ru sv pt-BR
[Store]
StoreType=Store
Key=1234567890abcdef
Backup=Backup
StoreType.Add=DE_EPAGES::Diagnostics DE_EPAGES::ELogistics DE_EPAGES::Envialia DE_EPAGES::TagCommander
StoreType.Lang=en ca cs da de es fi fr it nl no pt ru sv
[StoreenUS]
StoreType=StoreenUS
Backup=Backup
StoreType.Base=Store
StoreType.Lang=en-US
[StoreptBR]
StoreType=StoreptBR
Backup=Backup
StoreType.Base=Store
StoreType.Lang=pt-BR
[System]
StoreType=System
Backup=Backup
[Monitor]
StoreType=Monitor
Backup=Backup
StoreType.Base=Common
StoreType.Add=DE_EPAGES::Calendar DE_EPAGES::Address DE_EPAGES::Shop DE_EPAGES::MediaGallery DE_EPAGES::Design DE_EPAGES::ShopMailType DE_EPAGES::ShopCSVExportImport DE_EPAGES::Content DE_EPAGES::ShopWizard DE_EPAGES::SEO DE_EPAGES::Forum DE_EPAGES::Tax DE_EPAGES::Customer DE_EPAGES::Shipping DE_EPAGES::Payment DE_EPAGES::Order DE_EPAGES::ShippingOption DE_EPAGES::Product DE_EPAGES::Catalog DE_EPAGES::ProductRating DE_EPAGES::ShoppingList DE_EPAGES::ProductComparison DE_EPAGES::RemoteSearch DE_EPAGES::ShippingTypes DE_EPAGES::PaymentTypes DE_EPAGES::CustomForm
[Backup]
Database=mysql
login=root
[ASPoolDB]
StoreType=ASPoolDB
Database=aspooldb
Backup=Backup
[epagesj]
Database=epagesj
Backup=Backup
[PatchSystemInfoDB]
Database=patchsysteminfodb
login=usr_patch
Backup=Backup
EOF

  local INI_CONFIG=$(find_ini_config)
  $INI_CONFIG -f $MYSQLD_CONF -G | while read section ; do
    echo "[$section]"
    $INI_CONFIG -f $MYSQLD_CONF -s "$section" -g
  done > $TEST_OUTPUT.found
  echo "-> $INI_CONFIG -f $MYSQLD_CONF -g"
  cmp $TEST_OUTPUT.found $TEST_OUTPUT.expected
  assertTrue "[FAILED] cmp $TEST_OUTPUT.found $TEST_OUTPUT.expected" $?
}

function test_mysqld_simple_config() {
  local TEST_CONFD=/tmp/${0##*/}.~$$~
  rm -rf $TEST_CONFD
  cp -rp ${TEST_DIR%/*}/conf $TEST_CONFD

  local TEST_CONF=$TEST_CONFD/install-epages.conf.sample
  test -r $TEST_CONF
  assertTrue "[FAILED] test -r $TEST_CONF" $? || exit 1

  type get_ini_value >/dev/null 2>&1
  assertTrue "[FAILED] type get_ini_value" $? || return

  local MYSQLD_CONF=$(get_ini_value $TEST_CONF mysqld localhost)
  local EXPECTED=mysqld/simple.conf
  assertEquals "[FAILED] get_ini_value $TEST_CONF mysqld localhost" $EXPECTED $MYSQLD_CONF

  MYSQLD_CONF=$(absolute_path $MYSQLD_CONF $TEST_CONF)
  assertTrue "[FAILED] absolute_path $MYSQLD_CONF $TEST_CONF" $? || return
  local EXPECTED=$TEST_CONFD/mysqld/simple.conf
  assertEquals "[FAILED] absolute_path $MYSQLD_CONF $TEST_CONF" $EXPECTED $MYSQLD_CONF
  test -r $MYSQLD_CONF
  assertTrue "[FAILED] test -r $MYSQLD_CONF" $? || return

  local CMD="$SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock 2>&1 \
  | grep -q 'install epages-mysqld'
  assertTrue "[FAILED] $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock" $?

  cat > $TEST_OUTPUT.expected <<EOF
[general definitions]
my-cnf=
[Site]
StoreType=Site
Database=sitedb
login=usr_sitedb
passwd=epages
Backup=Backup
BackupHost=localhost
[Store]
StoreType=Store
Backup=Backup
[System]
StoreType=System
Backup=Backup
[Backup]
Database=mysql
login=root
[ASPoolDB]
StoreType=ASPoolDB
Database=aspooldb
Backup=Backup
[epagesj]
Database=epagesj
Backup=Backup
[PatchSystemInfoDB]
Database=patchsysteminfodb
login=usr_patch
Backup=Backup
EOF

  local INI_CONFIG=$(find_ini_config)
  $INI_CONFIG -f $MYSQLD_CONF -G | while read section ; do
    echo "[$section]"
    $INI_CONFIG -f $MYSQLD_CONF -s "$section" -g
  done > $TEST_OUTPUT.found
  echo "-> $INI_CONFIG -f $MYSQLD_CONF -g"
  cmp $TEST_OUTPUT.found $TEST_OUTPUT.expected
  assertTrue "[FAILED] cmp $TEST_OUTPUT.found $TEST_OUTPUT.expected" $?
}

function test_mysqld_configfile_password() {
  local TEST_CONFD=/tmp/${0##*/}.~$$~
  local TEST_CONF=$TEST_CONFD/install-epages.conf.sample
  test -r $TEST_CONF
  assertTrue "[FAILED] test -r $TEST_CONF" $? || exit 1

  local MYSQLD_CONF=$(get_ini_value $TEST_CONF mysqld localhost)
  MYSQLD_CONF=$(absolute_path $MYSQLD_CONF $TEST_CONF)
  test -r $MYSQLD_CONF
  assertTrue "[FAILED] test -r $MYSQLD_CONF" $? || return

  local INI_CONFIG=$(find_ini_config)

  # use mysqld-root-password from base config if server config is not defined
  $INI_CONFIG -f $TEST_CONF -s 'general definitions' -k mysqld-root-password -v abcde -i
  local CMD="$SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock"
  $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock 2>&1 \
  | grep -q "EPAGES_INSTALL_MYSQL_PASS -v 'abcde'"
  assertTrue "[FAILED] $TEST_CONF: [general definition] mysqld-root-password=abcde" $?

  # use empty mysqld-root-password from base config if server config is not defined
  $INI_CONFIG -f $TEST_CONF -s 'general definitions' -k mysqld-root-password -v '' -i
  local CMD="$SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock"
  $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock 2>&1 \
  | grep -q "EPAGES_INSTALL_MYSQL_PASS -v '' -i"
  assertTrue "[FAILED] $TEST_CONF: [general definition] mysqld-root-password=" $?

  # no mysqld-root-password in base config or server config
  sed -i 's,mysqld-root-password.*,,' $TEST_CONF
  local CMD="$SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock"
  $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock 2>&1 \
  | grep -q EPAGES_INSTALL_MYSQL_PASS
  assertFalse "[FAILED] $TEST_CONF: [general definition] mysqld-root-password=(unset)" $?

  # use empty mysqld-root-password from server config
  $INI_CONFIG -f $TEST_CONF -s 'general definitions' -k mysqld-root-password -v abcde -i
  $INI_CONFIG -f $MYSQLD_CONF -s 'general definitions' -k mysqld-root-password -v '' -i
  local CMD="$SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock"
  $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock 2>&1 \
  | grep -q "EPAGES_INSTALL_MYSQL_PASS -v '' -i"
  assertTrue "[FAILED] $TEST_CONF: [general definition] mysqld-root-password=abcde" $?

  # use mysqld-root-password from server config
  $INI_CONFIG -f $MYSQLD_CONF -s 'general definitions' -k mysqld-root-password -v 123 -i
  local CMD="$SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock"
  $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock 2>&1 \
  | grep -q "EPAGES_INSTALL_MYSQL_PASS -v '123'"
  assertTrue "[FAILED] $TEST_CONF: [general definition] mysqld-root-password=abcde" $?
}

function test_configfile_setupmysql() {
  local TEST_CONFD=/tmp/${0##*/}.~$$~
  local TEST_CONF=$TEST_CONFD/install-epages.conf.sample
  sed -i "/^preinstalled=/d" $TEST_CONF
  test -r $TEST_CONF
  assertTrue "[FAILED] test -r $TEST_CONF" $? || exit 1

  local MYSQLD_CONF=$(get_ini_value $TEST_CONF mysqld localhost)
  MYSQLD_CONF=$(absolute_path $MYSQLD_CONF $TEST_CONF)
  test -r $MYSQLD_CONF
  assertTrue "[FAILED] test -r $MYSQLD_CONF" $? || return

  local INI_CONFIG=$(find_ini_config)

  # default values for EPAGES_ROOT_DATABASES and INSTALL_EPAGES_ASPOOLDB
  $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock >$TEST_OUTPUT 2>&1
  grep -q '^EPAGES_ROOT_DATABASES=Backup$' $TEST_OUTPUT
  assertTrue "[FAILED] $TEST_CONF: EPAGES_ROOT_DATABASES=Backup" $?
  grep -q '^INSTALL_EPAGES_ASPOOLDB=1' $TEST_OUTPUT
  assertTrue "[FAILED] $TEST_CONF: INSTALL_EPAGES_ASPOOLDB=1" $?
  grep -q "/var/epages/preinstall/.*$TEST_CONF" $TEST_OUTPUT
  assertTrue "[FAILED] /var/epages/preinstall/.*$TEST_CONF" $?

  # add Backup2 as EPAGES_ROOT_DATABASES and remove ASPoolDB section
  echo -e "[Backup2]\nlogin=root" >> $MYSQLD_CONF
  sed -i '/ASPoolDB/d' $MYSQLD_CONF
  $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock >$TEST_OUTPUT 2>&1
  grep -q '^EPAGES_ROOT_DATABASES=Backup Backup2$' $TEST_OUTPUT
  assertTrue "[FAILED] $TEST_CONF: EPAGES_ROOT_DATABASES='Backup Backup2'" $?
  grep -q '^INSTALL_EPAGES_ASPOOLDB=$' $TEST_OUTPUT
  assertTrue "[FAILED] $TEST_CONF: INSTALL_EPAGES_ASPOOLDB=" $?

  # unset EPAGES_ROOT_DATABASES
  $INI_CONFIG -f $MYSQLD_CONF -s Backup -k login -v kwats -i
  $INI_CONFIG -f $MYSQLD_CONF -s Backup2 -k login -v kwats -i
  $SCRIPT_TO_TEST --distributed mysqld --configfile $TEST_CONF --mock 2>&1 \
  | grep -q '^EPAGES_ROOT_DATABASES='
  assertFalse "[FAILED] $TEST_CONF: EPAGES_ROOT_DATABASES=" $?

  rm -rf $TEST_CONFD
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
