#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods
##############################################################################

# die if shunit2 not found
type shunit2 && SHUNIT2=$(type -p shunit2) || exit 1

# test script directory (/var/epages/t)
TEST_DIR=$(cd $(dirname $0) && pwd)

# script to test (/var/epages/install-epages.sh)
SCRIPT_TO_TEST=${TEST_DIR%/*}/install-epages.sh

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  echo '-> export EPAGES_INSTALLATION_MOCK=1'
}

function setUp() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_INSTALLATION
  TEST_OUTPUT=/tmp/${0##*/}
  TEST_OUTPUT=${TEST_OUTPUT%.sh}.txt.~$$~
}

function oneTimeTearDown() {
  if [[ -n $KEEP_TEST_CONF ]] ; then
    echo "test_outputfile: $TEST_OUTPUT"
  else
    rm -rf $TEST_OUTPUT
  fi
}

##############################################################################
# define functions                                                           #
##############################################################################

function test_script_syntax() {
  # script must have correct syntax
  bash -n $SCRIPT_TO_TEST
  assertTrue "[FAILED] bash -n $SCRIPT_TO_TEST" $? || exit 1
}

function test_script_executable() {
  # script must be executable
  test -x $SCRIPT_TO_TEST
  assertTrue "[FAILED] test -x $SCRIPT_TO_TEST" $?
  chmod 755 $SCRIPT_TO_TEST
}

function test_distributed() {
  # args: --distributed
  local CMD="$SCRIPT_TO_TEST --distributed --mock"
  echo "-> $CMD"
  local EXPECTED='appsrv ascsrv j mysqld reqsrv webconf'
  $SCRIPT_TO_TEST --distributed --mock | grep -q "INCLUDE=$EXPECTED"
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_no_epages() {
  # args: --distributed -epages
  local CMD="$SCRIPT_TO_TEST --distributed -epages --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --distributed -epages --mock 2>&1 | grep -q '^DISTRIBUTED=-epages$'
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_no_mysqld() {
  # args: --distributed 'epages -mysqld'
  local CMD="$SCRIPT_TO_TEST --distributed 'epages -mysqld' --mock"
  echo "-> $CMD"
  local EXPECTED='appsrv ascsrv j reqsrv webconf'
  $SCRIPT_TO_TEST --distributed 'epages -mysqld' --mock 2>&1 | grep -q "INCLUDE=$EXPECTED"
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_patchhost() {
  # patchhost starts epages installation
  local INSTALLER=yum
  [[ -f /etc/debian_version ]] && INSTALLER=apt-get
  local INSTALL=patchhost
  local CMD="$SCRIPT_TO_TEST --distributed $INSTALL --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --distributed $INSTALL --mock 2>&1 \
  | grep -q "IGNORE_INI= $INSTALLER -y install epages-appsrv"
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_appsrv() {
  # appsrv does not start epages installation
  local INSTALLER=yum
  [[ -f /etc/debian_version ]] && INSTALLER=apt-get
  local INSTALL=appsrv
  local CMD="$SCRIPT_TO_TEST --distributed $INSTALL --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --distributed $INSTALL --mock 2>&1 \
  | grep -q "IGNORE_INI=1 $INSTALLER -y install epages-appsrv"
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_void_package() {
  # args: --distributed kwats
  local CMD="$SCRIPT_TO_TEST --distributed kwats --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --distributed kwats --mock 2>&1 | grep -q 'kwats: not a valid .* package name'
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_logfile() {
  # args: --distributed --version kwats
  local CMD="$SCRIPT_TO_TEST --distributed --version kwats"
  echo "-> $CMD"
  LOGFILE=$($SCRIPT_TO_TEST --distributed --version kwats 2>&1 | awk '/^LOGFILE:/{print $2}')
  grep -q 'version must contain only numbers and dots' $LOGFILE
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_old() {
  # args: --distributed --version 6.17.0
  local CMD="$SCRIPT_TO_TEST --distributed --version 6.17.0 --mock"
  echo "-> $CMD"
  local EXPECTED='appsrv j mysqld reqsrv webconf'
  $SCRIPT_TO_TEST --distributed --version 6.17.0 --mock | grep -q "INCLUDE=$EXPECTED"
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_veryold() {
  # args: --distributed --version 6.12.0
  local CMD="$SCRIPT_TO_TEST --distributed --version 6.12.0 --mock"
  echo "-> $CMD"
  local EXPECTED='appsrv mysqld reqsrv webconf'
  $SCRIPT_TO_TEST --distributed --version 6.12.0 --mock 2>&1 | grep -q "INCLUDE=$EXPECTED"
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_nfsserver() {
  # args: --fileserver
  local CMD="$SCRIPT_TO_TEST --fileserver --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --fileserver --mock > $TEST_OUTPUT
  grep -q '^FILESERVER=$' $TEST_OUTPUT && grep -q '^DISTRIBUTED=-epages$' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_nfsserver_parts() {
  # args: --fileserver --distributed 'j mysqld'
  local CMD="$SCRIPT_TO_TEST --fileserver --distributed 'j mysqld' --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --fileserver --distributed 'j mysqld' --mock > $TEST_OUTPUT
  grep -q '^FILESERVER=$' $TEST_OUTPUT && grep -q '^DISTRIBUTED=j mysqld$' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_nfsclient() {
  # args: --fileserver nfshost
  local CMD="$SCRIPT_TO_TEST --fileserver nfshost --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --fileserver nfshost --mock > $TEST_OUTPUT
  grep -q '^FILESERVER=nfshost$' $TEST_OUTPUT && grep -q '^DISTRIBUTED=-epages$' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_distributed_nfsclient_parts() {
  # args: --fileserver nfshost --distributed 'j mysqld'
  local CMD="$SCRIPT_TO_TEST --fileserver nfshost --distributed 'j mysqld' --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --fileserver nfshost --distributed 'j mysqld' --mock > $TEST_OUTPUT
  grep -q '^FILESERVER=nfshost$' $TEST_OUTPUT && grep -q '^DISTRIBUTED=j mysqld$' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_distributed() {
  # EPAGES_INSTALLATION=distributed -> same as: --distributed
  EPAGES_INSTALLATION=distributed
  local CMD="export EPAGES_INSTALLATION=$EPAGES_INSTALLATION ; cat $SCRIPT_TO_TEST | bash"
  echo "-> $CMD"
  local EXPECTED='appsrv ascsrv j mysqld reqsrv webconf'
  cat $SCRIPT_TO_TEST | bash | grep -q "INCLUDE=$EXPECTED"
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_distributed_parts() {
  #  EPAGES_INSTALLATION='j mysqld' -> same as: --distributed 'j mysqld'
  local EXPECTED='j mysqld'
  EPAGES_INSTALLATION=$EXPECTED
  local CMD="export EPAGES_INSTALLATION='$EPAGES_INSTALLATION' ; cat $SCRIPT_TO_TEST | bash"
  echo "-> $CMD"
  cat $SCRIPT_TO_TEST | bash | grep -q "INCLUDE=$EXPECTED"
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_distributed_nfsserver() {
  # EPAGES_INSTALLATION=fileserver -> same as: --fileserver
  EPAGES_INSTALLATION=fileserver
  local CMD="export EPAGES_INSTALLATION=$EPAGES_INSTALLATION ; cat $SCRIPT_TO_TEST | bash"
  echo "-> $CMD"
  cat $SCRIPT_TO_TEST | bash > $TEST_OUTPUT
  grep -q '^FILESERVER=$' $TEST_OUTPUT && grep -q '^DISTRIBUTED=-epages$' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_distributed_nfsserver_parts() {
  # EPAGES_INSTALLATION='fileserver j mysqld' -> same as: --fileserver --distributed 'j mysqld'
  EPAGES_INSTALLATION='fileserver j mysqld'
  local CMD="export EPAGES_INSTALLATION='$EPAGES_INSTALLATION' ; cat $SCRIPT_TO_TEST | bash"
  echo "-> $CMD"
  cat $SCRIPT_TO_TEST | bash > $TEST_OUTPUT
  grep -q '^FILESERVER=$' $TEST_OUTPUT && grep -q '^DISTRIBUTED=j mysqld$' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_distributed_nfsclient() {
  # EPAGES_INSTALLATION=fileserver=nfshost -> same as: --fileserver nfshost
  EPAGES_INSTALLATION=fileserver=nfshost
  local CMD="export EPAGES_INSTALLATION=$EPAGES_INSTALLATION ; cat $SCRIPT_TO_TEST | bash"
  echo "-> $CMD"
  cat $SCRIPT_TO_TEST | bash > $TEST_OUTPUT
  grep -q '^FILESERVER=nfshost$' $TEST_OUTPUT && grep -q '^DISTRIBUTED=-epages$' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_distributed_nfsclient_parts() {
  # EPAGES_INSTALLATION='fileserver=nfshost j mysqld'
  # -> same as: --fileserver nfshost --distributed 'j mysqld'
  EPAGES_INSTALLATION='fileserver=nfshost j mysqld'
  local CMD="export EPAGES_INSTALLATION='$EPAGES_INSTALLATION' ; cat $SCRIPT_TO_TEST | bash"
  echo "-> $CMD"
  cat $SCRIPT_TO_TEST | bash > $TEST_OUTPUT
  grep -q '^FILESERVER=nfshost$' $TEST_OUTPUT && grep -q '^DISTRIBUTED=j mysqld$' $TEST_OUTPUT
  assertTrue "[FAILED] $CMD" $?
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
