#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods
##############################################################################

# die if shunit2 not found
type shunit2 && SHUNIT2=$(type -p shunit2) || exit 1

# test script directory (/var/epages/t)
TEST_DIR=$(cd $(dirname $0) && pwd)

# script to test (/var/epages/install-epages.sh)
SCRIPT_TO_TEST=${TEST_DIR%/*}/install-epages.sh

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  echo '-> export EPAGES_INSTALLATION_MOCK=1'
}

function setUp() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_INSTALLATION
}

##############################################################################
# define functions                                                           #
##############################################################################

function test_script_syntax() {
  # script must have correct syntax
  bash -n $SCRIPT_TO_TEST
  assertTrue "[FAILED] bash -n $SCRIPT_TO_TEST" $? || exit 1
}

function test_script_executable() {
  # script must be executable
  test -x $SCRIPT_TO_TEST
  assertTrue "[FAILED] test -x $SCRIPT_TO_TEST" $?
  chmod 755 $SCRIPT_TO_TEST
}

function test_wrong_interpreter() {
  # script must be executed by bash
  sh $SCRIPT_TO_TEST 2>&1 | grep -q 'only.*bash'
  assertTrue "[FAILED] sh $SCRIPT_TO_TEST" $?
}

function test_no_arguments() {
  # script requires arguments
  local CMD="$SCRIPT_TO_TEST"
  echo "-> $CMD"
  $SCRIPT_TO_TEST 2>&1 | grep -q choose
  assertTrue "[FAILED] $CMD" $?
}

function test_wrong_arguments() {
  # script requires correct arguments
  local CMD="$SCRIPT_TO_TEST --kwats"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --kwats 2>&1 | grep -q 'Void option: --kwats'
  assertTrue "[FAILED] $CMD" $?
}

function test_wrong_version() {
  # script requires correct arguments
  local CMD="$SCRIPT_TO_TEST --singlehost --version 1.2.3"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --singlehost --version 1.2.3 2>&1 | grep -q 'version must be at least'
  assertTrue "[FAILED] $CMD" $?
}

function test_singlehost() {
  # args: --singlehost
  local CMD="$SCRIPT_TO_TEST --singlehost --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --singlehost --mock | grep -q 'install epages'
  assertTrue "[FAILED] $CMD" $?
}

function test_repo() {
  # args: --singlehost --repo foobar
  local CMD="$SCRIPT_TO_TEST --singlehost --repo foobar --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --singlehost --repo foobar --mock | grep -q 'REPO=foobar'
  assertTrue "[FAILED] $CMD" $?
}

function test_missing_repo() {
  # args: --singlehost --repo
  local CMD="$SCRIPT_TO_TEST --singlehost --repo --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --singlehost --repo --mock 2>&1 | grep -q 'Option requires 1 argument'
  assertTrue "[FAILED] $CMD" $?
}

function test_version() {
  # args: --singlehost --version 6.17.40
  local CMD="$SCRIPT_TO_TEST --singlehost --version 6.17.40 --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --singlehost --version 6.17.40 --mock | grep -q 'VERSION=6.17.40'
  assertTrue "[FAILED] $CMD" $?
}

function test_missing_version() {
  # args: --singlehost --version
  local CMD="$SCRIPT_TO_TEST --singlehost --version --mock"
  echo "-> $CMD"
  $SCRIPT_TO_TEST --singlehost --version --mock 2>&1 | grep -q 'Option requires 1 argument'
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_wrong_interpreter() {
  # script must be executed by bash in pipe
  local CMD="cat $SCRIPT_TO_TEST | sh"
  echo "-> $CMD"
  cat $SCRIPT_TO_TEST | sh | grep -q 'only.*bash'
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_xtrace() {
  # write each command to standard error (preceded by '+') before it is executed
  local CMD="cat $SCRIPT_TO_TEST | bash -x"
  echo "-> $CMD"
  cat $SCRIPT_TO_TEST | bash -x 2>&1 | grep -q '^+ exit 0$'
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_curl() {
  # get install-epages.sh from web server
  local CMD="curl -s http://www.epages.com/repo/epages/install-epages.sh | bash"
  echo "-> $CMD"
  curl -s http://www.epages.com/repo/epages/install-epages.sh | bash | grep -q '^SINGLEHOST=1$'
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_no_environment() {
  # EPAGES_INSTALLATION is unset -> same as: --singlehost
  EPAGES_INSTALLATION=
  local CMD="cat $SCRIPT_TO_TEST | bash"
  echo "-> $CMD"
  cat $SCRIPT_TO_TEST | bash | grep -q '^SINGLEHOST=1$'
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_void_environment() {
  # EPAGES_INSTALLATION=kwats -> throw error
  EPAGES_INSTALLATION=kwats
  local CMD="export EPAGES_INSTALLATION=$EPAGES_INSTALLATION ; cat $SCRIPT_TO_TEST | bash"
  echo "-> $CMD"
  cat $SCRIPT_TO_TEST | bash | grep -q 'kwats: not a valid .* package name'
  assertTrue "[FAILED] $CMD" $?
}

function test_pipe_singlehost() {
  # EPAGES_INSTALLATION=singlehost -> same as: --singlehost
  EPAGES_INSTALLATION=singlehost
  local CMD="export EPAGES_INSTALLATION=$EPAGES_INSTALLATION ; cat $SCRIPT_TO_TEST | bash"
  echo "-> $CMD"
  cat $SCRIPT_TO_TEST | bash | grep -q '^SINGLEHOST=1$'
  assertTrue "[FAILED] $CMD" $?
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
