#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods                                #
##############################################################################

# install if shunit2 not found
PATH=$PATH:/usr/share/shunit2
if ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/debian_version ]] ; then
  apt-get install shunit2
elif ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/redhat-release ]] ; then
  rpm -hiv http://download.fedoraproject.org/pub/epel/6/i386/shunit2-2.1.6-3.el6.noarch.rpm
fi

# die if shunit2 not found
SHUNIT2=$(type -p shunit2) || exit 1

# test script directory: /var/epages/t
TEST_DIR=$(cd $(dirname $0) && pwd)
TEST_FILE=$(basename $0)

# script to test: /var/epages/functions/${TEST_FILE%-test.sh}.sh
SCRIPT_TO_TEST=${TEST_DIR%/*}/functions/${TEST_FILE%-test.sh}.sh

. $SCRIPT_TO_TEST
. ${TEST_DIR%/*}/functions/fun-configure.sh
export get_install_host

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  TMP_DIR=/tmp/${0##*/}.~$$~
  TMP_CONF=$TMP_DIR/Config
  INSTALL_EPAGES_CONF=$TMP_DIR/conf/install-epages.conf.sample
  mkdir -p -m 755 $TMP_DIR
  rm -rf $TMP_DIR/conf
  cp -rp ${TEST_DIR%/*}/conf $TMP_DIR
}

function oneTimeTearDown() {
  if [[ -n $KEEP_TEST_CONF ]] ; then
    echo "test_configdir: $TMP_DIR"
  else
    rm -rf $TMP_DIR
  fi
}

##############################################################################
# test functions                                                             #
##############################################################################

function test_update_database_conf() {
  export EPAGES_CONFIG=$TMP_CONF

  # Store: new values are added
  unset INSTALL_EPAGES_HOST
  local CMD="update_database_conf Store"
  echo "-> $CMD"
  update_database_conf Store
  assertTrue "[FAILED] $CMD" $?

  echo 'active=1
Backup=Backup
BackupHost=localhost
Database=storedb
datasource=dbi:mysql:host=localhost
login=usr_storedb
passwd=epages
StoreType=Store' > $TMP_CONF/Database.d/Store.conf.expected

  grep = $TMP_CONF/Database.d/Store.conf | sort > $TMP_CONF/Database.d/Store.conf.sorted
  cmp $TMP_CONF/Database.d/Store.conf.expected $TMP_CONF/Database.d/Store.conf.sorted
  assertTrue "[FAILED] cmp $TMP_CONF/Database.d/Store.conf.*" $?

  # Store: existing values are not overwitten -> use dbi:mysql:host=localhost
  export INSTALL_EPAGES_HOST=$(get_install_host $INSTALL_EPAGES_CONF localhost)
  CMD="INSTALL_EPAGES_HOST=$INSTALL_EPAGES_HOST update_database_conf Store"
  echo "-> $CMD"
  update_database_conf Store
  assertTrue "[FAILED] $CMD" $?
  grep -q 'dbi:mysql:host=' $TMP_CONF/Database.d/Store.conf
  grep = $TMP_CONF/Database.d/Store.conf | sort > $TMP_CONF/Database.d/Store.conf.sorted
  cmp $TMP_CONF/Database.d/Store.conf.expected $TMP_CONF/Database.d/Store.conf.sorted
  assertTrue "[FAILED] cmp $TMP_CONF/Database.d/Store.conf.*" $?

  # Store: new values are added -> use dbi:mysql:host=$INSTALL_EPAGES_HOST
  echo "-> rm -f $TMP_CONF/Database.d/Store.conf"
  rm -f $TMP_CONF/Database.d/Store.conf
  echo "-> $CMD"
  update_database_conf Store
  assertTrue "[FAILED] $CMD" $?
  egrep -q 'dbi:mysql:host=[0-9]+\.' $TMP_CONF/Database.d/Store.conf
  assertTrue "[FAILED] egrep 'dbi:mysql:host=[0-9]+\.' Store.conf" $?

  # Store: existing values are modified
  CMD="update_database_conf Store passwd=1234"
  echo "-> $CMD"
  update_database_conf Store passwd=1234
  assertTrue "[FAILED] $CMD" $?
  grep -q passwd=1234 $TMP_CONF/Database.d/Store.conf
  assertTrue "[FAILED] grep passwd=1234 Store.conf" $?

  # Site: new values are added
  CMD="update_database_conf Site StoreType=FlexSite"
  echo "-> $CMD"
  update_database_conf Site StoreType=FlexSite
  assertTrue "[FAILED] $CMD" $?
  egrep -q 'dbi:mysql:host=[0-9]+\.' $TMP_CONF/Database.d/Site.conf
  assertTrue "[FAILED] egrep 'dbi:mysql:host=[0-9]+\.' Site.conf" $?
  grep -q StoreType=FlexSite $TMP_CONF/Database.d/Site.conf
  assertTrue "[FAILED] grep StoreType=FlexSite Site.conf" $?

  # Site: existing values are not overwitten
  CMD="update_database_conf Site"
  echo "-> $CMD"
  update_database_conf Site
  assertTrue "[FAILED] $CMD" $?
  grep -q StoreType=FlexSite $TMP_CONF/Database.d/Site.conf
  assertTrue "[FAILED] grep StoreType=FlexSite Site.conf" $?

  # Site: existing values are modified
  CMD="update_database_conf Site $TMP_DIR/conf/mysqld/simple.conf"
  echo "-> $CMD"
  update_database_conf Site $TMP_DIR/conf/mysqld/simple.conf
  assertTrue "[FAILED] $CMD" $?
  grep -q StoreType=Site $TMP_CONF/Database.d/Site.conf
  assertTrue "[FAILED] grep StoreType=Site Site.conf" $?

  # Backup: new values are added
  unset INSTALL_EPAGES_HOST
  export INSTALL_EPAGES_MYSQL_PASS=1234
  export INSTALL_EPAGES_MY_CNF=/etc/my.cnf
  CMD="update_database_conf Backup login=root Database=mysql"
  echo "-> $CMD"
  update_database_conf Backup login=root Database=mysql
  assertTrue "[FAILED] $CMD" $?

  echo 'active=1
Database=mysql
datasource=dbi:mysql:host=localhost
login=root
MySQLConfigFile=/etc/my.cnf
passwd=1234' > $TMP_CONF/Database.d/Backup.conf.expected

  grep = $TMP_CONF/Database.d/Backup.conf | sort > $TMP_CONF/Database.d/Backup.conf.sorted
  cmp $TMP_CONF/Database.d/Backup.conf.expected $TMP_CONF/Database.d/Backup.conf.sorted
  assertTrue "[FAILED] cmp $TMP_CONF/Database.d/Backup.conf.*" $?

  # Backup: existing values are modified
  CMD="update_database_conf Backup passwd=''"
  echo "-> $CMD"
  update_database_conf Backup passwd=''
  assertTrue "[FAILED] $CMD" $?
  grep -q '^passwd=$' $TMP_CONF/Database.d/Backup.conf
  assertTrue "[FAILED] grep passwd= Backup.conf" $?
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
