#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods
##############################################################################

# die if shunit2 not found
type shunit2 && SHUNIT2=$(type -p shunit2) || exit 1

# test script directory (/var/epages/t)
TEST_DIR=$(cd $(dirname $0) && pwd)
TEST_FILE=$(basename $0)
TEST_FILE=${TEST_FILE#post-}

# script to test
SCRIPT_TO_TEST=${TEST_DIR%/*}/postinstall/${TEST_FILE%-test.sh}.sh

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  TEMP=/tmp/${0##*/}.~$$~
  TEST_CONFD=$TEMP/eproot/Shared/Config
  INSTALL_EPAGES_CONF=$TEMP/conf/install-epages.conf.sample
  TEST_OUTPUT=$TEMP/test.output
  rm -rf $TEMP/*
  mkdir -m 755 -p ${TEST_CONFD%/*} $TEMP/eproot/Perl/bin $TEMP/etc/default
  cp -rp ${TEST_DIR%/*}/conf $TEMP

  # use Perl/bin/perl with changed EPAGES_CONFIG
  cp -p /srv/epages/eproot/Perl/bin/perl $TEMP/eproot/Perl/bin || exit 1
  /var/epages/ini-config.sh -f /etc/default/epages6 -k EPAGES_CONFIG -v $TEST_CONFD > $TEMP/etc/default/epages6
  /var/epages/ini-config.sh -f $TEMP/etc/default/epages6 -k PERL -v $TEMP/eproot/Perl/bin/perl -i

  chmod 644 $TEMP/etc/default/epages6
  cp -rp /srv/epages/eproot/Shared/Config $TEST_CONFD || exit 1
  rm -rf $TEST_CONFD/Database.d/* || exit 1
  chmod 755 $TEST_DIR/../*/*
}

function oneTimeTearDown() {
  if [[ -n $KEEP_TEST_CONF ]] ; then
    echo "test_configdir: $TEMP"
  else
    rm -rf $TEMP
  fi
}

##############################################################################
# define functions                                                           #
##############################################################################

#-----------------------------------------------------------------------------
# test functions
#-----------------------------------------------------------------------------

function test_script_syntax() {
  # script must have correct syntax
  bash -n $SCRIPT_TO_TEST
  assertTrue "[FAILED] bash -n $SCRIPT_TO_TEST" $? || exit 1
}

function test_script_executable() {
  # script must be executable
  test -x $SCRIPT_TO_TEST
  assertTrue "[FAILED] test -x $SCRIPT_TO_TEST" $?
  chmod 755 $SCRIPT_TO_TEST
  echo
}

function test_basic() {
  export EPAGES_INSTALLATION_MOCK=6.17.42
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  echo "-> export EPAGES_INSTALLATION_MOCK=$EPAGES_INSTALLATION_MOCK"
  echo "-> $TEST_DIR/../preinstall/mysqld.sh"
  $TEST_DIR/../preinstall/mysqld.sh

  cp -p $TEST_DIR/expected/Cartridges.xml $TEMP/eproot || exit 1
  export EPAGES_CARTRIDGES_XML=$TEMP/eproot/Cartridges.xml

  echo "-> $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF | tee $TEST_OUTPUT

  diff -wBq $EPAGES_CARTRIDGES_XML $TEST_DIR/expected/Cartridges.xml
  assertTrue "[FAILED] diff -wBq $EPAGES_CARTRIDGES_XML $TEST_DIR/expected/Cartridges.xml" $?

  # ePages 6.17.42 -> no update of TaxUSSystemDB
  grep -q 'TaxUSSystemDB/API/Zip2TaxUS.pm' $TEST_OUTPUT
  assertFalse "[FAILED] grep -q 'TaxUSSystemDB/API/Zip2TaxUS.pm'" $?
  grep -q 'TaxUSSystemDB/API/Table/Zip2TaxUSMeta.pm' $TEST_OUTPUT
  assertFalse "[FAILED] grep -q 'TaxUSSystemDB/API/Table/Zip2TaxUSMeta.pm'" $?

  for i in Site Store System ; do
    grep -q "installDatabase.pl -storename $i" $TEST_OUTPUT
    assertTrue "[FAILED] installDatabase.pl -storename $i" $?
  done
  grep -q "installDatabase.pl -storename Monitor" $TEST_OUTPUT
  assertFalse "[FAILED] installDatabase.pl -storename Monitor" $?

  # has no languages to install
  grep -q "installLanguageFiles.pl" $TEST_OUTPUT
  assertFalse "[FAILED] installLanguageFiles.pl" $?

  # therefore has no StoreTypes.d/6.17.42/Store.conf
  test -f $TEST_CONFD/StoreTypes.d/6.17.42/Store.conf
  assertFalse "[FAILED] test -d $TEST_CONFD/StoreTypes.d/6.17.42/Store.conf" $?
  echo
}

function test_hotfix() {
  export EPAGES_INSTALLATION_MOCK=6.17.30
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  cp -p $TEST_DIR/expected/Cartridges.xml $TEMP/eproot || exit 1
  export EPAGES_CARTRIDGES_XML=$TEMP/eproot/Cartridges.xml

  echo "-> export EPAGES_INSTALLATION_MOCK=$EPAGES_INSTALLATION_MOCK"
  echo "-> $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF | tee $TEST_OUTPUT
  diff -wBq $EPAGES_CARTRIDGES_XML $TEST_DIR/expected/Cartridges.xml
  assertTrue "[FAILED] diff -wBq $EPAGES_CARTRIDGES_XML $TEST_DIR/expected/Cartridges.xml" $?

  # ePages 6.17.30 -> update of TaxUSSystemDB
  grep -q 'TaxUSSystemDB/API/Zip2TaxUS.pm' $TEST_OUTPUT
  assertTrue "[FAILED] grep -q 'TaxUSSystemDB/API/Zip2TaxUS.pm'" $?
  grep -q 'TaxUSSystemDB/API/Table/Zip2TaxUSMeta.pm' $TEST_OUTPUT
  assertTrue "[FAILED] grep -q 'TaxUSSystemDB/API/Table/Zip2TaxUSMeta.pm'" $?
  echo
}

function test_standby() {
  export EPAGES_INSTALLATION_MOCK=1

  echo "-> export EPAGES_INSTALLATION_MOCK=$EPAGES_INSTALLATION_MOCK"
  echo "-> $SCRIPT_TO_TEST /tmp/standby"
  $SCRIPT_TO_TEST /tmp/standby | tee $TEST_OUTPUT
  grep -q 'EPAGES_SPEC_SKIP_RESTART=1 (/standby in /tmp/standby)' $TEST_OUTPUT
  assertTrue "[FAILED] grep -q 'EPAGES_SPEC_SKIP_RESTART=1 (/standby in /tmp/standby)'" $?
  echo
}

function test_no_config() {
  export EPAGES_INSTALLATION_MOCK=6.17.42
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  cp -p $TEST_DIR/expected/Cartridges.xml $TEMP/eproot || exit 1
  export EPAGES_CARTRIDGES_XML=$TEMP/eproot/Cartridges.xml

  echo "-> export EPAGES_INSTALLATION_MOCK=$EPAGES_INSTALLATION_MOCK"
  echo "-> $SCRIPT_TO_TEST foo/bar/kwats"
  $SCRIPT_TO_TEST foo/bar/kwats | tee $TEST_OUTPUT

  # no config file -> just run: make install
  grep -q 'make install' $TEST_OUTPUT
  assertTrue "[FAILED] grep -q 'make install'" $?
  echo
}

function test_complex() {
  export EPAGES_INSTALLATION_MOCK=6.17.42
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  sed -ri "s,^(mysqld=).*,\1mysqld/complex.conf," $INSTALL_EPAGES_CONF
  echo -e "[DE_EPAGES::AppStore]\nTokenMATe=1" >> $TEST_CONFD/epages.conf

  echo "-> export EPAGES_INSTALLATION_MOCK=$EPAGES_INSTALLATION_MOCK"
  echo "-> mysqld=mysqld/complex.conf"
  echo "-> $TEST_DIR/../preinstall/mysqld.sh"
  rm -rf $TEST_CONFD/Database.d/* $TEST_CONFD/StoreTypes.d/*
  $TEST_DIR/../preinstall/mysqld.sh $INSTALL_EPAGES_CONF

  cp -p $TEST_DIR/expected/Cartridges.xml $TEMP/eproot || exit 1
  export EPAGES_CARTRIDGES_XML=$TEMP/eproot/Cartridges.xml

  echo "-> $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF | tee $TEST_OUTPUT

  local EXPECTED=$(find $TEST_DIR/expected/Cartridges-complex.xml -printf "%s")
  local FOUND=$(find $EPAGES_CARTRIDGES_XML -printf "%s")
  assertEquals "[FAILED] diff -wBq $EPAGES_CARTRIDGES_XML $TEST_DIR/expected/Cartridges-complex.xml" "$EXPECTED" "$FOUND"

  grep -q "source $TEMP/conf/patchhost.conf" $TEST_OUTPUT
  assertTrue "[FAILED] source $TEMP/conf/patchhost.conf" $?

  # changed storetypes
  for i in Monitor Site Store StoreenUS StoreptBR ; do
    grep -q "update_cartridges_xml.sh -s $i" $TEST_OUTPUT
    assertTrue "[FAILED] update_cartridges_xml.sh -s $i" $?
  done
  # unchanged storetype
  grep -q "update_cartridges_xml.sh -s System" $TEST_OUTPUT
  assertFalse "[FAILED] update_cartridges_xml.sh -s System" $?

  for i in Monitor System ; do
    grep -q "EPAGES_LANGUAGES='' .*installDatabase.pl -storename $i" $TEST_OUTPUT
    assertTrue "[FAILED] installDatabase.pl -storename $i" $?
  done

  grep -q "EPAGES_LANGUAGES=.en-US .*installDatabase.pl -storename Site" $TEST_OUTPUT
  assertTrue "[FAILED] installDatabase.pl -storename Site" $?

  grep -q "EPAGES_LANGUAGES=.en .*installDatabase.pl -storename Store" $TEST_OUTPUT
  assertTrue "[FAILED] installDatabase.pl -storename Store" $?

  grep -q "EPAGES_LOCALES='en_US en_AU en_GB en_IE en_CA en_NZ'" $TEST_OUTPUT
  assertTrue "[FAILED] EPAGES_LOCALES='en_US en_AU en_GB en_IE en_CA en_NZ'" $?

  grep -q "EPAGES_LANGUAGES='en-US' .*installDatabase.pl -storename StoreenUS" $TEST_OUTPUT
  assertTrue "[FAILED] installDatabase.pl -storename StoreenUS" $?

  grep -q "EPAGES_LOCALES='pt_BR pt_PT'" $TEST_OUTPUT
  assertTrue "[FAILED] EPAGES_LOCALES='pt_BR pt_PT'" $?

  grep -q "EPAGES_LANGUAGES='pt-BR' .*installDatabase.pl -storename StoreptBR" $TEST_OUTPUT
  assertTrue "[FAILED] installDatabase.pl -storename StoreptBR" $?

  # run remoteSearch on stores with DE_EPAGES::RemoteSearch
  for i in Monitor Store StoreenUS StoreptBR ; do
    grep -q "remoteSearch.pl -storename $i" $TEST_OUTPUT
    assertTrue "[FAILED] remoteSearch.pl -storename $i" $?
  done
  for i in Site System ; do
    grep -q "remoteSearch.pl -storename $i" $TEST_OUTPUT
    assertFalse "[FAILED] remoteSearch.pl -storename $i" $?
  done

  # run importScripts.pl on stores with DE_EPAGES::AppStore
  for i in Store StoreenUS StoreptBR ; do
    grep -q "DE_EPAGES/AppStore/Scripts/importScripts.pl -storename $i" $TEST_OUTPUT
    assertTrue "[FAILED] DE_EPAGES/AppStore/Scripts/importScripts.pl -storename $i" $?
  done
  for i in Monitor Site System ; do
    grep -q "DE_EPAGES/AppStore/Scripts/importScripts.pl -storename $i" $TEST_OUTPUT
    assertFalse "[FAILED] DE_EPAGES/AppStore/Scripts/importScripts.pl -storename $i" $?
  done

  # insert business units into Site
  for i in Monitor Store StoreenUS StoreptBR ; do
    grep -q "store.pl -action insert -storename $i" $TEST_OUTPUT
    assertTrue "[FAILED] store.pl -action insert -storename $i" $?
  done
  # but not into Site or FlexStore
  for i in Site FlexStore System ; do
    grep -q "store.pl -action insert -storename $i" $TEST_OUTPUT
    assertFalse "[FAILED] store.pl -action insert -storename $i" $?
  done

  # run import.pl for stores with DE_EPAGES::Styles
  for i in Store StoreenUS StoreptBR ; do
    grep -q "import.pl -storename Site -path / /tmp/epages/epages-appsrv/$i-import.xml" $TEST_OUTPUT
    assertTrue "[FAILED] import.pl -storename Site -path / /tmp/epages/epages-appsrv/$i-import.xml" $?
  done
  for i in Monitor Site System ; do
    grep -q "import.pl -storename Site -path / /tmp/epages/epages-appsrv/$i-import.xml" $TEST_OUTPUT
    assertFalse "[FAILED] import.pl -storename Site -path / /tmp/epages/epages-appsrv/$i-import.xml" $?
  done

  # run make install_storetype for stores with DE_EPAGES::Shop or DE_EPAGES::Store
  for i in Monitor Store StoreenUS StoreptBR Site ; do
    grep -q "make install_storetype STORE_TYPE=$i" $TEST_OUTPUT
    assertTrue "[FAILED] make install_storetype STORE_TYPE=$i" $?
  done
  grep -q "make install_storetype STORE_TYPE=System" $TEST_OUTPUT
  assertFalse "[FAILED] make install_storetype STORE_TYPE=System" $?

  grep -q "installLanguageFiles.pl .*-lang .* pt-BR" $TEST_OUTPUT
  assertTrue "[FAILED] installLanguageFiles.pl .*-lang .* pt-BR" $?

  for i in Site Store StoreenUS StoreptBR ; do
    cmp $TEST_CONFD/StoreTypes.d/6.17.42/$i.conf $TEST_DIR/expected/$i.conf
    assertTrue "[FAILED] cmp $TEST_CONFD/StoreTypes.d/6.17.42/$i.conf $TEST_DIR/expected/$i.conf" $?
  done

  # Monitor has no languages
  test -f $TEST_CONFD/StoreTypes.d/6.17.42/Monitor.conf
  assertFalse "[FAILED] test -f $TEST_CONFD/StoreTypes.d/6.17.42/Monitor.conf" $?
  echo

  # language pack found
  grep -q "LanguagePacks/langpack-pt-BR-epages617p42.tar.gz" $TEST_OUTPUT
  assertTrue "[FAILED] grep -q LanguagePacks/langpack-pt-BR-epages617p42.tar.gz" $?

  # run make install_demoshop if stores include Store but not MultiStore
  grep -q "make install_demoshop" $TEST_OUTPUT
  assertTrue "[FAILED] make install_demoshop" $?

  # no MultiStore
  grep -q "make install_multistore_complete" $TEST_OUTPUT
  assertFalse "[FAILED] make install_multistore_complete" $?

  # no FlexStore
  grep -q "make FlexStore" $TEST_OUTPUT
  assertFalse "[FAILED] make FlexStore" $?

  # no FlexSite
  grep -q "make FlexSite" $TEST_OUTPUT
  assertFalse "[FAILED] make FlexSite" $?
  echo
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
