#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods
##############################################################################

# die if shunit2 not found
type shunit2 && SHUNIT2=$(type -p shunit2) || exit 1

# test script directory (/var/epages/t)
TEST_DIR=$(cd $(dirname $0) && pwd)
TEST_FILE=$(basename $0)
TEST_FILE=${TEST_FILE#pre-}

# script to test
SCRIPT_TO_TEST=${TEST_DIR%/*}/preinstall/${TEST_FILE%-test.sh}.sh

. ${TEST_DIR%/*}/functions/fun-epages.sh
export get_ini_value set_ini_value

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  echo '-> export EPAGES_INSTALLATION_MOCK=1'
  TEMP=/tmp/${0##*/}.~$$~
  TEST_CONFD=$TEMP/eproot/Shared/Config
  INSTALL_EPAGES_CONF=$TEMP/conf/install-epages.conf.sample
  TEST_OUTPUT=$TEMP/test.output
  rm -rf $TEMP/*
  mkdir -m 755 -p ${TEST_CONFD%/*}
  cp -rp ${TEST_DIR%/*}/conf $TEMP
  cp -rp /srv/epages/eproot/Shared/Config $TEST_CONFD || exit 1
}

function oneTimeTearDown() {
  if [[ -n $KEEP_TEST_CONF ]] ; then
    echo "test_configdir: $TEMP"
  else
    rm -rf $TEMP
  fi
}

##############################################################################
# define functions                                                           #
##############################################################################

#-----------------------------------------------------------------------------
# test functions
#-----------------------------------------------------------------------------

function test_script_syntax() {
  # script must have correct syntax
  bash -n $SCRIPT_TO_TEST
  assertTrue "[FAILED] bash -n $SCRIPT_TO_TEST" $? || exit 1
}

function test_script_executable() {
  # script must be executable
  test -x $SCRIPT_TO_TEST
  assertTrue "[FAILED] test -x $SCRIPT_TO_TEST" $?
  chmod 755 $SCRIPT_TO_TEST
  echo
}

function test_no_keys() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD

  echo "-> $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF > $TEST_OUTPUT

  grep -q 'overwrite .*/epages.conf SystemDomainName'  $TEST_OUTPUT
  assertTrue "[FAILED] $INSTALL_EPAGES_CONF: no keys" $?
  echo
}

function test_keys() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD

  local CONFIGFILE=$TEMP/conf/epages.conf
  test -r $CONFIGFILE
  assertTrue "[FAILED] test -r $CONFIGFILE" $?

  # set values in /var/epages/conf/epages.conf
  set_ini_value --overwrite $CONFIGFILE SystemDomainName x.y.z DE_EPAGES::Object
  set_ini_value --overwrite $CONFIGFILE DefaultWebServiceProxy abc DE_EPAGES::Store

  # preinstall/epages.sh copies values to $EPAGES_CONFIG/epages.conf
  echo "-> $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  # get values from $EPAGES_CONFIG/epages.conf
  local FOUND=$(get_ini_value $EPAGES_CONFIG/epages.conf SystemDomainName DE_EPAGES::Object)
  local EXPECTED=x.y.z
  assertEquals "[FAILED] $SCRIPT_TO_TEST: $CONFIGFILE" "$EXPECTED" "$FOUND"
  FOUND=$(get_ini_value $EPAGES_CONFIG/epages.conf DefaultWebServiceProxy DE_EPAGES::Store)
  EXPECTED=abc
  assertEquals "[FAILED] $SCRIPT_TO_TEST: $CONFIGFILE" "$EXPECTED" "$FOUND"
  echo
}

function test_conf() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD

  rm -f $EPAGES_CONFIG/epages.conf
  touch $EPAGES_CONFIG/epages.conf
  mv $TEMP/conf/epages.conf $TEMP/conf/xxx
  sed -ri 's,^(epages-conf)=.*,\1=xxx,' $INSTALL_EPAGES_CONF

  # epages-conf=xxx -> changes in epages.conf
  echo "-> $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  local FOUND=$(get_ini_value $EPAGES_CONFIG/epages.conf DefaultWebServiceProxy DE_EPAGES::Store)
  local EXPECTED=abc
  assertEquals "[FAILED] $SCRIPT_TO_TEST: $CONFIGFILE" "$EXPECTED" "$FOUND"
  echo
}

function test_no_conf() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD

  rm -f $EPAGES_CONFIG/epages.conf
  touch $EPAGES_CONFIG/epages.conf
  sed -ri 's,^epages-conf=.*,,' $INSTALL_EPAGES_CONF

  # no epages-conf= -> no changes in epages.conf
  echo "-> $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  local FOUND=$(get_ini_value $EPAGES_CONFIG/epages.conf DefaultWebServiceProxy DE_EPAGES::Store)
  local EXPECTED=
  assertEquals "[FAILED] $SCRIPT_TO_TEST: $CONFIGFILE" "$EXPECTED" "$FOUND"
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
