#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods
##############################################################################

# die if shunit2 not found
type shunit2 && SHUNIT2=$(type -p shunit2) || exit 1

# test script directory (/var/epages/t)
TEST_DIR=$(cd $(dirname $0) && pwd)
TEST_FILE=$(basename $0)
TEST_FILE=${TEST_FILE#pre-}

# script to test
SCRIPT_TO_TEST=${TEST_DIR%/*}/preinstall/${TEST_FILE%-test.sh}.sh

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  echo '-> export EPAGES_INSTALLATION_MOCK=1'
  TEMP=/tmp/${0##*/}.~$$~
  TEST_CONFD=$TEMP/eproot/Shared/Config
  INSTALL_EPAGES_CONF=$TEMP/conf/install-epages.conf.sample
  TEST_OUTPUT=$TEMP/test.output
  mkdir -p -m 755 $TEMP
  rm -rf $TEMP/*
  cp -rp ${TEST_DIR%/*}/conf $TEMP
}

function oneTimeTearDown() {
  if [[ -n $KEEP_TEST_CONF ]] ; then
    echo "test_configdir: $TEMP"
  else
    rm -rf $TEMP
  fi
}

##############################################################################
# define functions                                                           #
##############################################################################

#-----------------------------------------------------------------------------
# auxiliary functions
#-----------------------------------------------------------------------------

default_store_conf() {
  local STORE=$1
  local StoreType="$STORE "
  [[ $STORE = epagesj ]] && StoreType=
  local Database=$(echo $STORE | tr '[:upper:]' '[:lower:]')db
  case $STORE in
    ASPoolDB|epagesj) Database=$(echo $STORE | tr '[:upper:]' '[:lower:]') ;;
  esac
  local login=usr_$Database
  local passwd=epages
  local active=1
  local Backup=Backup
  local BackupHost=localhost
  [[ $STORE != ASPoolDB ]] || BackupHost=none
  echo $StoreType $Database $login $passwd $Backup $BackupHost $active
}

found_store_conf() {
  local STORE=$1
  local INI=/var/epages/ini-config.sh
  local StoreType=$($INI -f $STORE.conf -s Connection -k StoreType -G)
  local Database=$($INI -f $STORE.conf -s Connection -k Database -G)
  local login=$($INI -f $STORE.conf -s Connection -k login -G)
  local passwd=$($INI -f $STORE.conf -s Connection -k passwd -G)
  local Backup=$($INI -f $STORE.conf -s Connection -k Backup -G)
  local BackupHost=$($INI -f $STORE.conf -s Connection -k BackupHost -G)
  local active=$($INI -f $STORE.conf -s Connection -k active -G)
  echo $StoreType $Database $login $passwd $Backup $BackupHost $active
}

#-----------------------------------------------------------------------------
# test functions
#-----------------------------------------------------------------------------

function test_script_syntax() {
  # script must have correct syntax
  bash -n $SCRIPT_TO_TEST
  assertTrue "[FAILED] bash -n $SCRIPT_TO_TEST" $? || exit 1
}

function test_script_executable() {
  # script must be executable
  test -x $SCRIPT_TO_TEST
  assertTrue "[FAILED] test -x $SCRIPT_TO_TEST" $?
  chmod 755 $SCRIPT_TO_TEST
  echo
}

function test_basic() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  echo "-> export EPAGES_CONFIG=$TEST_CONFD"
  local CMD="$SCRIPT_TO_TEST"
  echo "-> $CMD"

  $SCRIPT_TO_TEST
  assertTrue "[FAILED] $CMD" $?

  pushd $TEST_CONFD/Database.d >/dev/null
  # database server with standard databases
  local EXPECTED="ASPoolDB.conf epagesj.conf Site.conf Store.conf System.conf"
  local FOUND=$(echo *.*)
  assertEquals "[FAILED] ls $TEST_CONFD/Database.d" "$EXPECTED" "$FOUND"

  # check STORE.conf
  for STORE in Site Store System ASPoolDB epagesj ; do
    echo "-> basic: $STORE.conf"
    assertEquals "[FAILED] $STORE.conf" "$(default_store_conf $STORE)" "$(found_store_conf $STORE)"
  done
  echo
}

function test_simple_conf() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  echo "-> export EPAGES_CONFIG=$TEST_CONFD"
  local CMD="$SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  echo "-> $CMD"

  $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF
  assertTrue "[FAILED] $CMD" $?

  pushd $TEST_CONFD/Database.d >/dev/null
  # database server with standard databases
  local EXPECTED="ASPoolDB.conf Backup.conf epagesj.conf PatchSystemInfoDB.conf Site.conf Store.conf System.conf"
  local FOUND=$(echo *.*)
  assertEquals "[FAILED] ls $TEST_CONFD/Database.d" "$EXPECTED" "$FOUND"

  # check STORE.conf
  for STORE in Site Store System ASPoolDB epagesj ; do
    echo "-> simple: $STORE.conf"
    assertEquals "[FAILED] $STORE.conf" "$(default_store_conf $STORE)" "$(found_store_conf $STORE)"
  done

  # check PatchSystemInfoDB.conf
  echo "-> simple: PatchSystemInfoDB.conf"
  EXPECTED="patchsysteminfodb usr_patch epages Backup none 1"
  assertEquals "[FAILED] PatchSystemInfoDB.conf" "$EXPECTED" "$(found_store_conf PatchSystemInfoDB)"

  # check Backup.conf
  echo "-> simple: Backup.conf"
  EXPECTED="mysql root 1"
  assertEquals "[FAILED] Backup.conf" "$EXPECTED" "$(found_store_conf Backup)"
  echo
}

function test_mysqld_conf() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  echo "-> export EPAGES_CONFIG=$TEST_CONFD"

  local CMD="$SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  echo "-> $CMD"

  pushd $TEMP/conf >/dev/null
  sed -ri "s,^(mysqld=).*,\1mysqld/complex.conf," $INSTALL_EPAGES_CONF
  echo "-> mysqld=mysqld/complex.conf"

  rm -f $TEST_CONFD/Database.d/*
  $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF
  assertTrue "[FAILED] $CMD" $?

  pushd $TEST_CONFD/Database.d >/dev/null
  BU_EXPECTED="ASPoolDB Backup epagesj Monitor PatchSystemInfoDB Site Store StoreenUS StoreptBR System"
  BU_EXPECTED=${BU_EXPECTED// /.conf }.conf
  FOUND=$(echo *.*)
  assertEquals "[FAILED] mysqld=mysqld/complex.conf" "$BU_EXPECTED" "$FOUND"

  for STORE in $BU_EXPECTED ; do
    STORE=${STORE%.conf}
    MSG="complex: $STORE.conf"
    echo "-> $MSG"
    case $STORE in
      Site|Store|StoreenUS|StoreptBR|System|ASPoolDB|epagesj|Monitor)
	assertEquals "[FAILED] $MSG" "$(default_store_conf $STORE)" "$(found_store_conf $STORE)"
	;;
      Backup)
	EXPECTED="mysql root 1"
	assertEquals "[FAILED] $MSG" "$EXPECTED" "$(found_store_conf $STORE)"
	;;
      PatchSystemInfoDB)
	EXPECTED="patchsysteminfodb usr_patch epages Backup none 1"
	assertEquals "[FAILED] $MSG" "$EXPECTED" "$(found_store_conf $STORE)"
	;;
    esac
    if [[ $STORE = Site || $STORE = Store ]] ; then
      EXPECTED=1234567890abcdef
      FOUND=$(/var/epages/ini-config.sh -f $STORE.conf -s Connection -k Key -G)
      assertEquals "[FAILED] $MSG: Key" "$EXPECTED" "$FOUND"
    fi
  done
  echo
}

function test_no_backup_conf() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  echo "-> export EPAGES_CONFIG=$TEST_CONFD"

  local CMD="$SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  echo "-> $CMD"

  pushd $TEMP/conf >/dev/null
  echo -e "[Store]\nStoreType=Store\nBackup=Backup" > nobackup.conf

  sed -ri "s,^(mysqld=).*,\1nobackup.conf," $INSTALL_EPAGES_CONF
  echo "-> mysqld=nobackup.conf"

  rm -f $TEST_CONFD/Database.d/*
  $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF
  assertTrue "[FAILED] $CMD" $?

  BU_EXPECTED="Store.conf"
  pushd $TEST_CONFD/Database.d >/dev/null
  FOUND=$(echo *.*)
  assertEquals "[FAILED] mysqld=nobackup.conf" "$BU_EXPECTED" "$FOUND"
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
