#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods
##############################################################################

# die if shunit2 not found
type shunit2 && SHUNIT2=$(type -p shunit2) || exit 1

# test script directory (/var/epages/t)
TEST_DIR=$(cd $(dirname $0) && pwd)
TEST_FILE=$(basename $0)
TEST_FILE=${TEST_FILE#pre-}

# script to test
SCRIPT_TO_TEST=${TEST_DIR%/*}/preinstall/${TEST_FILE%-test.sh}.pl

. ${TEST_DIR%/*}/functions/fun-epages.sh
export get_ini_value set_ini_value

echo -e "testing: $0\nscript to test: $SCRIPT_TO_TEST\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  echo '-> export EPAGES_INSTALLATION_MOCK=1'
  TEMP=/tmp/${0##*/}.~$$~
  TEST_CONFD=$TEMP/eproot/Shared/Config
  INSTALL_EPAGES_CONF=$TEMP/conf/install-epages.conf.sample
  TEST_OUTPUT=$TEMP/test.output
  rm -rf $TEMP/*
  mkdir -m 755 -p ${TEST_CONFD%/*} $TEMP/eproot/Perl/bin $TEMP/etc/default
  cp -rp ${TEST_DIR%/*}/conf $TEMP

  # use Perl/bin/perl with changed EPAGES_CONFIG
  cp -p /srv/epages/eproot/Perl/bin/perl $TEMP/eproot/Perl/bin || exit 1
  /var/epages/ini-config.sh -f /etc/default/epages6 -k EPAGES_CONFIG -v $TEST_CONFD > $TEMP/etc/default/epages6
  /var/epages/ini-config.sh -f $TEMP/etc/default/epages6 -k PERL -v $TEMP/eproot/Perl/bin/perl -i
  chmod 644 $TEMP/etc/default/epages6
  cp -rp /srv/epages/eproot/Shared/Config $TEST_CONFD || exit 1
  rm -rf $TEST_CONFD/Database.d/* || exit 1
}

function oneTimeTearDown() {
  if [[ -n $KEEP_TEST_CONF ]] ; then
    echo "test_configdir: $TEMP"
  else
    rm -rf $TEMP
  fi
}

##############################################################################
# define functions                                                           #
##############################################################################

#-----------------------------------------------------------------------------
# test functions
#-----------------------------------------------------------------------------

function test_script_syntax() {
  # script must have correct syntax
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  echo "-> $PERL -cw $SCRIPT_TO_TEST"
  $PERL -cw $SCRIPT_TO_TEST
  assertTrue "[FAILED] $PERL -cw $SCRIPT_TO_TEST" $? || exit 1
}

function test_script_executable() {
  # script must be executable
  test -x $SCRIPT_TO_TEST
  assertTrue "[FAILED] test -x $SCRIPT_TO_TEST" $?
  chmod 755 $SCRIPT_TO_TEST
  echo
}

function test_basic() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  echo "-> $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  local EXPECTED=$TEST_DIR/expected/ServerConfig-test_basic.xml
  local FOUND=$TEST_CONFD/ServerConfig.xml
  # normalize IPs
  sed -ri "s,host='[0-9.]+',host='',g" $FOUND

  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?

  EXPECTED=$TEST_DIR/expected/WebInterface-test_basic.conf
  FOUND=$TEST_CONFD/WebInterface.conf
  # normalize IPs/hostnames
  sed -ri 's,(ExceptionHost=).*,\1,' $FOUND

  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?
  echo
}

function test_legacy() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  # remove generateServerConfig.pl temporarily to check legacy
  local GENERATE_SERVER_CONFIG=DE_EPAGES/WebInterface/Scripts/generateServerConfig.pl
  local EPAGES_CARTRIDGES=${EPAGES_CARTRIDGES:-/srv/epages/eproot/Cartridges}
  local MOVED
  if [[ -f $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG ]] ; then
    mv $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG.test
    MOVED=1
  fi

  echo "-> $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  if [[ -n $MOVED ]] ; then
    mv $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG.test $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG
  fi

  local EXPECTED=$TEST_DIR/expected/WebInterface-test_legacy.conf
  local FOUND=$TEST_CONFD/WebInterface.conf
  # normalize IPs/hostnames
  sed -ri 's,(ExceptionHost|DEDICATED)=.*,\1=,' $FOUND
  sed -ri 's,\[[:0-9][:0-9f.*]+\],\[\],' $FOUND

  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?
  echo
}

function test_ip() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  set_ini_value --overwrite $INSTALL_EPAGES_CONF ip 0.0.0.0 localhost

  echo "-> $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  local EXPECTED=$TEST_DIR/expected/ServerConfig-test_ip.xml
  local FOUND=$TEST_CONFD/ServerConfig.xml
  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?

  EXPECTED=$TEST_DIR/expected/WebInterface-test_ip.conf
  FOUND=$TEST_CONFD/WebInterface.conf
  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?
  echo
}

function test_legacy_ip() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  # remove generateServerConfig.pl temporarily to check legacy
  local GENERATE_SERVER_CONFIG=DE_EPAGES/WebInterface/Scripts/generateServerConfig.pl
  local EPAGES_CARTRIDGES=${EPAGES_CARTRIDGES:-/srv/epages/eproot/Cartridges}
  local MOVED
  if [[ -f $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG ]] ; then
    mv $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG.test
    MOVED=1
  fi

  echo "-> $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  if [[ -n $MOVED ]] ; then
    mv $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG.test $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG
  fi

  local EXPECTED=$TEST_DIR/expected/WebInterface-test_legacy_ip.conf
  local FOUND=$TEST_CONFD/WebInterface.conf

  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?
  echo
}

function test_hostname() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  set_ini_value --overwrite $INSTALL_EPAGES_CONF hostname foobar.kwats localhost

  echo "-> $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  local EXPECTED=$TEST_DIR/expected/ServerConfig-test_ip.xml
  local FOUND=$TEST_CONFD/ServerConfig.xml
  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?

  EXPECTED=$TEST_DIR/expected/WebInterface-test_hostname.conf
  FOUND=$TEST_CONFD/WebInterface.conf
  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?
  echo
}

function test_legacy_hostname() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  # remove generateServerConfig.pl temporarily to check legacy
  local GENERATE_SERVER_CONFIG=DE_EPAGES/WebInterface/Scripts/generateServerConfig.pl
  local EPAGES_CARTRIDGES=${EPAGES_CARTRIDGES:-/srv/epages/eproot/Cartridges}
  local MOVED
  if [[ -f $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG ]] ; then
    mv $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG.test
    MOVED=1
  fi

  echo "-> $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  if [[ -n $MOVED ]] ; then
    mv $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG.test $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG
  fi

  local EXPECTED=$TEST_DIR/expected/WebInterface-test_legacy_hostname.conf
  local FOUND=$TEST_CONFD/WebInterface.conf

  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?
  echo
}

function test_distributed() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  sed -n '/\[localhost\]/,$p' $INSTALL_EPAGES_CONF | sed -r 's,(localhost),\1.localdomain,' >> $INSTALL_EPAGES_CONF
  set_ini_value --overwrite $INSTALL_EPAGES_CONF hostname kwats.foobaz localhost.localdomain
  set_ini_value --overwrite $INSTALL_EPAGES_CONF ip 255.255.255.255 localhost.localdomain

  echo "-> $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  local EXPECTED=$TEST_DIR/expected/ServerConfig-test_distributed.xml
  local FOUND=$TEST_CONFD/ServerConfig.xml
  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?

  EXPECTED=$TEST_DIR/expected/WebInterface-test_distributed.conf
  FOUND=$TEST_CONFD/WebInterface.conf
  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?
  echo
}

function test_legacy_distributed() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  # remove generateServerConfig.pl temporarily to check legacy
  local GENERATE_SERVER_CONFIG=DE_EPAGES/WebInterface/Scripts/generateServerConfig.pl
  local EPAGES_CARTRIDGES=${EPAGES_CARTRIDGES:-/srv/epages/eproot/Cartridges}
  local MOVED
  if [[ -f $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG ]] ; then
    mv $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG.test
    MOVED=1
  fi

  echo "-> $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  if [[ -n $MOVED ]] ; then
    mv $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG.test $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG
  fi

  local EXPECTED=$TEST_DIR/expected/WebInterface-test_legacy_distributed.conf
  local FOUND=$TEST_CONFD/WebInterface.conf

  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?
  echo
}

function test_customized() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  sed -r 's,^;(MAX|Ex),\1,;s,(requestrouters)=.*,\1=RequestRouter1,' \
    $TEMP/conf/appsrv/localhost.conf > $TEMP/conf/appsrv/localhost.localdomain.conf
  cp -p $TEMP/conf/ascsrv/localhost.conf $TEMP/conf/ascsrv/localhost.localdomain.conf

  echo "-> $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  local EXPECTED=$TEST_DIR/expected/ServerConfig-test_customized.xml
  local FOUND=$TEST_CONFD/ServerConfig.xml
  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?

  EXPECTED=$TEST_DIR/expected/WebInterface-test_customized.conf
  FOUND=$TEST_CONFD/WebInterface.conf
  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?
  echo
}

function test_legacy_customized() {
  export EPAGES_INSTALLATION_MOCK=1
  export EPAGES_CONFIG=$TEST_CONFD
  export PERL=$TEMP/eproot/Perl/bin/perl

  # remove generateServerConfig.pl temporarily to check legacy
  local GENERATE_SERVER_CONFIG=DE_EPAGES/WebInterface/Scripts/generateServerConfig.pl
  local EPAGES_CARTRIDGES=${EPAGES_CARTRIDGES:-/srv/epages/eproot/Cartridges}
  local MOVED
  if [[ -f $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG ]] ; then
    mv $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG.test
    MOVED=1
  fi

  echo "-> $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF"
  $PERL $SCRIPT_TO_TEST $INSTALL_EPAGES_CONF

  if [[ -n $MOVED ]] ; then
    mv $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG.test $EPAGES_CARTRIDGES/$GENERATE_SERVER_CONFIG
  fi

  local EXPECTED=$TEST_DIR/expected/WebInterface-test_legacy_customized.conf
  local FOUND=$TEST_CONFD/WebInterface.conf

  echo "-> cmp $EXPECTED $FOUND"
  cmp $EXPECTED $FOUND
  assertTrue "[FAILED] cmp $EXPECTED $FOUND" $?
  echo
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
