#!/bin/bash

##############################################################################
# this script runs tests with shunit2 methods                                #
##############################################################################

# install if shunit2 not found
PATH=$PATH:/usr/share/shunit2
if ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/debian_version ]] ; then
  apt-get install shunit2
elif ! type shunit2 >/dev/null 2>&1 && [[ -f /etc/redhat-release ]] ; then
  rpm -hiv http://download.fedoraproject.org/pub/epel/6/i386/shunit2-2.1.6-3.el6.noarch.rpm
fi

# die if shunit2 not found
SHUNIT2=$(type -p shunit2) || exit 1

echo -e "testing: $0\n"

##############################################################################
# re-define functions                                                        #
##############################################################################

function oneTimeSetUp() {
  export EPAGES_REPO=${EPAGES_REPO:-ftp://ftp.epages.com/pub/epages/build/repo/epages}
  [[ $EPAGES_REPO =~ ftp:// ]]
  assertTrue "[FAILED] EPAGES_REPO must start with ftp:// $EPAGES_REPO" $? || exit 1
  if [[ -f /etc/redhat-release ]] ; then
    echo "rpm -U $EPAGES_REPO/yum/epages-release.noarch.rpm"
    rpm -U $EPAGES_REPO/yum/epages-release.noarch.rpm || true
  else
    assertTrue "[FAILED] file missing: /etc/redhat-release" 1 || exit 1
  fi

  . /var/epages/functions/fun-epages.sh
  export start_epages_if_stopped start_epagesj_if_stopped
  start_epages_if_stopped
  start_epagesj_if_stopped

  echo "/var/epages/SetRepository.sh -r $EPAGES_REPO"
  /var/epages/SetRepository.sh -r $EPAGES_REPO >/dev/null
  assertTrue "[FAILED] /var/epages/SetRepository.sh -r $EPAGES_REPO" $? || exit 1
}

##############################################################################
# auxiliary functions                                                        #
##############################################################################

function package_location() {
  local PACKAGE=$1
  case $PACKAGE in
    epages-webconf) SECTION=epages-base ;;
    epages-patch) SECTION=epages-update ;;
    *) SECTION=epages-setup ;;
  esac
  local REPO=/etc/yum.repos.d/$SECTION.repo
  local BASEURL=$(/var/epages/ini-config.sh -f $REPO -s $SECTION -k baseurl -G)
  local FILE=$(curl -s $BASEURL/ | awk "/$PACKAGE-/{print \$9}")
  echo $BASEURL/$FILE
}

function update_package() {
  local PACKAGE=$1
  local PROCESS=$2
  export EPAGES_SPEC_SKIP_RESTART=$3
  export EPAGES_SPEC_KEEP_CONFIG=$4

  if [[ $PACKAGE = patch ]] ; then
    rm -rf /tmp/epages
    unset EPAGES_DO_PATCH
    [[ -z $EPAGES_SPEC_SKIP_RESTART ]] && export EPAGES_DO_PATCH=0
  fi

  local PACKAGE_LOCATION=$(package_location epages-$PACKAGE)

  local PIDS_BEFORE=$(pgrep $PROCESS | tr '\n' ' ')
  local CMD="rpm -U --force --nodeps $PACKAGE_LOCATION"
  echo $CMD
  rpm -U --force --nodeps $PACKAGE_LOCATION >/dev/null
  assertTrue "[FAILED] $CMD" $?
  local PIDS_AFTER=$(pgrep $PROCESS | tr '\n' ' ')

  if [[ $PACKAGE = patch && -n $EPAGES_SPEC_SKIP_RESTART ]] ; then
    ls /tmp/epages/epages-patch_*/install.log >/dev/null 2>&1
    assertFalse "[FAILED] ls /tmp/epages/epages-patch_*/install.log" $?
  elif [[ $PACKAGE = patch ]] ; then
    ls /tmp/epages/epages-patch_*/install.log >/dev/null 2>&1
    assertTrue "[FAILED] ls /tmp/epages/epages-patch_*/install.log" $?
  elif [[ -n $EPAGES_SPEC_SKIP_RESTART ]] ; then
    assertEquals "[FAILED] PIDs: $PIDS_BEFORE -> $PIDS_AFTER" "$PIDS_BEFORE" "$PIDS_AFTER"
  else
    assertNotEquals "[FAILED] PIDs: $PIDS_BEFORE -> $PIDS_AFTER" "$PIDS_BEFORE" "$PIDS_AFTER"
  fi
}

##############################################################################
# test functions                                                             #
##############################################################################

function test_update_j_obey_spec() {
  find /srv/epages/eproot/j/{solr/,}conf -type f | sort | while read i ; do
    echo >> $i
  done

  update_package j java

  pushd /srv/epages/eproot/j/rpmsave >/dev/null || exit 1
  local DIFF=$(find * -type f | while read file ; do
    diff -q $file ../$file
  done)
  popd >/dev/null

  for i in conf/epagesj-xhtmlrenderer.conf conf/epagesj.conf conf/logback-epagesj.xml \
           conf/logback-solr.xml solr/conf/solrconfig.xml ; do
    [[ $DIFF =~ $i ]]
    assertTrue "[FAILED] update file: $i" $?
  done
}

function test_update_j_skip_spec() {
  find /srv/epages/eproot/j/{solr/,}conf -type f | sort | while read i ; do
    echo >> $i
  done

  update_package j java skip_restart keep_config

  pushd /srv/epages/eproot/j/rpmsave >/dev/null || exit 1
  local DIFF=$(find * -type f | while read file ; do
    diff -q $file ../$file
  done)
  popd >/dev/null

  for i in conf/epagesj-xhtmlrenderer.conf conf/epagesj.conf conf/logback-epagesj.xml \
           conf/logback-solr.xml solr/conf/solrconfig.xml ; do
    [[ $DIFF =~ $i ]]
    assertFalse "[FAILED] keep file: $i" $?
  done
}

#--------------------------------------------------------------------------

function test_update_reqsrv_obey_spec() {
  update_package reqsrv RequestRouter
}

function test_update_reqsrv_skip_spec() {
  update_package reqsrv RequestRouter skip_restart
}

function test_update_ascsrv_obey_spec() {
  update_package ascsrv ASPoolDBCacheSe
}

function test_update_ascsrv_skip_spec() {
  update_package ascsrv ASPoolDBCacheSe skip_restart
}

function test_update_appsrv_obey_spec() {
  update_package appsrv epagesService
}

function test_update_appsrv_skip_spec() {
  update_package appsrv epagesService skip_restart
}

function test_update_mysqld_obey_spec() {
  # mysqld is not restarted if already running
  return
}

function test_update_mysqld_skip_spec() {
  update_package mysqld mysqld skip_restart
}

function test_update_webconf_obey_spec() {
  # httpd is restarted graceful -> processes remain
  return
}

function test_update_webconf_skip_spec() {
  update_package webconf httpd skip_restart
}

function test_update_perl_obey_spec() {
  update_package perl epagesService
}

function test_update_perl_skip_spec() {
  update_package perl epagesService skip_restart
}

function test_update_patch_obey_spec() {
  update_package patch epagesService
}

function test_update_patch_skip_spec() {
  update_package patch epagesService skip_restart
}

##############################################################################
# source shunit2                                                             #
##############################################################################

. $SHUNIT2
