#!/bin/bash

#==============================================================================
# update-epages.sh
#==============================================================================

if [ ! -w /etc/passwd ] ; then
  echo "$0: only 'root' can execute this program"
  exit 1
fi

if [ -z "$BASH_VERSION" ] ; then
  echo "$0: this script can only be executed by bash"
  exit 1
fi

LANG=C # use base language for standard output of some commands
LC_ALL=C

#------------------------------------------------------------------------------
# arguments
#------------------------------------------------------------------------------

usage() {
  [[ -n $1 ]] && echo -e "Error:\n  $@"
  echo "Usage:"
  echo "  $0 {--singlehost|--distributed} [--version V] [--repo R]"
  echo "Options:"
  echo "  --version : target version for ePages patch (default: latest version)"
  echo "  --repo    : ePages repository (default: http://epages.com/repo/epages)"
  echo "  --singlehost : unpack and install patch on single host installation"
  echo "  --distributed : just unpack patch on distributed ePages installation"
  echo "Examples:"
  echo "  $0 --singlehost"
  echo "  $0 --singlehost --version 6.17.34 --repo ftp://epages-software.de/repo/epages"
  echo "  $0 --distributed --version 6.17.34"
  exit 1
}

TEMP=$(getopt -o '' --long singlehost,distributed,version:,repo: -n $0 -- "$@") || usage
eval set -- "$TEMP"

unset VERSION REPO SINGLEHOST DISTRIBUTED BASEARGS
while true ; do
  case "$1" in
    --singlehost) BASEARGS="$BASEARGS $1"; SINGLEHOST=1; export EPAGES_DO_PATCH=1; shift ;;
    --distributed) BASEARGS="$BASEARGS $1"; DISTRIBUTED=1; export EPAGES_DO_PATCH=0; shift ;;
    --version) [[ -z $2 ]] && usage ; VERSION=$2 ; shift 2 ;;
    --repo) [[ -z $2 ]] && usage ; BASEARGS="$BASEARGS $1 $2"; REPO=$2 ; shift 2 ;;
    --) shift ; break ;;
    *) echo $@ ; usage ;;
  esac
done
# no other arguments allowed
[[ -n $1 ]] && usage

if [[ ( -n $SINGLEHOST && -n $DISTRIBUTED ) || -z "$SINGLEHOST$DISTRIBUTED" ]] ; then
  usage "$0: choose either --singlehost or --distributed or --help"
fi

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

#-----------------------------------------------------------------------
# find OS
#-----------------------------------------------------------------------

unset OS
if [[ -f /etc/debian_version ]] ; then
  ECHO $LINENO "Debian found: /etc/debian_version -> $(cat /etc/debian_version)"
  OS=debian
elif [[ -f /etc/redhat-release ]] ; then
  ECHO $LINENO "Redhat found: /etc/redhat-release -> $(cat /etc/redhat-release)"
  OS=redhat
else
  echo "$0: operating system not supported: $(uname -a)"
  exit 1
fi

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. /etc/default/epages6
export get_cartridge_fullversion pkg_installed test_version

. $(dirname $0)/functions/fun-repoedit.sh
export next_version_in_upgrade_path

INSTALL_PACKAGE=/var/epages/install-RPM.sh

#-----------------------------------------------------------------------
# find_update_version
#-----------------------------------------------------------------------

find_update_version() {
  local TARGET_VERSION=$VERSION
  [[ $TARGET_VERSION = yum ]] && TARGET_VERSION=

  # get version from $VERSION
  if [[ -n $TARGET_VERSION && -n ${VERSION//[0-9.]/} ]] ; then
    echo "$0: void format in --version $VERSION (must be dot-sep. numbers only)"
    exit 1
  fi
  [[ -n $TARGET_VERSION ]] && { echo $TARGET_VERSION; return; }

  # get version from $REPO
  if [[ -n $REPO ]] ; then
    TARGET_VERSION=$(basename $(dirname $REPO))
    [[ -z ${TARGET_VERSION//[0-9.]/} ]] && { echo $TARGET_VERSION; return; }
    TARGET_VERSION=
  fi

  # get latest published version
  local DOWNLOAD=ftp://epages-software.de/repo/epages/yum/x86_64/
  TARGET_VERSION=$(curl -sLk $DOWNLOAD | awk '/epages-common-/{print $9}')
  if [[ $TARGET_VERSION = ${TARGET_VERSION#epages-common-} ]] ; then
    echo "$0: no latest published version in $DOWNLOAD found"
    exit 1
  fi
  TARGET_VERSION=${TARGET_VERSION#epages-common-}
  TARGET_VERSION=${TARGET_VERSION%%-*}
  if [[ -n ${TARGET_VERSION//[0-9.]/} ]] ; then
    echo "$0: void format in latest published version from $DOWNLOAD"
    exit 1
  fi
  if test_version $TARGET_VERSION -lt 6.17.5 ; then
    echo "$0: target version is $TARGET_VERSION, must be at least 6.17.5"
    exit 1
  fi
  echo $TARGET_VERSION
}

#-----------------------------------------------------------------------
# test installed version
#-----------------------------------------------------------------------

INSTALLED_VERSION=$(get_cartridge_fullversion Core)
INSTALLED_VERSION=${INSTALLED_VERSION//[^0-9.]/}

if [[ -z $INSTALLED_VERSION ]] ; then
  echo "$0: installed version not found, exiting."
  exit 1
fi

TARGET_VERSION=$(find_update_version)
NEXT_VERSION=$(next_version_in_upgrade_path $INSTALLED_VERSION $TARGET_VERSION) || exit 1
if [[ -n $NEXT_VERSION ]] ; then
  echo
  echo "  Cannot patch from $INSTALLED_VERSION to $TARGET_VERSION directly."
  echo "  First patch to $NEXT_VERSION by running:"
  echo "    $0$BASEARGS --version $NEXT_VERSION"
  echo "  then continue."
  echo
  exit 1
fi

#-----------------------------------------------------------------------
# update epages-release
#-----------------------------------------------------------------------

REPO_URL=${REPO:-http://epages.com/repo/epages}

unset INSTALLED_RELEASE NEW_RELEASE
if [[ $OS = debian ]] ; then
  ECHO $LINENO curl -sLkO $REPO_URL/apt/epages-release_all.deb
  curl -sLkO $REPO_URL/apt/epages-release_all.deb
  NEW_RELEASE=$(dpkg-deb -f epages-release_all.deb Version) || true
  INSTALLED_RELEASE=$(dpkg-query --show -f '${Version}\n' epages-release 2>/dev/null) || true
  if dpkg --compare-versions ${INSTALLED_RELEASE:-0} lt ${NEW_RELEASE:-0.1} ; then
    ECHO $LINENO dpkg -i epages-release_all.deb
    dpkg -i epages-release_all.deb
  fi
else
  NEW_RELEASE=$(rpm --quiet -qp $REPO_URL/yum/epages-release.noarch.rpm --qf '[%{VERSION}-%{RELEASE}\n]') || true
  INSTALLED_RELEASE=$(rpm --quiet -q epages-release --qf '[%{VERSION}-%{RELEASE}\n]' 2>/dev/null) || true
  if [[ $(echo -e "${NEW_RELEASE:-0}\n$INSTALLED_RELEASE" | sort -V | tail -1) != $INSTALLED_RELEASE ]] ; then
    ECHO $LINENO rpm -hUv $REPO_URL/yum/epages-release.noarch.rpm
    rpm -hUv $REPO_URL/yum/epages-release.noarch.rpm || true
  fi
fi

if [[ -n "$REPO$VERSION" ]] ; then
  VERSION_ARG=${VERSION:+-b $VERSION}
  REPO_ARG=${REPO:+-r $REPO_URL}
  rm -f /etc/yum/vars/EPAGES_VERSION
  ECHO $LINENO /var/epages/SetRepository.sh $VERSION_ARG $REPO_ARG
  /var/epages/SetRepository.sh $VERSION_ARG $REPO_ARG
elif [[ $OS = debian ]] && dpkg --compare-versions ${INSTALLED_RELEASE:-0} lt ${NEW_RELEASE:-0} ; then
  ECHO $LINENO apt-get -q -q -y update
  apt-get -q -q -y update
fi

#-----------------------------------------------------------------------
# update or install packages from epages-setup
#-----------------------------------------------------------------------

ECHO $LINENO ENABLE_REPO=epages-setup $INSTALL_PACKAGE --update epages-perl epages-nodejs
ENABLE_REPO=epages-setup $INSTALL_PACKAGE --update epages-perl epages-nodejs || exit 1

INSTALLED_REQSRV=$(pkg_installed -V epages-reqsrv)
ECHO $LINENO ENABLE_REPO=epages-setup $INSTALL_PACKAGE --update-only epages-reqsrv epages-ascsrv
EPAGES_IGNORE=cron ENABLE_REPO=epages-setup $INSTALL_PACKAGE --update-only epages-reqsrv epages-ascsrv || exit 1

INSTALLED_RELEASE=$(pkg_installed -V epages-release)

if test_version ${INSTALLED_REQSRV:-99} -lt 6.17.5 ; then
  # for versions < 6.17.5 epages-reqsrv uninstall stops application/RR servers
  ECHO $LINENO EPAGES_IGNORE=cron /etc/init.d/epages6 start_service
  EPAGES_IGNORE=cron /etc/init.d/epages6 start_service
  ECHO $LINENO EPAGES_IGNORE=cron /etc/init.d/epages6 start_rr
  EPAGES_IGNORE=cron /etc/init.d/epages6 start_rr
elif test_version ${INSTALLED_RELEASE:-99} -le 6.17.40 ; then
  # for versions <= 6.17.40 epages-reqsrv uninstall stops RR servers
  ECHO $LINENO EPAGES_IGNORE=cron /etc/init.d/epages6 start_rr
  EPAGES_IGNORE=cron /etc/init.d/epages6 start_rr
fi

#-----------------------------------------------------------------------
# update packages from epages-base
#-----------------------------------------------------------------------

ECHO $LINENO ENABLE_REPO=epages-base $INSTALL_PACKAGE --update-only epages-requires epages-webconf
ENABLE_REPO=epages-base $INSTALL_PACKAGE --update-only epages-requires epages-webconf || exit 1

#-----------------------------------------------------------------------
# update packages from epages-update
#-----------------------------------------------------------------------

if [[ $OS = debian ]] ; then
  ECHO $LINENO apt-get -y --only-upgrade install 'epages-[^p]'
  apt-get -y --only-upgrade install 'epages-[^p]' || exit 1
else
  ECHO $LINENO yum -y update 'epages-[a-oq-z]*' --disablerepo epages-setup --enablerepo epages-update
  yum -y update 'epages-[a-oq-z]*' --disablerepo epages-setup --enablerepo epages-update || exit 1
fi

#-----------------------------------------------------------------------
# update patch package
#-----------------------------------------------------------------------

rm -f $(mktempdir epages-patch)/post.*

# EPAGES_REPO is used by patch API
[[ -n $REPO ]] && export EPAGES_REPO=$REPO

if [[ $OS = debian ]] ; then
  ECHO $LINENO EPAGES_DO_PATCH=$EPAGES_DO_PATCH apt-get -y install epages-patch
  apt-get -y install epages-patch || exit 1
else
  ECHO $LINENO EPAGES_DO_PATCH=$EPAGES_DO_PATCH yum -y update epages-patch --disablerepo epages-setup --enablerepo epages-update
  yum -y update epages-patch --disablerepo epages-setup --enablerepo epages-update || exit 1
fi

[[ -f $(mktempdir epages-patch)/post.failure ]] && exit 1

#-----------------------------------------------------------------------
# install epages-ascsrv for INSTALLED_VERSION < 6.17.5
#-----------------------------------------------------------------------

if [[ -n $INSTALLED_REQSRV ]] && test_version $INSTALLED_VERSION -lt 6.17.5 ; then
  ECHO $LINENO ENABLE_REPO=epages-setup $INSTALL_PACKAGE --update epages-ascsrv
  EPAGES_IGNORE=cron ENABLE_REPO=epages-setup $INSTALL_PACKAGE --update epages-ascsrv || exit 1
fi

if [[ -n $DISTRIBUTED ]] ; then
  echo
  echo "Now go on with the steps explained in:"
  echo "http://helpcenter.epages.com/Doc/current/epages/Manual/en/ePages_Patch_Installation.html#Patch_a_Distributed_ePages_Installation"
  echo
fi

#-----------------------------------------------------------------------
# exit
#-----------------------------------------------------------------------

exit 0
